unit ContextM;

interface

uses
  Windows, ActiveX, ComObj, ShlObj, Dialogs, Classes;


type
  TContextMenu = class(TComObject, IShellExtInit, IContextMenu, IPersistFile, IDropTarget)
  private
    files : cardinal;
    FFileName: array[0..MAX_PATH] of Char;
    fileset : TStringList;
  protected
    { IShellExtInit }
    function IShellExtInit.Initialize = SEIInitialize; // Avoid compiler warning
    function SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HResult; stdcall;
    { IContextMenu }
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
      uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;

    { IPersistFile }
        function GetClassID(out classID: TCLSID): HResult; stdcall;
   function IsDirty: HResult; stdcall;
    function Load(pszFileName: POleStr; dwMode: Longint): HResult;
      stdcall;
    function Save(pszFileName: POleStr; fRemember: BOOL): HResult;
      stdcall;
    function SaveCompleted(pszFileName: POleStr): HResult;
      stdcall;
    function GetCurFile(out pszFileName: POleStr): HResult;
      stdcall;
    { IDropTarget }
    function DragEnter(const dataObj: IDataObject; grfKeyState: Longint;
      pt: TPoint; var dwEffect: Longint): HResult; stdcall;
    function DragOver(grfKeyState: Longint; pt: TPoint;
      var dwEffect: Longint): HResult; stdcall;
    function DragLeave: HResult; stdcall;
    function Drop(const dataObj: IDataObject; grfKeyState: Longint; pt: TPoint;
      var dwEffect: Longint): HResult; stdcall;

      //
      //

  end;

const
  Class_ContextMenu: TGUID = '{EBDF1F20-C829-11D1-8233-0020AF3E97A9}';

implementation

uses ComServ, SysUtils, ShellApi, Registry;


//
// passed filenames when right-clicked
//


function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  StgMedium: TStgMedium;
  FormatEtc: TFormatEtc;
  i : cardinal;
begin
  // Fail the call if lpdobj is Nil.
  if (lpdobj = nil) then begin
    Result := E_INVALIDARG;
    Exit;
  end;
  //.ShowMessage('spot1');

  with FormatEtc do begin
    cfFormat := CF_HDROP;
    ptd      := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex   := -1;
    tymed    := TYMED_HGLOBAL;
  end;
  //ShowMessage('spot2');

  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := lpdobj.GetData(FormatEtc, StgMedium);
  if Failed(Result) then
    Exit;
  //ShowMessage('spot3');

  fileset := TStringList.Create;
  // copy the name of every dropped file
  files := DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0);
  if ( files > 1) then begin
    //ShowMessage('spot4');

    for i := 0 to FILES - 1 do begin
        DragQueryFile(StgMedium.hGlobal, i, FFileName, SizeOf(FFileName));
        fileset.Add(FFilename);
    end;
    Result := NOERROR;
  end
  else begin
    FFileName[0] := #0;
    Result := E_FAIL;
  end;
  ReleaseStgMedium(StgMedium);
end;


//
// Add our menu command to the context menu
//
function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
          idCmdLast, uFlags: UINT): HResult;
begin
  Result := 0; // or use MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 0);

  if ((uFlags and $0000000F) = CMF_NORMAL) or
     ((uFlags and CMF_EXPLORE) <> 0) then begin
    // Add one menu item to context menu
    InsertMenu(Menu, indexMenu, MF_STRING or MF_BYPOSITION, idCmdFirst,
      'Rename Master');

    // Return number of menu items added
    Result := 1; // or use MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 1)
  end;
end;

//
// user clicked command
//
function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
resourcestring
  sPathError = 'Error setting current directory';

var
  H: THandle;

  tf : TextFile;
  i : integer;
  DLL : array[0..MAX_PATH] of char;
  path : string;
begin
  ShowMessage('hi');
  
  Result := E_FAIL;
  // Make sure we are not being called by an application
  if (HiWord(Integer(lpici.lpVerb)) <> 0) then
  begin
    Exit;
  end;

  // Make sure we aren't being passed an invalid argument number
  if (LoWord(lpici.lpVerb) <> 0) then begin
    Result := E_INVALIDARG;
    Exit;
  end;

  try
    GetModuleFileName(hInstance,dll, sizeof(dll));

    path := IncludeTrailingPathDelimiter(ExtractFilePath(dll));
    assign(tf, path +  'filedrop.dat');
    rewrite(tf);
    for i := 0 to fileset.count - 1 do begin
        writeln(tf, (fileset[i]));
    end;
    close(tf);

    h := WinExec(PChar(path + 'RenameMaster.exe filedrop.dat'), lpici.nShow);
    if h <= 31 then begin
        assign(tf, path + 'filedroperr.dat');
        rewrite(tf);
        writeln(tf, IntToStr(h) + ':' + IntToStr(GetLastError) + SysErrorMessage(GetLastError));
        close(tf);
    end;
    Result := NOERROR;
  finally
    
  end;
end;



//
// Don't know when or if this is ever used
//




function TContextMenu.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HRESULT;
begin
  if (idCmd = 0) then begin
    if (uType = GCS_HELPTEXT) then
      // return help string for menu item
      StrCopy(pszName, 'Open selected with Rename Master');
    Result := NOERROR;
  end
  else
    Result := E_INVALIDARG;
end;


type
  TContextMenuFactory = class(TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

procedure TContextMenuFactory.UpdateRegistry(Register: Boolean);
    function GetPath(CLSIDL : word) : string;
    var p : PItemIDList;
        Path: array[0..MAX_PATH] of char;
    begin
        SHGetSpecialFolderLocation(0 , CLSIDL, p);
        SHGetPathFromIDList(p, path);
        result := path;
    end;


var
  ClassID: string;
  TestID: string;
  path : string;
  tf : TextFile;
  dll : array[0..MAX_PATH] of char;
begin
  if Register then begin
    inherited UpdateRegistry(Register);

    GetModuleFileName(hInstance,dll, sizeof(dll));

    path := IncludeTrailingPathDelimiter(ExtractFilePath(dll));
    ClassID := GUIDToString(Class_ContextMenu);

    //CreateRegKey('DelphiProject\shellex', '', '');
    //CreateRegKey('DelphiProject\shellex\ContextMenuHandlers', '', '');
    CreateRegKey('*\shellex\ContextMenuHandlers\RenameMaster', '', ClassID);
    CreateRegKey('.RMSendTo','','CLSID\'+ClassID);
    CreateRegKey('.RMSendTo\ShellEx','','');
    CreateRegKey('.RMSendTo\ShellEx\DropHandler','',ClassID);
    CreateRegKey('CLSID\'+ClassID,'NeverShowExt','');
    CreateRegKey('CLSID\'+ClassID +'\DefaultIcon','',path + 'RenameMaster.exe,0');

    path := IncludeTrailingPathDelimiter(GetPath(CSIDL_SENDTO));
    assignfile(tf,path + 'Rename Master.rmsendto');
    rewrite(tf);
    closefile(tf);

    if (Win32Platform = VER_PLATFORM_WIN32_NT) then
      with TRegistry.Create do
        try
          RootKey := HKEY_LOCAL_MACHINE;
          OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions', True);
          OpenKey('Approved', True);
          WriteString(ClassID, 'RenameMaster Right-Click');
        finally
          Free;
        end;
  end
  else begin
    path := IncludeTrailingPathDelimiter(GetPath(CSIDL_SENDTO));
    if not DeleteFile(path + 'Rename Master.rmsendto') then begin
        ShowMessage('not removed:' + path + 'Rename Master.rmsendto');
    end;

    DeleteRegKey('*\shellex\ContextMenuHandlers\RenameMaster');
    DeleteRegKey('.RMSendTo\ShellEx\DropHandler');
    DeleteRegKey('.RMSendTo\ShellEx');
    DeleteRegKey('.RMSendTo');
    inherited UpdateRegistry(Register);
  end;
end;



//==============================================================
// IDropTarget routines
//==============================================================
function TContextMenu.DragEnter(const dataObj: IDataObject;
  grfKeyState: Integer; pt: TPoint; var dwEffect: Integer): HResult;
begin
  dwEffect := DROPEFFECT_COPY;
    result := S_OK;
end;

function TContextMenu.DragLeave: HResult;
begin
    result := S_OK;
end;

function TContextMenu.DragOver(grfKeyState: Integer; pt: TPoint;
  var dwEffect: Integer): HResult;
begin
  dwEffect := DROPEFFECT_COPY;
    result := S_OK;
end;

function TContextMenu.Drop(const dataObj: IDataObject; grfKeyState: Integer;
  pt: TPoint; var dwEffect: Integer): HResult;
var
  StgMedium: TStgMedium;
  FormatEtc: TFormatEtc;
  i : cardinal;
  lp : TCMInvokeCommandInfo;
begin
 // Fail the call if lpdobj is Nil.
  if (dataObj = nil) then begin
    Result := E_INVALIDARG;
    Exit;
  end;
  //.ShowMessage('spot1');

  with FormatEtc do begin
    cfFormat := CF_HDROP;
    ptd      := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex   := -1;
    tymed    := TYMED_HGLOBAL;
  end;
  //ShowMessage('spot2');

  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := dataObj.GetData(FormatEtc, StgMedium);
  if Failed(Result) then
    Exit;
  //ShowMessage('spot3');

  fileset := TStringList.Create;
  // copy the name of every dropped file
  files := DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0);
  if ( files >= 1) then begin
    for i := 0 to FILES - 1 do begin
        DragQueryFile(StgMedium.hGlobal, i, FFileName, SizeOf(FFileName));
        fileset.Add(FFilename);
    end;
    Result := NOERROR;
  end
  else begin
    FFileName[0] := #0;
    Result := E_FAIL;
  end;
  ReleaseStgMedium(StgMedium);

  lp.lpVerb := 0;
  lp.nShow := SW_SHOWNORMAL;
  self.InvokeCommand(lp);
end;


//=============================================================
// IPersistFile
//=============================================================

function TContextMenu.GetClassID(out classID: TCLSID): HResult;
begin
    result := E_NOTIMPL;
end;
function TContextMenu.Save(pszFileName: POleStr; fRemember: BOOL): HResult;
begin
    result := E_NOTIMPL;
end;

function TContextMenu.SaveCompleted(pszFileName: POleStr): HResult;
begin
    result := E_NOTIMPL;
end;

function TContextMenu.IsDirty: HResult;
begin
    result := E_NOTIMPL;
end;

function TContextMenu.Load(pszFileName: POleStr; dwMode: Integer): HResult;
begin
    result := S_OK;
end;
function TContextMenu.GetCurFile(out pszFileName: POleStr): HResult;
begin
    result := E_NOTIMPL;
end;



initialization
  TContextMenuFactory.Create(ComServer, TContextMenu, Class_ContextMenu,
    '', 'RenameMaster Right-Click', ciMultiInstance,
    tmApartment);
end.
