unit DiskTreeView;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ComCtrls, StrUtils, Dialogs, forms;


const Icon_Drive_Floppy: integer = 0;
const Icon_Drive_CDROM: integer = 1;
const Icon_Drive_Fixed: integer = 2;
const Icon_Drive_Network: integer = 3;
const Icon_Drive_Ghost: integer = 4;
const Icon_Folder_Normal: integer = 5;
const Icon_Folder_Network: integer = 6;
const Icon_Back_Arrow: integer = 7;
const Icon_File: integer = 8;
const Icon_Up_arrow: integer = 9;
const Icon_MyPictures = 12;
const Icon_MyDesktop = 13;

{
For all drives, the lowest order bit is set
(drive_constant and node_is_root) > 0 is TRUE
}
const Node_Is_Root = 1;
const Node_Fixed_Drive = 3;
const Node_Network_Drive = 5;
const Node_Floppy_Drive = 7;
const Node_CDROM_Drive = 9;
const Node_Ghost_Drive = 11;

{
For all folders,
(folder_constand and node_is_root) > 0 is FALSE
For all files,
(folder_constant = 0) is TRUE
}
const Node_Folder = 2;
const Node_Folder_Network = 4;
const Node_Folder_Back = 6;

const Node_File = 0;

const Node_Placeholder = 8;



Type TTreeNodeType = record
    NodeType : integer;
end;
Type PTreeNodeType = ^TTreeNodeType;


type TDiskTreeViewThread = class;

  TDiskTreeView = class(TTreeView)
  private
    { Private declarations }
    xOnExpanding: TTVExpandingEvent;
    xOnClick: TNotifyEvent;
    xOnExpanded: TTVExpandedEvent;
    xOnMouseDown: TMouseEvent;
    //xOnKeyUp: TKeyEvent;
    {
    Node Types - Each Node.Data points to a record containing a node identifier
    }
    NodeFixed: TTreeNodeType;
    NodeFloppy: TTreeNodeType;
    NodeCDROM: TTreeNodeType;
    NodeNetwork: TTreeNodeType;
    NodeGhost: TTreeNodeType;

    NodeFolder: TTreeNodeType;
    NodeFolderBack: TTreeNodeType;
    NodeFolderNetwork: TTreeNodeType;

    NodeFile: TTreeNodeType;
    NodePlaceholder: TTreeNodeType;

    NewFolder : TTreeNode;
    AddCurrent : TTreeNode;
    AltText : TStringList;

    LastChangeToPath : string;

    subs : TDiskTreeViewThread;

    DisableSubfolderScan : boolean;
    xShowHidden : boolean;
    WasExpaned : boolean;
    function GetAltText(n : TTreeNode) : string;
  protected
    { Protected declarations }
    function HasSubfolders(path: string) : boolean;
    procedure AddSubfolderPlaceholder(Node: TTreeNode); overload;
    procedure AddSubfolderPlaceholder(Node: TTreeNode; Unconditional: boolean); overload;
    procedure AddSubfolders(Node: TTreeNode);

    function DoKeyUp(var Message: TWMKey): Boolean;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure InitDiskTree(skiprootfolders : boolean = false);
    procedure ReInit;
    {Events}
    function CanExpand(Node: TTreeNode): Boolean; override;
    procedure Click; override;
    procedure Expand(Node: TTreeNode); override;
    procedure MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer); override;

    {navigation}
    procedure ChangeTo(Path: string);
    procedure GoUpOneLevel();
    procedure GoDown(Subfolder: string);
    procedure SelectFirstFixedDisk();
    procedure SelectFirstNode;
    {Utility}
    function GetFullPathOf(Node: TTreeNode) : string;
    function GetFullPathSelected(): string;
    function IsAnyFolder(Node: TTreeNode): boolean;
    function IsFolderNormal(Node: TTreeNode): boolean;
    function IsFolderNetwork(Node: TTreeNode): boolean;
    function IsRoot(Node: TTreeNode): boolean;
    function IsFixedDisk(Node: TTreeNode): boolean;


    procedure RefreshSelectedNode;

    procedure CenterSelected;
  published
    { Published declarations }
    property ShowHidden : boolean read xShowHidden write xShowHidden;
    property OnExpanding: TTVExpandingEvent read xOnExpanding write xOnExpanding;
    property OnExpanded: TTVExpandedEvent read xOnExpanded write xOnExpanded;

    property OnClick: TNotifyEvent read xOnClick write xOnClick;

    property OnMouseDown: TMouseEvent read xOnMouseDown write xOnMouseDown;
    //property OnKeyUp: TKeyEvent read OnKeyUp write OnKeyUp;
  end;


    TDiskTreeViewThread = class(TThread)
    private
        dtv : TDiskTreeView;
    public
        procedure Execute();
        Constructor Create(d : TDiskTreeView);
    end;

procedure Register;




{////////////////////}
{\\}implementation{//}

Uses UnitToken, UnitSpecialPaths, UnitFrmNewFolder, UnitFrmMain,
  UnitFrmDummyShowPreview;

procedure Register;
begin
  RegisterComponents('Samples', [TDiskTreeView]);
end;


{
'--=======================
'-- // Inititilization //
'--=======================
}

constructor TDiskTreeView.Create(AOwner: TComponent);
begin
    inherited create(aowner);

    {
    Init each node type ID, all nodes will point to one of these
    records
    }
    NodeFixed.nodetype := node_fixed_drive;
    NodeFloppy.NodeType := Node_Floppy_Drive;
    NodeCDROM.NodeType := Node_CDROM_Drive;
    NodeNetwork.NodeType := Node_Network_Drive;
    NodeGhost.NodeType := Node_Ghost_Drive;

    NodeFolder.NodeType := Node_Folder;
    NodeFolderBack.NodeType := Node_Folder_Back;
    NodeFolderNetwork.NodeType := Node_Folder_Network;

    NodeFile.NodeType := Node_File;
    NodePlaceholder.NodeType := Node_Placeholder;

    self.HideSelection := false;

    AltText := TStringList.Create;
    MultiSelect := false;
    subs := TDiskTreeViewThread.Create(self);
    self.ReadOnly := true;
end;


function TDiskTreeView.GetAltText(n: TTreeNode): string;
begin
    result := alttext.Values[IntToStr(n.absoluteindex)]
end;


procedure TDiskTreeView.InitDiskTree(skiprootfolders : boolean = false);
var Root: string;
    DriveLetter: char;
    Node: TTreeNode;
    DriveType: integer;
    pnt: PTreeNodeType;
    sp : TSpecialPaths;
    s : string;
    sl : TStringList;
    i, cnt : integer;
begin

    self.Items.clear;

    if not skiprootfolders then begin
        sp := TSpecialPaths.Create;
        Node := self.Items.Add( self.items.GetFirstNode,  Root);
        alttext.Values[IntToStr( Node.absoluteindex )]  := sp.GetMyDocuments;
        node.Text := 'My Documents';
        node.ImageIndex := Icon_Folder_Normal ;
        node.SelectedIndex := node.ImageIndex;
        node.Data := Pointer(@NodeFolderNetwork);

        s := sp.GetMyPictures;
        if s <> '' then begin
            Node := self.Items.Add( self.items.GetFirstNode,  Root);
            alttext.Values[IntToStr( Node.absoluteindex )] := s;
            node.Text := 'My Pictures';
            node.ImageIndex := Icon_MyPictures;
            node.SelectedIndex := node.ImageIndex;
            node.Data := Pointer(@NodeFolderNetwork);
        end;

        s := sp.GetDesktop;
        if s <> '' then begin
            Node := self.Items.Add( self.items.GetFirstNode,  Root);
            alttext.Values[IntToStr( Node.absoluteindex )] := s;
            node.Text := 'Desktop';
            node.ImageIndex := Icon_MyDesktop;
            node.SelectedIndex := node.ImageIndex;
            node.Data := Pointer(@NodeFolderNetwork);
        end;

        if FrmNewFolder <> nil then begin
        sl := FrmNewFolder.GetFolders;
        for i := 0 to sl.Count - 1 do begin
            Node := self.Items.Add( self.items.GetFirstNode,  Root);
            node.Text := sl.Names[i];
            alttext.Values[IntToStr( Node.absoluteindex )] := sl.Values[node.Text];

            node.ImageIndex := Icon_Folder_Normal;
            node.SelectedIndex := node.ImageIndex;
            node.Data := Pointer(@NodeFolderNetwork);
        end;
        end;

        NewFolder := self.Items.Add(self.items.GetFirstNode,  Root);
        NewFolder.Text := 'New Folder ...';
        NewFolder.ImageIndex := Icon_Back_Arrow;
        NewFolder.data := Pointer(@NodeFolderBack);
        NewFolder.SelectedIndex := NewFolder.ImageIndex;


        AddCurrent := self.Items.Add(self.items.GetFirstNode,  Root);
        AddCurrent.Text := 'Add Current ...';
        AddCurrent.ImageIndex := Icon_Back_Arrow;
        AddCurrent.data := Pointer(@NodeFolderBack);
        AddCurrent.SelectedIndex := AddCurrent.ImageIndex;
        sp.free;
    end;

    cnt := 0;
    for DriveLetter := 'A' to 'Z' do begin
        Root := DriveLetter + ':';

        Node := nil;
        DriveType := GetDriveType(PChar( Root + '/' )) ;
        if (DriveType <> DRIVE_NO_ROOT_DIR) then begin
           Node := self.Items.Add( self.items.GetFirstNode,  Root);
        end else begin
            CONTINUE;
        end;
        
        case DriveType of
            DRIVE_REMOVABLE: begin
                node.ImageIndex := icon_drive_floppy;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeFloppy;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_CDROM: begin
                node.ImageIndex := icon_drive_cdrom;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeCDROM;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_FIXED: begin
                node.ImageIndex := icon_drive_fixed;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeFixed;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_REMOTE: begin
                node.ImageIndex := icon_drive_network;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeNetwork;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_NO_ROOT_DIR: begin
            end;
            else begin
                node.ImageIndex := icon_drive_ghost;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeGhost;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
        end;
        if node <> nil then begin
            node.ImageIndex := cnt+Icon_MyDesktop+1;
            node.SelectedIndex := node.ImageIndex;
            inc(cnt);
        end;
    end;


    if self.LastChangeToPath <> '' then begin
        ChangeTo(self.LastChangeToPath);
    end;
end;


{
'--=======================
'-- // Event Handlers //
'--=======================
}
function TDiskTreeView.CanExpand(Node: TTreeNode): Boolean;
begin
    result := true;
    if assigned(xOnExpanding) then xOnExpanding(self, node, result);

    //ShowMessage('name: ' + node.Text + ' canExpand: ' + BoolToStr(result));

    // If expansion is allowed...
    //
    // Delete the placeholder
    // Add any folders found
    //
    if (result) then begin
        if (node.getfirstchild <> nil) then begin
            if (Node.getFirstChild.text = '?') then begin
                Node.DeleteChildren;
                AddSubfolders(Node);
                result := (node.count <> 0);
                //ShowMessage('Folders count: ' + IntToSTr(node.count));
            end else begin
                //ShowMessage('no placeholder found for ' + node.getFirstChild.text );
            end;
        end else begin
            //ShowMessage('getFirstChild is null');
            Result := false;
        end;

        
    end;
end;


procedure TDiskTreeView.CenterSelected;
var x, y : integer;
    r, r2 : TRect;
begin
    if self.selected = nil then EXIT;
    self.Selected.MakeVisible;

    x := self.Left;
    y := trunc(self.Height / 2);
    r2 := self.Selected.DisplayRect(true);

    if (r2.top > y) or (r2.left > x + 25) then begin

        while (r2.top > y) do begin
            r.top := r2.top;
            SendMessage(self.Handle, WM_VSCROLL, SB_LINEDOWN, 0);

            r2 := self.Selected.DisplayRect(true);
            if r.top = r2.top then BREAK; // infinite loop condition
        end;

        while (r2.Left > x + 25) do begin
            r.Left := r2.left;
            SendMessage(self.Handle, WM_HSCROLL, SB_LINERIGHT, 0);

            r2 := self.Selected.DisplayRect(true);
            if r.Left = r2.left then BREAK;
        end;
    end else begin
        {while (r2.top < y) do begin
            r.top := r2.top;
            SendMessage(self.Handle, WM_VSCROLL, SB_LINEUP, 0);

            r2 := self.Selected.DisplayRect(true);
            if r.top = r2.top then BREAK; // infinite loop condition
        end;
        }
    end;
end;

procedure TDiskTreeView.Expand(Node: TTreeNode);
begin
    if Assigned(xOnExpanded) then xOnExpanded(Self, Node);

    {
    if IsRoot(node) then   begin
        n := self.Items.GetFirstNode;
        while n <> nil do begin
            if n <> node then n.Collapse(false);

            n := n.getNextSibling;
        end;
    end;
    }
    if not self.DisableSubfolderScan then
        subs.execute;

    self.WasExpaned := true;
end;

procedure TDiskTreeView.Click;
var node: TTreeNode;
    s : string;
begin
    {if wasexpaned then begin
        WasExpaned := false;
        EXIT;
    end;
    }



    node := self.selected;
    if (node = NewFolder) or (node = AddCurrent) then begin
        FrmNewFolder.txtName.text := '';
        FrmNewFolder.txtFolder.text := '';

        if node = AddCurrent then begin
            FrmNewFolder.txtFolder.text := frmmain.GetPath;
        end;

        FrmNewFolder.ShowModal;
        self.InitDiskTree;
        EXIT;
    end;


    if assigned(xOnClick) then xOnClick(self);

    // find selected node
    // if node is network or already empty
    // if node is not currently expanded
    // refresh children


    if (node <> nil) then begin
        s := GetAltText(node);
        {if Windows.GetDriveType(pchar(ExtractFileDrive(s)  + '/')) =  DRIVE_NO_ROOT_DIR then begin
            self.ReInit;
            EXIT;
        end;
        }
        if s = '' then begin
            {
            if (IsFolderNormal(node) or IsRoot(node)) then begin
                if not node.Expanded then begin
                    node.DeleteChildren;
                    AddSubfolders(node);
                end;
            end else begin
                if not node.Expanded then begin
                    node.DeleteChildren;
                    AddSubfolderPlaceholder(node);
                end;
            end;
            if self.RowSelect and (not node.expanded) then  begin
                node.expand(false);
            end else if self.RowSelect and (node.expanded) then begin
                node.Collapse(true);
            end;
            }
        end else begin
            // a user can click collapse all and then click
            // a custom shortcut to the same path and it will be ignored otherwise
            self.LastChangeToPath := '';
            self.ChangeTo(s);
            self.CenterSelected;
        end;
    end;
end;

function TDiskTreeView.DoKeyUp(var Message: TWMKey): Boolean;
begin
    result := true;
    if message.CharCode in [VK_SPACE, VK_RETURN] then begin

    end else begin
        result := inherited DoKeyUp(message);
    end;
end;


{
// Still, to date, there is NO method of listing folders only
// FindExSearchLimitToDirectories - is not implemented yet in Vista
function HasSubfoldersEx(path : string) : boolean;
var h: THandle;
    wfa: WIN32_FIND_DATA;
    show: boolean;
    dwAdditionalFlags: Dword;
    indexInfoLevels: _FINDEX_INFO_LEVELS;
    indexSearchOps: _FINDEX_SEARCH_OPS;
begin
    result := false;


   dwAdditionalFlags := 0;
   indexInfoLevels := FindExInfoStandard;
   indexSearchOps := FindExSearchLimitToDirectories;
    h := THandle(
        Windows.FindFirstFileEx(
            PChar(path + '*.*'),
            indexInfoLevels,
            @wfa,
            indexSearchOps,
            nil,
            dwAdditionalFlags
        )
    );

    if h<>INVALID_HANDLE_VALUE then begin
        repeat
            if (String(wfa.cFileName) = '.') or
                (String(wfa.cFileName) = '..') then continue;

            result := true;
            break;
        until not Windows.FindNextFile(h,wfa);
    end;
    Windows.FindClose(h);
end;
}
function TDiskTreeView.HasSubfolders(path: string) : boolean;
var sr: TSearchRec;
    r: integer;
label  endloop;
begin
    //result := HasSubfoldersEx(path);
    //EXIT;

    result := false;
    {$i-}
    r := FindFirst(path + '*.*', faanyfile, sr);
    {$i+}
    while (r = 0) do begin

        if ((sr.Attr and  faDirectory) > 0) and
            (sr.Name[1] <> '.') then begin
            result := true;

            goto endloop;
        end;
        R := FindNext(sr);
    end;

endloop:
    FindClose(sr);
end;

procedure TDiskTreeView.AddSubfolderPlaceholder(Node: TTreeNode);
begin
    AddSubfolderPlaceholder(node, false);
end;

procedure TDiskTreeView.AddSubfolderPlaceholder(Node: TTreeNode; Unconditional : Boolean);
var
    Path: string;
    p: PTreeNodeType;
    n: TTreeNode;
begin

    // a check for subfolders can on a networked disk can freeze the program
    // for a long period of time
    p := @NodePlaceholder;


    // TODO: re-written for the thread scanner
    // - clean up this codde instead of forcing always Uncondition
    unconditional := true;
    if (Unconditional) then begin
        if node.Expanded  then EXIT;
        if node.Count > 0 then EXIT;
        
        n := self.Items.AddChild(Node, '?');
        n.Data := p;
    end else begin
        if (isFixedDisk(Node)) or (isFoldernormal(node)) then begin
            path := GetFullPathOf(node);
            if (HasSubfolders(path)) then begin
                n := self.Items.AddChild(Node, '?');
                n.Data := p;
            end;
        end else begin
            n := self.Items.AddChild(Node, '?');
            n.Data := p;
        end;
    end;
end;


procedure TDiskTreeView.AddSubfolders(Node: TTreeNode);
var sr: TSearchRec;
    Result: integer;
    n: TTreeNode;
    pnt: PTreeNodeType;
    Path: string;
    attr : integer;
begin
    path := GetFullPathOf(Node);
    try
        {$i-}
        {$WARN SYMBOL_PLATFORM OFF}
        attr := faDirectory or faSysFile or faHidden;
        if not self.xShowHidden then begin
            attr := attr xor faHidden;
        end;
        result := findfirst(path + '*.*', attr, sr);
        {$WARN SYMBOL_PLATFORM ON}
        {$i+}
    finally
        // don't show error when a floppy is selcted
    end;
    // desktop.ini modified folders won't show without faSysFile

    while (result = 0) do begin
        if ((sr.Attr and  fadirectory) > 0) then begin
            if (sr.Name <> '.') and (sr.Name <> '..') then begin
               n := self.Items.AddChild(Node, sr.name);

               // Mark folders on a normal fixed disk with a 5
               // mark all others with a 6
               if (isFixedDisk(n.Parent)) or
                   (IsFolderNormal(n.Parent)) then begin
                   n.ImageIndex := icon_folder_normal;
                   pnt := @nodefolder;
               end else begin
                   n.ImageIndex := icon_folder_network;
                   pnt := @nodefoldernetwork;
               end;
               n.SelectedIndex := n.ImageIndex;
               n.Data := pnt;
               AddSubfolderPlaceholder(n, not self.IsFolderNormal(n));
            end;
        end;
        Result := FindNext(sr);
    end;
    FindClose(sr);

    node.AlphaSort(false);
end;








// TODO: replace this method
// .Parent relies on SendMessage and is a very slow method when called 6000 times
function TDiskTreeView.GetFullPathOf(Node: TTreeNode) : string;
var n : TTreeNode;
    s, s2 : string;
begin
    n := Node;
    While n.Parent <> nil do begin
        s2 := GetAltText(n);
        if (s2 <> '')  then begin
            s := s2 +'\'+ s;
        end else begin
            s := n.Text +'\'+ s;
        end;
        n := n.Parent;
    end;
    
    s2 := GetAltText(n);
    if (s2 <> '')  then begin
        s := s2 +'\'+ s;
    end else begin
        s := n.Text +'\'+ s;
    end;

    result := s;
end;


procedure TDiskTreeView.GoUpOneLevel();
begin
    if (self.Selected <> nil) then begin
        if (self.selected.parent <> nil) then begin
            self.Selected := self.Selected.Parent;
        end;
    end;
end;

procedure TDiskTreeView.GoDown(Subfolder: string);
var node, sub: TTreeNode;


    s : string;
label endloop;
begin
    if (self.Selected <> nil) then begin

        //if self.selected.expanded = false then self.Selected.Expand(false);
        self.Selected.Expand(false);

        node := self.Selected;

        s := uppercase(subfolder);
        sub := node.GetFirstChild;

        while sub <> nil do begin
            if uppercase(sub.Text) = s then begin
                sub.Selected := true;
                goto endloop;
            end;

            sub := sub.getNextSibling;
        end;
        self.selected.Selected := false;
        EXIT;
        {
        for i := 0 to node.Count  - 1 do begin
            if (uppercase(node.Item[i].Text) = s) then begin
                self.Selected := node.Item[i];

                goto endloop;
            end;
        end;
        }
        
        endloop:
        self.Selected.Expand(false);
        //self.Selected.MakeVisible
    end else begin
        //ShowMessage('Node not selected');
    end;
end;


procedure TDiskTreeView.ChangeTo(Path: string);
var s: string;


    node : TTreeNode;
begin
    if path = LastChangeToPath then begin
        if (self.Selected<>nil) and (self.GetFullPathOf(self.Selected) = Path) then
            EXIT;
    end;
    
    self.LastChangeToPath := path;

    s := lowercase(ExtractFileDrive(path));
    path := rightstr(path, length(path) - length(s));
    path := ExcludeTrailingPathDelimiter(path); 


    // find the drive
    node := self.Items.GetFirstNode;
    while node <> nil do begin
        if lowercase(node.text) = s then begin
            self.Selected := node;
            BREAK;
        end;
        node := node.getNextSibling;

    end;


    {
    for i := 0 to self.Items.Count - 1 do begin
        if lowercase(self.Items[i].Text) = s then begin
            self.Selected := self.Items[i];
            break;
        end;
    end;}
    // drive may not exists any more
    if self.Selected = nil then Exit;

   

    self.DisableSubfolderScan := true;

    self.Items.BeginUpdate;
    self.selected.expand(false);

    if leftstr(path,1) = '\' then path := rightstr(path, length(path) - 1);

    s := TokenString(path, '\');
    while s <> '' do begin
        self.GoDown(s);
        if self.selected = nil then break;

        s := TokenString(path, '\');
    end;
    self.Items.EndUpdate;
    if (self.selected <> nil) then
        self.CenterSelected;

    self.DisableSubfolderScan := False;

    application.ProcessMessages;
    self.WasExpaned := false;
    subs.Execute;    
end;


procedure TDiskTreeView.SelectFirstFixedDisk();
var node : ttreenode;
label endloop;
begin

    node := self.Items.GetFirstNode;
    while node <> nil do begin
        if IsFixedDisk(node) then begin
            self.Selected := node;
            //self.Selected.Expanded := true;
            self.Select(node,  [ssLeft]);
            self.Repaint;
            BREAK;
        end;
        node := node.getNextSibling;
    end;

{
    for i := 0 to self.Items.Count - 1 do begin
        s := self.items[i].Text;

        if (IsFixedDisk(self.Items[i])) then begin
            self.Selected := self.Items[i];
            self.Selected.Expanded := true;
            self.Select(self.Items[i],  [ssLeft]);
            //self.Expand(self.Items[i]);
            self.Repaint;
            //self.Click;
            goto endloop;
        end;
    end;
 }
endloop:

end;


function TDiskTreeView.GetFullPathSelected(): string;
begin
    if (self.Selected <> nil) then begin
        result := getfullpathof(self.Selected);
    end else begin
        result := '';
    end;
end;




function TDiskTreeView.IsAnyFolder(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = node_folder)or( p.NodeType = node_folder_network);
end;
function TDiskTreeView.IsFolderNormal(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = node_folder);
end;

function TDiskTreeView.IsFolderNetwork(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = node_folder_network);
end;

function TDiskTreeView.IsRoot(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType and Node_Is_Root) > 0;
end;


procedure TDiskTreeView.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);

begin
    inherited;
    if Assigned(xOnMouseDown) then xOnMouseDown(Self, Button, Shift, X, Y);

    if button <> mbRight then begin
        if htOnItem in self.GetHitTestInfoAt(x,y) then begin
            self.Selected := self.GetNodeAt(x,y);
        end else begin
            self.Selected := nil;
        end;
    end;

end;

procedure TDiskTreeView.RefreshSelectedNode;
begin
    if self.Selected = nil then EXIT;

    self.Selected.DeleteChildren;
    self.AddSubfolderPlaceholder(self.Selected);

    self.Selected.Expand(false);
end;

procedure TDiskTreeView.ReInit;
var Root: string;
    DriveLetter: char;
    Node, n : TTreeNode;
    DriveType: integer;
    pnt: PTreeNodeType;

    sl : TStringList;
    cnt : integer;
    found{, newdrive} : boolean;
begin
    self.LastChangeToPath := '';

    sl := TStringList.create();
    // if drive is found, test next drive
    n := self.Items.GetFirstNode;
    while n <> nil do begin
        sl.AddObject(n.text,n);
        n := n.getNextSibling;
    end;
    //found := (n <> nil) and (n.text = root);


    // search for new drive letters
    cnt := 0;
    for DriveLetter := 'A' to 'Z' do begin
        Root := DriveLetter + ':';


        found := sl.IndexOf(root) <> -1;

        DriveType := GetDriveType(PChar( Root + '/' )) ;
        if (DriveType = DRIVE_NO_ROOT_DIR) then begin
            if found then begin
                node := TTreenode(sl.Objects[sl.IndexOf(root)]);
                self.items.Delete(node);
            end;
            CONTINUE;
        end;

        if found then begin
            node := TTreenode(sl.Objects[sl.IndexOf(root)]);
             if node <> nil then begin
                node.ImageIndex := cnt+Icon_MyDesktop+1 ;
                node.SelectedIndex := node.ImageIndex;
            end;
            //inc(cnt);
            //CONTINUE;
        end else begin

            Node := self.Items.Add( self.items.GetFirstNode,  Root);
           // newdrive := true;
        end;

        //Node := nil;
        case DriveType of
            DRIVE_REMOVABLE: begin
                node.ImageIndex := icon_drive_floppy;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeFloppy;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_CDROM: begin
                node.ImageIndex := icon_drive_cdrom;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeCDROM;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_FIXED: begin
                node.ImageIndex := icon_drive_fixed;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeFixed;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_REMOTE: begin
                node.ImageIndex := icon_drive_network;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeNetwork;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_NO_ROOT_DIR: begin
                //if found then node.Delete;

            end;
            else begin
                node.ImageIndex := icon_drive_ghost;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeGhost;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
        end;
        node.ImageIndex := cnt+Icon_MyDesktop+1 ;
        node.SelectedIndex := node.ImageIndex;
        inc(cnt)
    end;

    if (frmdummyshowpreview <> nil) then
        frmDummyShowPreview.refreshdriveicons;

    sl.Destroy;
end;

function TDiskTreeView.IsFixedDisk(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = Node_Fixed_Drive);
end;


procedure TDiskTreeView.SelectFirstNode;
begin
    self.Items[0].Selected := true;
    self.CenterSelected;
    //self.items[0].MakeVisible;
end;


{==============================================}
{ TDiskTreeViewThread }
{==============================================}

constructor TDiskTreeViewThread.Create(d: TDiskTreeView);
begin
    inherited create(true);

    self.dtv := d;
end;

procedure TDiskTreeViewThread.Execute;

    procedure ScanNodes(path : string; n : TTreeNode);
    var node : TTreeNode;
    begin
        path := IncludeTrailingPathDelimiter(path);
        node := n.getFirstChild;
        while node <> nil do begin
            if not dtv.HasSubfolders( IncludeTrailingPathDelimiter(path + node.text) ) then begin
                // remove placeholder
                node.DeleteChildren;
            end;

            if node.Expanded  then begin
                ScanNodes(path + node.text, node);
            end;
            node := node.getNextChild(node);
        end;
    end;
var node : TTreeNode;
begin

    // enumerate all root nodes
    // scan only Expanded folders

    //dtv.Items.BeginUpdate;
    node := dtv.Items.GetFirstNode;
    while node <> nil do begin
        if node.Expanded and dtv.IsFixedDisk(node) then begin
           ScanNodes(node.text, node);
        end;

        node := node.getNextSibling;
    end;
    //dtv.Items.EndUpdate;

    self.Suspend;
end;

end.
