unit DiskTreeView;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ComCtrls, StrUtils, Dialogs;


const Icon_Drive_Floppy: integer = 0;
const Icon_Drive_CDROM: integer = 1;
const Icon_Drive_Fixed: integer = 2;
const Icon_Drive_Network: integer = 3;
const Icon_Drive_Ghost: integer = 4;
const Icon_Folder_Normal: integer = 5;
const Icon_Folder_Network: integer = 6;
const Icon_Back_Arrow: integer = 7;
const Icon_File: integer = 8;
const Icon_Up_arrow: integer = 9;

{
For all drives, the lowest order bit is set
(drive_constant and node_is_root) > 0 is TRUE
}
const Node_Is_Root = 1;
const Node_Fixed_Drive = 3;
const Node_Network_Drive = 5;
const Node_Floppy_Drive = 7;
const Node_CDROM_Drive = 9;
const Node_Ghost_Drive = 11;

{
For all folders,
(folder_constand and node_is_root) > 0 is FALSE
For all files,
(folder_constant = 0) is TRUE
}
const Node_Folder = 2;
const Node_Folder_Network = 4;
const Node_Folder_Back = 6;

const Node_File = 0;

const Node_Placeholder = 8;



Type TTreeNodeType = record
    NodeType : integer;
end;
Type PTreeNodeType = ^TTreeNodeType;



type
  TDiskTreeView = class(TTreeView)
  private
    { Private declarations }
    xOnExpanding: TTVEditingEvent;
    xOnClick: TNotifyEvent;

    {
    Node Types - Each Node.Data points to a record containing a node identifier
    }
    NodeFixed: TTreeNodeType;
    NodeFloppy: TTreeNodeType;
    NodeCDROM: TTreeNodeType;
    NodeNetwork: TTreeNodeType;
    NodeGhost: TTreeNodeType;

    NodeFolder: TTreeNodeType;
    NodeFolderBack: TTreeNodeType;
    NodeFolderNetwork: TTreeNodeType;

    NodeFile: TTreeNodeType;
    NodePlaceholder: TTreeNodeType;

    NewFolder : TTreeNode;
    AddCurrent : TTreeNode;
    AltText : TStringList;
    function GetAltText(n : TTreeNode) : string;
  protected
    { Protected declarations }
    function HasSubfolders(path: string) : boolean;
    procedure AddSubfolderPlaceholder(Node: TTreeNode); overload;
    procedure AddSubfolderPlaceholder(Node: TTreeNode; Unconditional: boolean); overload;
    procedure AddSubfolders(Node: TTreeNode);

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure InitDiskTree(skiprootfolders : boolean = false);

    {Events}
    function CanExpand(Node: TTreeNode): Boolean; override;
    procedure Click; override;


    {navigation}
    procedure ChangeTo(Path: string);
    procedure GoUpOneLevel();
    procedure GoDown(Subfolder: string);
    procedure SelectFirstFixedDisk();
    procedure SelectFirstNode;
    {Utility}
    function GetFullPathOf(Node: TTreeNode) : string;
    function GetFullPathSelected(): string;
    function IsAnyFolder(Node: TTreeNode): boolean;
    function IsFolderNormal(Node: TTreeNode): boolean;
    function IsFolderNetwork(Node: TTreeNode): boolean;
    function IsRoot(Node: TTreeNode): boolean;
    function IsFixedDisk(Node: TTreeNode): boolean;

  published
    { Published declarations }
    property OnExpanding: TTVEditingEvent read xOnExpanding write xOnExpanding;
    property OnClick: TNotifyEvent read xOnClick write xOnClick;
  end;

procedure Register;

implementation

Uses UnitToken, UnitSpecialPaths, UnitFrmNewFolder, UnitFrmMain;

procedure Register;
begin
  RegisterComponents('Samples', [TDiskTreeView]);
end;


{
'--=======================
'-- // Inititilization //
'--=======================
}

constructor TDiskTreeView.Create(AOwner: TComponent);
begin
    inherited create(aowner);

    {
    Init each node type ID, all nodes will point to one of these
    records
    }
    NodeFixed.nodetype := node_fixed_drive;
    NodeFloppy.NodeType := Node_Floppy_Drive;
    NodeCDROM.NodeType := Node_CDROM_Drive;
    NodeNetwork.NodeType := Node_Network_Drive;
    NodeGhost.NodeType := Node_Ghost_Drive;

    NodeFolder.NodeType := Node_Folder;
    NodeFolderBack.NodeType := Node_Folder_Back;
    NodeFolderNetwork.NodeType := Node_Folder_Network;

    NodeFile.NodeType := Node_File;
    NodePlaceholder.NodeType := Node_Placeholder;

    self.HideSelection := false;

    AltText := TStringList.Create;
end;


function TDiskTreeView.GetAltText(n: TTreeNode): string;
begin
    result := alttext.Values[IntToStr(n.absoluteindex)]
end;


procedure TDiskTreeView.InitDiskTree(skiprootfolders : boolean = false);
var Root: string;
    DriveLetter: char;
    Node: TTreeNode;
    DriveType: integer;
    pnt: PTreeNodeType;
    sp : TSpecialPaths;
    s : string;
    sl : TStringList;
    i : integer;
begin

    self.Items.clear;

    if not skiprootfolders then begin
        sp := TSpecialPaths.Create;
        Node := self.Items.Add( self.items.GetFirstNode,  Root);
        alttext.Values[IntToStr( Node.absoluteindex )]  := sp.GetMyDocuments;
        node.Text := 'My Documents';
        node.ImageIndex := Icon_Folder_Normal ;
        node.SelectedIndex := node.ImageIndex;
        node.Data := Pointer(@NodeFolderNetwork);

        s := sp.GetMyPictures;
        if s <> '' then begin
            Node := self.Items.Add( self.items.GetFirstNode,  Root);
            alttext.Values[IntToStr( Node.absoluteindex )] := s;
            node.Text := 'My Pictures';
            node.ImageIndex := 12;
            node.SelectedIndex := node.ImageIndex;
            node.Data := Pointer(@NodeFolderNetwork);
        end;

        if FrmNewFolder <> nil then begin
        sl := FrmNewFolder.GetFolders;
        for i := 0 to sl.Count - 1 do begin
            Node := self.Items.Add( self.items.GetFirstNode,  Root);
            node.Text := sl.Names[i];
            alttext.Values[IntToStr( Node.absoluteindex )] := sl.Values[node.Text];

            node.ImageIndex := Icon_Folder_Normal;
            node.SelectedIndex := node.ImageIndex;
            node.Data := Pointer(@NodeFolderNetwork);
        end;
        end;

        NewFolder := self.Items.Add(self.items.GetFirstNode,  Root);
        NewFolder.Text := 'New Folder ...';
        NewFolder.ImageIndex := Icon_Back_Arrow;
        NewFolder.data := Pointer(@NodeFolderBack);
        NewFolder.SelectedIndex := NewFolder.ImageIndex;


        AddCurrent := self.Items.Add(self.items.GetFirstNode,  Root);
        AddCurrent.Text := 'Add Current ...';
        AddCurrent.ImageIndex := Icon_Back_Arrow;
        AddCurrent.data := Pointer(@NodeFolderBack);
        AddCurrent.SelectedIndex := AddCurrent.ImageIndex;
        sp.free;
    end;

    for DriveLetter := 'A' to 'Z' do begin
        Root := DriveLetter + ':';

        Node := nil;
        DriveType := GetDriveType(PChar( Root + '/' )) ;
        if (DriveType <> DRIVE_NO_ROOT_DIR) then begin
           Node := self.Items.Add( self.items.GetFirstNode,  Root);
        end;
        case DriveType of
            DRIVE_REMOVABLE: begin
                node.ImageIndex := icon_drive_floppy;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeFloppy;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_CDROM: begin
                node.ImageIndex := icon_drive_cdrom;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeCDROM;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_FIXED: begin
                node.ImageIndex := icon_drive_fixed;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeFixed;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_REMOTE: begin
                node.ImageIndex := icon_drive_network;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeNetwork;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
            DRIVE_NO_ROOT_DIR: begin
            end;
            else begin
                node.ImageIndex := icon_drive_ghost;
                node.SelectedIndex := node.ImageIndex;
                pnt := @NodeGhost;
                node.Data := pnt;
                AddSubfolderPlaceholder(node,true);
            end;
        end;

    end;

end;


{
'--=======================
'-- // Event Handlers //
'--=======================
}
function TDiskTreeView.CanExpand(Node: TTreeNode): Boolean;
begin
    result := true;
    if assigned(xOnExpanding) then xOnExpanding(self, node, result);

    //ShowMessage('name: ' + node.Text + ' canExpand: ' + BoolToStr(result));

    // If expansion is allowed...
    //
    // Delete the placeholder
    // Add any folders found
    //
    if (result) then begin
        if (node.getfirstchild <> nil) then begin
            if (Node.getFirstChild.text = '?') then begin
                Node.DeleteChildren;
                AddSubfolders(Node);
                result := (node.count <> 0);
                //ShowMessage('Folders count: ' + IntToSTr(node.count));
            end else begin
                //ShowMessage('no placeholder found for ' + node.getFirstChild.text );
            end;
        end else begin
            //ShowMessage('getFirstChild is null');
            Result := false;
        end;
    end;
end;

procedure TDiskTreeView.Click;
var node: TTreeNode;
    s : string;
begin
    node := self.selected;
    if (node = NewFolder) or (node = AddCurrent) then begin
        FrmNewFolder.txtName.text := '';
        FrmNewFolder.txtFolder.text := '';

        if node = AddCurrent then begin
            FrmNewFolder.txtFolder.text := frmmain.cbpath.text;
        end;

        FrmNewFolder.ShowModal;
        self.InitDiskTree;
        EXIT;
    end;


    if assigned(xOnClick) then xOnClick(self);

    // find selected node
    // if node is network or already empty
    // if node is not currently expanded
    // refresh children


    if (node <> nil) then begin
        s := GetAltText(node);
        if s = '' then begin
            if (IsFolderNormal(node) or IsRoot(node)) then begin
                if not node.Expanded then begin
                    node.DeleteChildren;
                    AddSubfolders(node);
                end;
            end else begin
                if not node.Expanded then begin
                    node.DeleteChildren;
                    AddSubfolderPlaceholder(node);
                end;
            end;
        end else begin
            self.ChangeTo(s);
        end;
    end;
end;






function TDiskTreeView.HasSubfolders(path: string) : boolean;
var sr: TSearchRec;
    r: integer;
label  endloop;
begin
    HasSubfolders := false;
    r := FindFirst(path + '*.*', faanyfile, sr);
    while (r = 0) do begin

        if ((sr.Attr and  faDirectory) > 0) and
            (RightStr(sr.Name, 1) <> '.') then begin
            HasSubfolders := true;

            goto endloop;
        end;
        R := FindNext(sr);
    end;

endloop:
    FindClose(sr);
end;

procedure TDiskTreeView.AddSubfolderPlaceholder(Node: TTreeNode);
begin
    AddSubfolderPlaceholder(node, false);
end;

procedure TDiskTreeView.AddSubfolderPlaceholder(Node: TTreeNode; Unconditional : Boolean);
var
    Path: string;
    p: PTreeNodeType;
    n: TTreeNode;
begin
    path := GetFullPathOf(node);

    // a check for subfolders can on a networked disk can freeze the program
    // for a long period of time
    p := @NodePlaceholder;
    if (Unconditional) then begin
        n := self.Items.AddChild(Node, '?');
        n.Data := p;
    end else begin
        if (isFixedDisk(Node)) or (isFoldernormal(node)) then begin
            if (HasSubfolders(path)) then begin
                n := self.Items.AddChild(Node, '?');
                n.Data := p;
            end;
        end else begin
            n := self.Items.AddChild(Node, '?');
            n.Data := p;
        end;
    end;
end;


procedure TDiskTreeView.AddSubfolders(Node: TTreeNode);
var sr: TSearchRec;
    Result: integer;
    n: TTreeNode;
    pnt: PTreeNodeType;
    Path: string;
begin
    path := GetFullPathOf(Node);
    result := findfirst(path + '*.*', faDirectory, sr);

    while (result = 0) do begin
        if ((sr.Attr and  fadirectory) > 0) then begin
            if (sr.Name <> '.') and (sr.Name <> '..') then begin
               n := self.Items.AddChild(Node, sr.name);

               // Mark folders on a normal fixed disk with a 4
               // mark all others with a 5
               if (isFixedDisk(n.Parent)) or
                   (IsFolderNormal(n.Parent)) then begin
                   n.ImageIndex := icon_folder_normal;
                   pnt := @nodefolder;
               end else begin
                   n.ImageIndex := icon_folder_network;
                   pnt := @nodenetwork;
               end;
               n.SelectedIndex := n.ImageIndex;
               n.Data := pnt;
               AddSubfolderPlaceholder(n);
            end;
        end;
        Result := FindNext(sr);
    end;
    FindClose(sr);

    node.AlphaSort(false);
end;









function TDiskTreeView.GetFullPathOf(Node: TTreeNode) : string;
var n : TTreeNode;
    s, s2 : string;
begin
    n := Node;
    While n.Parent <> nil do begin
        s2 := GetAltText(n);
        if (s2 <> '')  then begin
            s := s2 +'\'+ s;
        end else begin
            s := n.Text +'\'+ s;
        end;
        n := n.Parent;
    end;
    
    s2 := GetAltText(n);
    if (s2 <> '')  then begin
        s := s2 +'\'+ s;
    end else begin
        s := n.Text +'\'+ s;
    end;

    result := s;
end;


procedure TDiskTreeView.GoUpOneLevel();
begin
    if (self.Selected <> nil) then begin
        if (self.selected.parent <> nil) then begin
            self.Selected := self.Selected.Parent;
        end;
    end;
end;

procedure TDiskTreeView.GoDown(Subfolder: string);
var node: TTreeNode;
    i: integer;
label endloop;
begin
    if (self.Selected <> nil) then begin

        //if self.selected.expanded = false then self.Selected.Expand(false);
        self.Selected.Expand(false);

        node := self.Selected;

        for i := 0 to node.Count  - 1 do begin
            if (uppercase(node.Item[i].Text) = uppercase(subfolder)) then begin
                self.Selected := node.Item[i];

                goto endloop;
            end;
        end;

        endloop:
        self.Selected.Expand(false);
        self.Selected.MakeVisible
    end else begin
        //ShowMessage('Node not selected');
    end;
end;


procedure TDiskTreeView.ChangeTo(Path: string);
var s: string;
var i: integer;
begin


    s := lowercase(ExtractFileDrive(path));
    path := rightstr(path, length(path) - length(s));

    for i := 0 to self.Items.Count - 1 do begin
        if lowercase(self.Items[i].Text) = s then begin
            self.Selected := self.Items[i];
        end;
    end;
    // drive may not exists any more
    if self.Selected = nil then Exit;

    self.Items.BeginUpdate;
    self.selected.expand(false);

    if leftstr(path,1) = '\' then path := rightstr(path, length(path) - 1);

    s := TokenString(path, '\');
    while s <> '' do begin
        self.GoDown(s);
        s := TokenString(path, '\');
    end;


    self.Items.EndUpdate;
end;


procedure TDiskTreeView.SelectFirstFixedDisk();
var i: integer;
var s: string;
label endloop;
begin
    for i := 0 to self.Items.Count - 1 do begin
        s := self.items[i].Text;

        if (IsFixedDisk(self.Items[i])) then begin
            self.Selected := self.Items[i];
            self.Selected.Expanded := true;
            self.Select(self.Items[i],  [ssLeft]);
            //self.Expand(self.Items[i]);
            self.Repaint;
            //self.Click;
            goto endloop;
        end;
    end;

endloop:

end;


function TDiskTreeView.GetFullPathSelected(): string;
begin
    if (self.Selected <> nil) then begin
        result := getfullpathof(self.Selected);
    end else begin
        result := '';
    end;
end;




function TDiskTreeView.IsAnyFolder(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = node_folder)or( p.NodeType = node_folder_network);
end;
function TDiskTreeView.IsFolderNormal(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = node_folder);
end;

function TDiskTreeView.IsFolderNetwork(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = node_folder_network);
end;

function TDiskTreeView.IsRoot(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType and Node_Is_Root) > 0;
end;
function TDiskTreeView.IsFixedDisk(Node: TTreeNode): boolean;
var p: PTreeNodeType;
begin
    p := node.data;
    result := (p.NodeType = Node_Fixed_Drive);
end;


procedure TDiskTreeView.SelectFirstNode;
begin
    self.Items[0].Selected := true;
    self.items[0].MakeVisible;
end;

end.
