unit UnitColumnManager;

{
    Purpose:
        Abstract the TListItem method of creating/setting columns to allow
        for column positions to be moved on the fly.

    Notes:
        - The "TAG" field shows which position a column USED to be in
        - the TColumnType stores the original position this reason
        - Data rows act as if the positions have never changed

    Notes2:
        This code is a mess. Most of the garbage trying to releate a changing
        index to a column (when the column data is not fully populated on a click - thx Delphi)
        could have been avoided by using a StringList full of objects from the
        very beginning.


    Notes3:
        The .TAG of each column object (in CustomColumns) the  stores the type and/or original position
}

{///////////////}
{//}interface{//}
{///////////////}

uses Windows, SysUtils, StrUtils, ComCtrls, Dialogs;


type TColumnType = (
    COL_TYPE_NAME=0,
    COL_TYPE_NEWNAME=1,
    COL_TYPE_SIZE=2,
    COL_TYPE_TYPE=3,
    COL_TYPE_DATE=4,
    COL_TYPE_DATECREATED=5,
    COL_TYPE_CUSTOM=6
);


procedure SaveCustomColumns;
procedure LoadCustomColumns;

function CustomColumnCount : integer;
function GetCustomColumnData(i : integer) : string; overload;
function GetCustomColumnData(col : TListColumn) : string; overload;
function GetCustomColumnIndex(col : TListColumn) : integer;

procedure NewCustomColumn(Title, Data : string; coltype : TColumnType = COL_TYPE_NAME);
procedure DeleteCustomColumn(Title : string);
function GetCustomColumnType(title : string) : TColumnType; overload;
function GetCustomColumnType(col : TListColumn) : TColumnType; overload;

function GetNewListItem : TListItem;
procedure SetListView(value : TListView);

procedure SetColumnSize(OriginalIndex : integer; size : integer); overload;
procedure SetColumnSize(title : string; size : integer); overload;
procedure SetColumnSize(col : TListColumn; size : integer); overload;

procedure DefineColumn(col : TListColumn; coltype : TColumnType);
function GetColumnType(col : TListColumn) : TColumnType;
function GetColumnOldIndex(col : TListColumn) : integer;

function GetName(row : integer) : string;
function GetSize(row : integer) : string;
function GetDate(row : integer) : string;
function GetDateCreated(row : integer) : string;
function GetType(row : integer) : string;
function GetNewName(row : integer) : string;
procedure setName(row : integer; data : string); overload;
procedure setSize(row : integer; data : string); overload;
procedure setDate(row : integer; data : string); overload;
procedure setDateCreated(row : integer; data : string); overload;
procedure setType(row : integer; data : string); overload;
procedure setNewName(row : integer; data : string); overload;

procedure setName(li : TListItem; data : string); overload;
procedure setSize(li : TListItem; data : string); overload;
procedure setDate(li : TListItem; data : string); overload;
procedure setDateCreated(li : TListItem; data : string); overload;
procedure setType(li : TListItem; data : string); overload;
procedure setNewName(li : TListItem; data : string); overload;

//procedure setTypeDir(li : TListItem);
//function IsFolder(row : integer) : boolean; overload;
//function IsFolder(li : TListItem) : boolean; overload;


procedure setDateColumnName(name : string);
procedure HandleColumnClick(Column : TListColumn);
function GetClickedColumnType : TColumnType;
function GetClickedColumnDesc : boolean;
function GetClickedColumn : TListColumn;

//function NameToNewIndex(s : string) : integer;
function NameToOriginalIndex(s : string) : integer;
//procedure MoveToIndex(name : string; index: integer);


//function GetColumnHeader(i : integer) : string;

procedure lvFileListCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer
  );

const COL_NAME = 'Name';
const COL_SIZE = 'Size';
const COL_DATE = 'Date';
const COL_TYPE = 'Type';
const COL_NEWNAME = 'New Name';
const COL_DATECREATED = 'Date Created';

{////////////////////}
{//}implementation{//}
{////////////////////}

uses UnitFrmMain, UnitFrmDummyShowPreview, Classes, Controls, UnitUtil, inifiles,
    Menus, UnitFrmCustomColumns, UnitFrmDummyPreview, WinAPI.CommCtrl;

const DIR_TYPE = '<DIR>';
const CUSTOMCOLUMNSFILE = 'customcolumns.ini';
const TOTALCUSTOMCOLUMNS = 'TotalCustomColumns';
var CustomColumns : TStringList;
var lv : TListView;                        // reference to parent listview
var ColumnDesc : array[0 .. 999] of boolean; // stores state for columns sort order
var columns : TStringList;
var ClickedColumn : TListColumn;           // last clicked on column
var ClickedTag : TColumnType;              // original position of last clicked

procedure SetIsDesc(oldindex : integer); forward;
function GetIndexByTag(tag : TColumnType) : integer; forward;
procedure SetDataByTag(tag : TColumnType; ListItemIndex : integer; data : string); forward;
function GetDataByTag(tag : TColumnType; ListItemIndex : integer) : string; forward;

type TColumnTypeWrapper = class(TObject)
    columnitem : TListColumn;
    columnType : TColumnType;
end;



const CUSTOMCOLUMN_NAME = 'CustomColumn';
procedure SaveCustomColumns;
var ini : TINIFile;
    i : integer;
    section : string;
    ctw : TColumnTypeWrapper;
begin
    ini := TIniFile.Create(UnitUtil.GetINIPathOnly +  CUSTOMCOLUMNSFILE);



    ini.WriteInteger('Start', TOTALCUSTOMCOLUMNS, CustomColumns.count);
    for I := 0 to CustomCOlumns.count - 1 do begin
        section := CUSTOMCOLUMN_NAME + IntToStr(i);

        ctw := TColumnTypeWrapper(CustomColumns.Objects[i]);
        ini.WriteString(section, 'Title', CustomColumns.Names[i]);
        ini.WriteString(section, 'Data', CustomColumns.ValueFromIndex[i]);
        ini.WriteInteger(section, 'Type', Integer(ctw.columnType));
    end;

    ini.UpdateFile;
    ini.free;
end;
procedure LoadCustomColumns;
var ini : TINIFile;
    i, cnt : integer;
    section : string;
    ct : TColumnType;
    Title, Data : string;
begin
    ini := TIniFile.Create(UnitUtil.GetINIPathOnly + CUSTOMCOLUMNSFILE);
    cnt := ini.ReadInteger('Start', TOTALCUSTOMCOLUMNS, 0);
    for I := 0 to cnt - 1 do begin
        section := CUSTOMCOLUMN_NAME + IntToStr(i);

        title := ini.ReadString(section, 'Title', '');
        Data := ini.ReadString(section, 'Data', '');
        ct := TColumnType(ini.ReadInteger(section, 'Type', 0));
        NewCustomColumn(title,data,ct);
    end;
    // no update needed, no writes
    ini.free;
end;

function GetColumnOldIndex(col : TListColumn) : integer;
begin
    result := columns.IndexOfObject(col);
end;

function GetClickedColumn : TListColumn;
begin
    result := ClickedColumn;
end;



function GetCustomColumnIndex(col : TListColumn) : integer;
var i : integer;
    ctw : TColumnTypeWrapper;
begin
    Result := -1;

    for i := 0 to CustomColumns.count - 1 do begin
        ctw := TColumnTypeWrapper( CustomColumns.Objects[i] );
        if col = ctw.columnitem then begin
            result := i;
            EXIT;
        end;
    end;
end;

function GetCustomColumnType(col : TListColumn) : TColumnType;
var i : integer;
    ctw : TColumnTypeWrapper;
begin
    result := COL_TYPE_NAME; // doesn't make sense, but shuts up the compiler warning

    for i := 0 to CustomColumns.count - 1 do begin
        ctw := TColumnTypeWrapper( CustomColumns.Objects[i] );
        if col = ctw.columnitem then begin
            result := ctw.columnType;
            EXIT;
        end;
    end;
end;
function GetCustomColumnType(title : string) : TColumnType;
begin
    result :=    TColumnTypeWrapper(
        CustomColumns.Objects[ CustomColumns.IndexOf(title)]
    ).columnType;
end;


function GetCustomColumnData(i : integer) : string;
begin
    result := CustomColumns.ValueFromIndex[i] ;
end;

function GetCustomColumnData(col : TListColumn) : string;
var i : integer;
    ctw : TColumnTypeWrapper;
begin
    for i := 0 to CustomColumns.count - 1 do begin
        ctw := TColumnTypeWrapper(CustomColumns.Objects[i]);
        if ctw.columnitem = col then begin
            result := CustomColumns.Names[i];
            EXIT;
        end;
    end;
end;


procedure DeleteCustomColumn(Title : string);
var ctw : TColumnTypeWrapper;
    i : integer;
begin
    i := CustomColumns.IndexOfName(title);
    if i >= 0 then begin
        ctw := TColumnTypeWrapper(CustomColumns.objects[i]);

        lv.Columns.Delete( ctw.columnitem.Index );
        CustomColumns.Delete(i);
        frmmain.CustomColumns1.Delete( frmmain.CustomColumns1.Find('Edit ' + title).MenuIndex  );

        frmmain.delete1.Delete( frmmain.Delete1.Find(title).MenuIndex  );
        frmmain.Delete1.Enabled := frmmain.Delete1.Count <> 0;

        ctw.Free;
        // delete old columns and the associated custom column data
    end;
end;

procedure NewCustomColumn(Title, Data : string; coltype : TColumnType = COL_TYPE_NAME);
var ctw : TColumnTypeWrapper;
    i : integer;
    mi : TMenuItem;
//    s : string;
begin
    //
    // don't detect duplicates, allow for edits
    //

    Title := Title;

    i := CustomColumns.IndexOfName(title);
    if i >= 0 then begin
        ctw := TColumnTypeWrapper(CustomColumns.objects[i]);

        lv.Columns.Delete( ctw.columnitem.Index );
        CustomColumns.Delete(i);

        // delete old columns and the associated custom column data
    end;
    mi := frmmain.CustomColumns1.Find( 'Edit ' + Title );
    if mi = nil then begin
        mi := frmmain.MainMenu1.CreateMenuItem;
        mi.Caption := 'Edit ' + title;
        mi.Hint := data;
        mi.OnClick := FrmCustomColumns.EditColumnMenuClick;
        frmmain.CustomColumns1.Add(mi);
        //frmmain.CustomColumns1.Add(mi);

        mi := frmmain.MainMenu1.CreateMenuItem;
        mi.Caption :=  title;
        mi.Hint := data;
        mi.OnClick := FrmCustomColumns.DeleteColumnMenuClick;
        frmmain.Delete1.Add(mi);
        frmmain.Delete1.Enabled := true;
    end else begin
        mi.Hint := data;
    end;


    CustomColumns.Values[title] := Data;
    ctw := TColumnTypeWrapper.Create;
    ctw.columnType := coltype;
    CustomColumns.Objects[ CustomColumns.IndexOfName(title)] := ctw;




    ctw.columnitem := lv.Columns.Add;
    ctw.columnitem.Caption := title;
    ctw.columnitem.Tag := CustomColumns.IndexOfName(title) * -1;
    if ColType = COL_TYPE_SIZE then begin
        ctw.columnitem.Alignment := taRightJustify;
    end;

    DefineColumn(ctw.columnitem, COL_TYPE_CUSTOM);
end;



function CustomColumnCount : integer;
begin
    result := CustomColumns.Count;
end;

{
NOTE: This is now unused for the OWNERDRAWN filelist
}
procedure lvFileListCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer
  );
var c1, c2 : double;
    d1, d2 : tdate;
    s1, s2 : string;
    IgnoreSortDirection : boolean;

    a1, a2 : array[0.. Windows.MAX_PATH] of integer;

    function GetFileFlag(li : TListItem) : string;
    begin
        result := '1';
        if FrmDummyShowPreview.IsFolder(li) then result := '0';
    end;

    procedure StringToAlphaArray(s : string; var a : array of integer);
    var i,j  : integer;
        s2 : string;

        function IsDigit(c: Char): boolean;
        begin
            result := CharInSet(C, ['0'..'9']);
        end;
    begin
        for i := low(a) to high(a) do begin
            a[i] := -1;
        end;
        j := 0;
        i := 1;
        s2 := '';

        while (i <= length(s)) do begin

            while (i <= length(s)) and isdigit(s[i])  do begin
                s2 := s2 + s[i];
                inc(i);
            end;

            if (s2 = '') then begin
                a[j] := ord(s[i]);
                inc(i);
            end else begin
                try
                    a[j] :=  StrToInt(s2); 
                    s2 := '';
                except
                    // the numbers in a file can easily max out
                    a[j] := MaxInt;
                    s2 := '';
                end;
            end;
            inc(j);
        end;
    end;

    function AlphaCompare(a1, a2 : array of integer) : integer;
        function IntCompare(c1, c2 : integer) : integer;
        begin
            result := 0;
            if c1 <> c2 then begin
                if c1 < c2 then begin
                    result := -1;
                end else begin
                    result := 1;
                end;
            end;
        end;
    var i : integer;
    begin
        i := 0;
        repeat
            result := IntCompare(a1[i], a2[i]);
            if result <> 0 then EXIT;

            inc(i);
        until ((a1[i] = -1) or (a2[i] = -1));
        result := IntCompare(a1[i], a2[i]);
    end;
var  clicked : TColumnType;
    i : integer;
begin
    IgnoreSortDirection := false;

    // folders flags are less than file flags, and appear first
    
    if (GetFileFlag(Item1) <> GetFileFlag(Item2)) then begin
        // a folder always comes before a file
        s1 := GetFileFlag(Item1);
        s2 := GetFileFlag(Item2);
        compare := StrComp(Pchar(s1),
                    PChar(s2));
        IgnoreSortDirection := true;
    end else begin

        clicked := UnitColumnManager.GetClickedColumnType;

        {if clicked = COL_TYPE_CUSTOM then begin

        end;}

        case clicked of
        COL_TYPE_NAME:
            begin
                // name
                // show folders before files (both sorted)
                // sort by folder path, filename second


                s1 := ExtractFilepath( lowercase(item1.caption) );
                s2 := ExtractFilepath( lowercase(item2.caption) );
                if (s1 = s2) then begin
                    s1 := ExtractFilename( lowercase(item1.caption) );
                    s2 := ExtractFilename( lowercase(item2.caption) );
                    if (item1.ImageIndex = 6) then s1 := uppercase(s1);
                    if (item2.ImageIndex = 6) then s2 := uppercase(s2);
                    //compare := StrComp(pchar(s1), pchar(s2));
                end else begin
                    //compare := strcomp(pchar(s1), pchar(s2));
                end;



                if (ExtractFileExt(s1)= ExtractFileExt(s2)) then begin
                    // chop off the extensions
                    i := length(ExtractFileExt(s1));

                    s1 := copy(s1,1,length(s1) - i);
                    s2 := copy(s2,1,length(s2) - i);
                end;
                StringToAlphaArray(s1, a1);
                StringToAlphaArray(s2, a2);

                compare := AlphaCompare(a1, a2);
            end;

        COL_TYPE_SIZE:
            begin
                // size
                c1 := strtofloat(
                    UnitUtil.StripNumberFormat(
                         UnitColumnManager.GetSize(item1.Index)
                         )
                    );
                c2 := strtofloat(
                    UnitUtil.StripNumberFormat(
                         UnitColumnManager.GetSize(item2.Index)
                         )
                    );

                //c2 := strtofloat( self.GetDataByColumn(COL_SIZE, item2.Index));
                if (item1.ImageIndex = 6) then s1 := uppercase(s1);
                if (item2.ImageIndex = 6) then s2 := uppercase(s2);
                if (c1 < c2) then begin
                    compare := -1;
                end else if (c1 = c2) then begin
                    compare := 0;
                end else begin
                    compare := 1;
                end;
            end;
        COL_TYPE_TYPE:
            begin
                //type
                compare := StrComp(PChar(UnitColumnManager.GetType(item1.Index)),
                                   PChar(UnitColumnManager.GetType(item2.Index)) );
            end;
        COL_TYPE_DATE
        ,COL_TYPE_DATECREATED: begin
            s1 := UnitColumnManager.GetDate(item1.Index);
            if (clicked = COL_TYPE_DATECREATED) then begin
                s1 := UnitColumnManager.GetDateCreated(item1.Index);
            end;

            if (s1[1] = '[') then begin
                s1 := copy(s1,2,length(s1) - 2);
            end;

            s2 := UnitColumnManager.GetDate(item2.Index);
            if (clicked = COL_TYPE_DATECREATED) then begin
                s2 := UnitColumnManager.GetDateCreated(item2.Index);
            end;
            if (s2[1] = '[') then begin
                s2 := copy(s2,2,length(s2) - 2);
            end;

            d1 := strtodatetime(s1);
            d2 := strtodatetime(s2);

            if (d1 < d2) then begin
                compare := -1;
            end else if (d1 = d2) then begin
                compare := 0;
            end else begin
                compare := 1;
            end;

        end;
        end;

        if (clicked = COL_TYPE_NAME) then begin
        end else if (clicked = COL_TYPE_SIZE) then begin
        end else if (clicked  = COL_TYPE_TYPE) then begin
        end else if (clicked = COL_TYPE_DATE) or (clicked = COL_TYPE_DATECREATED) then begin
        end;
    end;

    if not IgnoreSortDirection then begin
        if UnitColumnManager.GetClickedColumnDesc  then begin
            compare := compare * -1;
        end;
    end;


end;

procedure DefineColumn(col : TListColumn; coltype : TColumnType);
begin
    if not assigned(columns) then columns := TStringList.Create;

    columns.AddObject(IntToStr(Integer(coltype)), col);
end;
function GetColumnType(col : TListColumn) : TColumnType;
var j : integer;
    s : string;
begin
    j := columns.IndexOfObject(col);
    s := columns.Strings[j];
    result := TColumnType(StrToInt(s));
end;


function GetIsDesc(index : integer) : boolean;
begin
    result := ColumnDesc[index];
end;

procedure SetListView(value : TListView);
    function GetIndexByName(ColName : string) : integer;
    var i : integer;
    begin
        result := 0;
        for i := 0 to (lv.Columns.Count - 1) do begin
            if lv.Columns.items[i].Caption = ColName then begin
                result := i;
                exit;
            end;
        end;
    end;
begin
    //
    // Set the default selected column and tag each column it's original position
    //
    lv := value;
    ClickedColumn := lv.columns[0];

    lv.Columns[Integer(COL_TYPE_NAME)].tag := Integer(COL_TYPE_NAME);
    lv.Columns[Integer(COL_TYPE_SIZE)].tag := Integer(COL_TYPE_SIZE);
    lv.Columns[Integer(COL_TYPE_DATE)].tag := Integer(COL_TYPE_DATE);
    lv.Columns[Integer(COL_TYPE_TYPE)].tag := Integer(COL_TYPE_TYPE);
    lv.Columns[Integer(COL_TYPE_NEWNAME)].tag := Integer(COL_TYPE_NEWNAME);
    lv.Columns[Integer(COL_TYPE_DATECREATED)].tag := Integer(COL_TYPE_DATECREATED);

end;

function GetNewListItem : TListItem;
var i : integer;
begin
    // create a new blank item
    // note: the "caption" counts as one column

    //result := TlistItem.Create(lv.items);
    result := lv.Items.Add;
    result.Selected := false;

    result.Caption := '';
    for i := 0 to (lv.Columns.Count - 2) do begin
           result.SubItems.Add('');
    end;

end;
function GetName(row : integer) : string;
begin
    result := GetDataByTag(COL_TYPE_NAME, row);
end;

function GetSize(row : integer) : string;
begin
    result := GetDataByTag(COL_TYPE_SIZE, row);
end;
function GetDate(row : integer) : string;
begin
    result := GetDataByTag(COL_TYPE_DATE, row);
end;

function GetDateCreated(row : integer) : string;
begin
    result := GetDataByTag(COL_TYPE_DATECREATED, row);
end;

function GetType(row : integer) : string;
begin
    result := GetDataByTag(COL_TYPE_TYPE, row);
end;
function GetNewName(row : integer) : string;
var s : string;
begin
    s := GetName(row);
    s := ExtractFilePath(s);
    if s = '' then begin
        result := GetDataByTag(COL_TYPE_NEWNAME, row);
    end else begin
        result := IncludeTrailingPathDelimiter(s) + GetDataByTag(COL_TYPE_NEWNAME, row);
    end;
end;
procedure setName(row : integer; data : string);
begin
    SetDataByTag(COL_TYPE_NAME, row, data);
end;
procedure setSize(row : integer; data : string);
begin
    SetDataByTag(COL_TYPE_SIZE, row, data);
end;
procedure setDate(row : integer; data : string);
begin
    SetDataByTAg(COL_TYPE_DATE, row, data);
end;

procedure setDateCreated(row : integer; data : string);
begin
    SetDataByTAg(COL_TYPE_DATECREATED, row, data);
end;


procedure setType(row : integer; data : string);
begin
    SetDataByTag(COL_TYPE_TYPE, row, data);
end;
procedure setNewName(row : integer; data : string);
begin
    SetDataByTag(COL_TYPE_NEWNAME, row, data);
end;


procedure setName(li : TListItem; data : string);
begin
    SetDataByTag(COL_TYPE_NAME, li.Index, data);
end;
procedure setSize(li : TListItem; data : string);
begin
    SetDataByTag(COL_TYPE_SIZE, li.Index, data);
end;
procedure setDate(li : TListItem; data : string);
begin
    SetDataByTag(COL_TYPE_DATE, li.Index, data);
end;
procedure setDateCreated(li : TListItem; data : string);
begin
    SetDataByTag(COL_TYPE_DATECREATED, li.Index, data);
end;

procedure setType(li : TListItem; data : string);
begin
    SetDataByTag(COL_TYPE_TYPE, li.Index, data);
end;
procedure setNewName(li : TListItem; data : string);
begin
    SetDataByTag(COL_TYPE_NEWNAME, li.Index, data);
end;



function OriginalIndexToName(i : integer) : string;
begin
    result := TListColumn(columns.objects[i]).Caption;
    {
    case i of
    Integer(COL_TYPE_NAME) : result := COL_NAME;
    Integer(COL_TYPE_NEWNAME) : result := COL_NEWNAME;
    Integer(COL_TYPE_SIZE) : result := COL_SIZE;
    Integer(COL_TYPE_TYPE) : result := COL_TYPE;
    Integer(COL_TYPE_DATE) :
        result := COL_DATE;
    integer(COL_TYPE_DATECREATED) : result := COL_DATECREATED;
    else
        if i<0 then begin
            i := i * -1;
            result := CustomColumns.Names[i];
        end;
        if i>5 then begin
            result := CustomColumns.Names[i-6];
        end;
    end;}
end;

function NameToNewIndex(s : string) : integer;
var i : integer;
begin
    // TODO: Investigate the problem between old/new integers positions
    result := -1;
    for i := 0 to lv.columns.count -1 do begin
        if ContainsTerm(s, lv.columns[i].Caption) then begin
            result := i;
            BREAK;
        end;
    end;
end;
function NameToOriginalIndex(s : string) : integer;
var i : integer;
    li : TListColumn;
begin
    result := -1;

    // TODO: Investigate the problem between old/new integers positions
    for i := 0 to columns.Count - 1 do begin
        li := TListColumn(Columns.objects[i]);
        if s = Li.caption then begin
            result := i;
            EXIT;
        end;
    end;
end;

procedure SetColumnSize(col : TListColumn; size : integer);
begin
   lv.Columns[ col.Index ].Width := size;
end;
procedure SetColumnSize(title : string; size : integer);
var
    i : integer;
begin
    i := NameToNewIndex(title);

    lv.Columns[i].Width := size;
end;
procedure SetColumnSize(OriginalIndex : integer; size : integer);
var name : string;
    i : integer;
begin

    name := OriginalIndexToName(OriginalIndex);
    i := NameToNewIndex(name);

    lv.Columns[i].Width := size;
end;

{
procedure MoveToIndex(name : string; index: integer);
var i : integer;
begin
    i := NameToNewIndex(name);

    lv.Columns[i].Index := index;
end;
}
procedure setDateColumnName(name : string);
begin
    lv.Columns[ NameToNewIndex(COL_DATE)].Caption  := name;
end;


function GetClickedColumnDesc : boolean;
begin
    result := ColumnDesc[ GetColumnOldIndex(ClickedColumn) ];
end;

function GetClickedColumnType : TColumnType;
begin
    //
    result := ClickedTag;
end;

procedure HandleColumnClick(Column : TListColumn);
var col : TListColumn;
    index : integer;
    s : string;
begin
    //
    // The Column.Index is the original index, not the current index
    // everything else of Column is wrong (caption, tag, etc)
    //

    ClickedTag := GetColumnType(column);
    index := GetColumnOldIndex(column);
    s := OriginalIndexToName(index);
    col := column;



    if (ClickedTag <> COL_TYPE_NEWNAME) then begin
        // clear previous column's icon
        if (ClickedColumn <> nil) then begin
            //ClickedColumn.ImageIndex := 10;
            s := clickedcolumn.Caption;
            s := stringreplace(s,'[+] ', '', []);
            s := stringreplace(s,'[-] ', '', []);

            clickedcolumn.Caption := s;
        end;

        // save clicked on column and set the icon for the sort order
        // and trigger a sort
        ClickedColumn := col;
        SetIsDesc(index);

        if lv.OwnerData then begin
            FrmDummyShowPreview.SortData ;
        end else begin
            lv.AlphaSort;
        end;

        FrmDummyAutoPreview.InformOfKeypress;
        FrmDummyShowPreview.ClearAllSelected;
    end;
end;

procedure SetIsDesc(oldindex : integer);
var i : integer;
    s : string;
begin
    i := oldindex;
    ColumnDesc[i] := ColumnDesc[i] xor true;
    ClickedColumn.ImageIndex := -1;

    s := clickedcolumn.caption;

    s := stringreplace(s,'[-] ', '',[]);
    s := stringreplace(s,'[+] ', '',[]);

    if (ColumnDesc[i]) then begin
        clickedcolumn.Caption := '[-] ' + s;
    end else begin
        clickedcolumn.Caption := '[+] ' + s;
    end;
end;



function GetIndexByTag(tag : TColumnType) : integer;
var
    ColumnOrder: array of Integer;
    i : integer;
begin
    result := -1;

    SetLength(ColumnOrder, lv.Columns.Count);
    ListView_GetColumnOrderArray(
        lv.Handle, lv.Columns.Count, PInteger(ColumnOrder)
    );
    for i := 0 to lv.Columns.Count - 1 do begin
        if ColumnOrder[i] = Integer(tag) then begin
            result := i;
            EXIT;
        end;
    end;
end;

function GetDataByTag(tag : TColumnType; ListItemIndex : integer) : string;
var i : integer;
begin
    i := GetIndexByTag(tag);

    if (i=0) then begin
        result := lv.items[ListItemIndex].caption;
    end else begin
        result := lv.items[ListItemIndex].subitems[i-1];
    end;
end;
procedure SetDataByTag(tag : TColumnType; ListItemIndex : integer; data : string);
var i : integer;
    li : Tlistitem;
begin
    i := GetIndexByTag(tag);
    li := lv.items[listitemindex];
    if (i=0) then begin
        li.caption := data;
    end else begin
        li.subitems[i-1] := data;
    end;

    FrmDummyShowPreview.UpdateFileData(li);
end;




initialization


begin
    CustomColumns := TStringList.Create;
    CustomColumns.Duplicates := dupIgnore;
end;


end.
