unit UnitFileIcon;
{
    Purpose:
        Given a file name, get it's associated or imbedded icon.
        Cache ICONs for quick lookups.

    Notes:
        Non-EXE files are hashed by file extension, since it's
        associated EXE's icon it used.

        EXE files can have embedding ICON so they're not
        hashed at all.

        Column Icons for the filelist screw up durring a thumbnail list

    Updates:
        Some folder icons not showing up
        .LNK files not excluded from cache
}
interface
uses Windows,  ComCtrls,   SysUtils, Variants, Classes, Graphics,
    Controls, Forms, ImgList;



function ExtractIconCached(ILIcons : TImageList; filename : string) : integer;
procedure ExtractAndSetIconSmall(ILIcons : TImageList; li : TListItem; filename : string); overload;
procedure ExtractAndSetIconSmall(ILIcons : TImageList; tn : TTreeNode; filename : string); overload;
procedure ExtractIconNoCache(ILIcons : TImageList; filename : string; index : integer = -1; AlternateExtract : boolean = false);
procedure ClearCache;
procedure ToggleThumbnailMode(value:boolean);

function GetSmallShortcutIcon(FileName: String): HICON;


implementation


uses shellAPI, shlobj, strutils,  INIFiles, UnitFrmThumbPreview,
  UnitFrmDummyShowPreview, UnitFrmDummyThumbExtract, UnitUtil, pngimage, gifimg;
var IconCache : THashedStringList;
    Icon : TIcon;
    ShowThumbnails : boolean = false;

procedure ToggleThumbnailMode(value:boolean);
begin
    ShowThumbnails := value;
    if value then begin
        FrmDummyShowPreview.ILDynamic.Width := 64;
        FrmDummyShowPreview.ILDynamic.Height := 64;
        FrmDummyThumbExtract.Image1.Width := FrmDummyShowPreview.ILDynamic.Width;
        FrmDummyThumbExtract.Image1.Height := FrmDummyShowPreview.ILDynamic.Height;
        FrmDummyThumbExtract.Image1.Picture.Bitmap.Width := FrmDummyShowPreview.ILDynamic.Width;
        FrmDummyThumbExtract.Image1.Picture.Bitmap.Height := FrmDummyShowPreview.ILDynamic.Height;
    end else begin
        FrmDummyShowPreview.ILDynamic.Width := 16;
        FrmDummyShowPreview.ILDynamic.Height := 16;
    end;
end;

procedure ClearCache;
begin
    IconCache.clear;
end;

procedure ExtractAndSetIconEXE(ILIcons : TImageList; li : TListItem; keyvalue : string);
var sEXE : string;
    IconIndex : word;
    //icon : TIcon;
    h : THandle;
    s : string;
    i : integer;


begin
    // find ICON of EXE item
    // add default icon if can't find the file
    //
    sEXE := '';
    sEXE := keyvalue;
    If FileExists(sEXE) or LongerDirectoryExists(sEXE) then begin
        IconIndex := 0;

        h := ShellAPI.ExtractAssociatedIcon(
            hInstance, PChar(sEXE), IconIndex);
        s := IconCache.Values[IntToSTr(h)];
        if (s <> '') then begin
            li.ImageIndex := StrToInt(s);
        end else begin
            icon.Handle := h;
            //icon.Handle := ShellAPI.ExtractAssociatedIcon(hInstance, PChar(sEXE), IconIndex);
            if (icon.Handle <> 0) then begin
                i := ILIcons.AddIcon(icon);
                li.ImageIndex := i;
                IconCache.Values[IntToSTr(h)] := IntToStr(i);
            end;
        end;
    end else begin
        li.ImageIndex := 0;
    end;
end;

function GetSmallShortcutIcon(FileName: String): HICON;
var
    FileInfo: TSHFileInfo;
    si : IExtractIcon;
    windex : word;
begin

    if length(filename) > 1 then begin
        if EndsText(':', filename) then begin
            filename := IncludeTrailingPathDelimiter(Filename);
        end;

    end;
    //filename := ExcludeTrailingBackslash(Filename);

    SHGetFileInfo(PChar(FileName), 0, FileInfo, SizeOf(FileInfo),
    {SHGFI_ICON or SHGFI_SYSICONINDEX or SHGFI_SMALLICON
        see
        http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnwui/html/msdn_icons.asp
    }


    // for testing Vista icon support

        //SHGFI_ICON or SHGFI_SHELLICONSIZE or SHGFI_SMALLICON
        SHGFI_ICON  or SHGFI_SHELLICONSIZE or SHGFI_SMALLICON
        );

    Result := FileInfo.hIcon;

    if result = 0 then
    result := ExtractAssociatedIcon(hinstance, pchar(filename), windex);

end;
function GetLargeShortcutIcon(FileName: String): HICON;
var
    FileInfo: TSHFileInfo;
begin
    SHGetFileInfo(PChar(FileName), 0, FileInfo, SizeOf(FileInfo), SHGFI_ICON
        or SHGFI_SYSICONINDEX or SHGFI_LARGEICON);
    Result := FileInfo.hIcon;
end;
procedure ExtractIconNoCache(ILIcons : TImageList; filename : string; index : integer = -1; AlternateExtract : boolean = false);
var h : THandle;
    icon : TIcon;
    i : integer;
    dummyword : word;
begin
    if filename = '' then EXIT;
    
    icon := TIcon.Create;
    i := ILIcons.Count;
    if AlternateExtract then begin
        h := ExtractAssociatedIcon(HInstance, pchar(filename),dummyword)
    end else begin
        h := GetSmallShortcutIcon(filename);
    end;
    if h = 0 then EXIT;

    icon.Handle := h;
    if index = -1 then begin
        i := ILIcons.AddIcon(icon);
    end else begin
        ILIcons.Delete(index);
        ILIcons.InsertIcon(index, icon);
    end;

    icon.Free;
end;
function ExtractIconCached(ILIcons : TImageList; filename : string) : integer;
var h : THandle;
    ext, s : string;
    i : integer;
    IsThumb : boolean;
    bit : TBitmap;
    bit2 : TBitmap;
    r : TRect;
begin
    ext := lowercase(ExtractFileExt(filename));
    if (ext = '') then ext := filename;

    IsThumb := false;
    if ShowThumbnails then begin
        if StringIn(ext, ['.tiff','.tif','.gif','.png','.jpg','.jpeg','.bmp','.ico','.wmf','.emf']) then begin
            ext := lowercase(ExtractFileName(filename));
            IsThumb := true;
        end;
    end;

    // Extension Found
    // New extension (don't cache EXE's)
    // Fix: treat all folders separate for custom icons

    result := 0;
    s := IconCache.Values[ext];

    if (s <> '') then begin
        // duplicate ignored
        result := StrToInt(s);
    end else begin
        IconCache.add(ext);
        i := ILIcons.Count;
        if (not ShowThumbnails) then begin
            h := GetSmallShortcutIcon(filename);
        end else begin
            h := GetLargeShortcutIcon(filename);
        end;
        icon.Handle := h;
        if (icon.Handle <> 0) or (IsThumb) then begin
            if (not ShowThumbnails) then begin
                i := ILIcons.AddIcon(icon);
            end else begin
                if (IsThumb) then begin
                    i := ILIcons.Add( FrmDummyThumbExtract.GetThumbBitmap(filename), nil);
                end else begin
                    try
                        bit := TBitmap.Create;
                        bit.Width := icon.Width;
                        bit.Height := icon.Height;
                        bit.Canvas.Draw(0,0, icon);
                        bit2 := TBitmap.Create;
                        bit2.Width := ILIcons.Width+1;
                        bit2.Height := ILIcons.Height+1;
                        r.Top := 0;
                        r.Right := 0;
                        r.Left := 64;
                        r.Bottom := 64;
                        bit2.Canvas.StretchDraw(r, bit);

                        i := ILIcons.Add(bit2, nil);
                    except
                        begin
                        end;
                    end;
                end;
            end;
            result := i;

            // exe and lnk files will have individual icons
            if (ext <> '.exe') and (ext <> '.lnk') then begin
                IconCache.values[ext] := IntToStr(i);
            end;
        end;
    end;
end;

procedure ExtractAndSetIconSmall(ILIcons : TImageList; li : TListItem; filename : string);
begin
    li.ImageIndex := ExtractIconCached(ILIcons, filename);
end;
procedure ExtractAndSetIconSmall(ILIcons : TImageList; tn : TTreeNode; filename : string);
begin
    tn.ImageIndex := ExtractIconCached(ILIcons, filename);
end;

initialization
begin
    IconCache := THashedStringList.Create;
    IconCache.Sorted := false;
    IconCache.CaseSensitive := false;
    IconCache.Duplicates := dupIgnore;

    Icon := TIcon.Create;
end;

finalization
begin
    IconCache.free;
    Icon.Free;
end;
end.
