unit UnitFileTags;
{
    Purpose:
        Given a filename, JPEG JFIF & EXIF, MP3 Id3V1 tags
        are read from a file

    Updates:
        fixed problem with TrimTrailingNulls and an empty (all spaces) field 

TODO: research UserComment

}
{///////////////}
{//}INTERFACE{//}
{///////////////}
uses INIFiles {for THashStringList}, Generics.collections;
const
    // JFIF tags
    JFIF_PRECISION = 'JFIF_PRECISION';
    JFIF_WIDTH = 'JFIF_WIDTH';
    JFIF_HEIGHT = 'JFIF_HEIGHT';


//
// convenience class to combine jpg/mp3/video tags into a single
// object
//
type TTags = class(TObject)
    public
    class function ReadTags(filename : string; tags : THashedStringList) : boolean;
    class function GetTagFromName(varName : string; var tag : string) : boolean;
end;

type MP3_Enum = (
    I_AL,
    I_AR,
    I_C,
    I_G,
    I_TI,
    I_TR,
    I_Y,
    // MP3 header info
    M_B,
    M_S,
    M_C
);
// EXIF tags

{$region 'original EXIF enum'}
type EXIF_Enum = (
    X_IW ,            // ImageWidth
    X_IL ,            // ImageLength
    X_BPS ,           // BitsPerSample
    X_C ,             // Compression
    X_PI ,            // PhotometricInterpretation
    X_FO ,            // FillOrder
    X_DN ,            // DocumentName
    X_ID ,            // ImageDescription
    X_MA ,             // Make
    X_MO ,             // Model
    X_SO ,            // StripOffsets
    X_O ,             // Orientation
    X_SPP ,           // SamplesPerPixel
    X_RPS ,           // RowsPerStrip
    X_SBC ,           // StripByteCounts
    X_XR ,            // XResolution
    X_YR ,            // YResolution
    X_PC ,            // PlanarConfiguration
    X_RU ,            // ResolutionUnit
    X_TF ,            // TransferFunction
    X_S ,             // Software
    X_DT ,            // DateTime
    X_A ,             // Artist
    X_WP ,            // WhitePoint
    X_PCr ,            // PrimaryChromaticities
    X_TR ,            // TransferRange
    X_JPEGP ,         // JPEGProc
    X_TO ,            // ThumbnailOffset
    X_TL ,            // ThumbnailLength
    X_YCCC ,          // YCbCrCoefficients
    X_YCCSS ,         // YCbCrSubSampling
    X_YCCP ,          // YCbCrPositioning
    X_RBW ,           // ReferenceBlackWhite
    X_SOf ,            // StripOffset
    X_GPSI ,         // GPSInfo
    X_EO ,           // ExifOffset
    X_II ,           // InteroperabilityIndex
    X_ET ,           // ExposureTime
    X_FN ,           // FNumber
    X_EP ,           // ExposureProgram
    X_ISOSR ,        // ISOSpeedRatings
    X_OEC ,          // OECF
    X_EV ,           // ExifVersion
    X_DTO ,          // DateTimeOriginal
    X_DTD ,          // DateTimeDigitized
    X_CC ,           // ComponentsConfiguration
    X_CBPP ,         // CompressedBitsPerPixel
    X_SSV ,          // ShutterSpeedValue
    X_AV ,           // ApertureValue
    X_BV ,           // BrightnessValue
    X_EBV ,          // ExposureBiasValue
    X_MAV ,          // MaxApertureValue
    X_SD ,           // SubjectDistance
    X_MM ,           // MeteringMode
    X_LS ,           // LightSource
    X_F ,            // Flash
    X_FL ,           // FocalLength
    X_MN ,           // MakerNote
    X_UC ,           // UserComment
    X_FPV ,          // FlashPixVersion
    X_CS ,           // ColorSpace
    X_EIW ,          // ExifImageWidth
    X_EIL ,          // ExifImageLength
    X_IO ,           // InteroperabilityOffset
    X_FE ,           // FlashEnergy
    X_SFR ,          // SpatialFrequencyResponse
    X_FPXR ,         // FocalPlaneXResolution
    X_FPYR ,         // FocalPlaneYResolution
    X_FPRU ,         // FocalPlaneResolutionUnit
    X_SL ,           // SubjectLocation
    X_EI ,           // ExposureIndex
    X_SM ,           // SensingMethod
    X_FS ,           // FileSource
    X_ST ,           // SceneType
    X_CFARPD ,       // CFARepeatPatternDim
    X_CFAP ,         // CFAPattern
    X_BL ,           // BatteryLevel
    X_CO ,            // Copyright
    X_IPTCNA ,       // IPTC/NAA
    X_ICP ,          // InterColorProfile
    X_SS ,           // SpectralSensitivity
    X_SST ,          // SubSecTime
    X_SSTO ,         // SubSecTimeOriginal
    X_SSTD ,         // SubSecTimeDigitized
    X_CR ,           // CustomRendered
    X_EM ,           // ExposureMode
    X_WB ,           // WhiteBalance
    X_DZR ,          // DigitalZoomRatio
    X_FL35 ,           // FocalLength35
    X_SCT ,          // SceneCaptureType
    X_GC ,           // GainControl
    X_CN ,            // Contrast
    X_SA ,            // Saturation
    X_SH ,            // Sharpness
    X_SDR,           // SubjectDistanceRange

    // fake tags for malformed dates

    X_DTOM,
    X_DTDM,
    X_DTM
);

{$endregion}
{$region 'EXIF file IDs'}
type EXIFEnumArray = array[low(EXIF_Enum)..high(EXIF_Enum)] of word;
const EXIF_FileID: EXIFEnumArray = (
    {X_IW =} $100,{X_IL =} $101,{X_BPS =} $102,{X_C =} $103,{X_PI =} $106,{X_FO =} $10A,
    {X_DN =} $10D,{X_ID =} $10E,{X_Ma =} $10F,{X_Mo =} $110,{X_SO =} $111,{X_O =} $112,
    {X_SPP =} $115,{X_RPS =} $116,{X_SBC =} $117,{X_XR =} $11A,{X_YR =} $11B,{X_PC =} $11C,
    {X_RU =} $128,{X_TF =} $12D,{X_S =}  $131,{X_DT =}  $132,{X_A =} $13B,{X_WP =} $13E,
    {X_PCr =} $13F,{X_TR =} $156,{X_JPEGP =} $200,{X_TO =} $201,{X_TL =} $202,{X_YCCC =} $211,
    {X_YCCSS =} $212,{X_YCCP =} $213,{X_RBW =} $214,{X_SOf =} $273,{X_GPSI =} $8825,{X_EO =} $8769,
    {X_II =} $0001 ,{X_ET =} $829A ,{X_FN =} $829D,{X_EP =} $8822,{X_ISOSR =} $8827 ,{X_OEC =} $8828,
    {X_EV =} $9000 ,{X_DTO =} $9003 ,{X_DTD =} $9004 ,{X_CC =} $9101,{X_CBPP =} $9102,{X_SSV =} $9201 ,
    {X_AV =} $9202 ,{X_BV =} $9203 ,{X_EBV =} $9204 ,{X_MAV =} $9205,{X_SD =} $9206,{X_MM =} $9207,
    {X_LS =} $9208,{X_F =} $9209,{X_FL =} $920A,{X_MN =} $927C,{X_UC =} $9286,{X_FPV =} $A000,
    {X_CS =} $A001,{X_EIW =} $A002,{X_EIL =} $A003,{X_IO =} $A005,{X_FE =} $A20B,{X_SFR =} $A20C,
    {X_FPXR =} $A20E,{X_FPYR =} $A20F,{X_FPRU =} $A210,{X_SL =} $A214,{X_EI =} $A215,{X_SM =} $A217,
    {X_FS =} $A300,{X_ST =} $A301,{X_CFARPD =} $828D,{X_CFAP =} $828E,{X_BL =} $828F,{X_Co =} $8298,
    {X_IPTCNA =} $83BB,{X_ICP =} $8773,{X_SS =} $8824,{X_SST =} $9290,{X_SSTO =} $9291,{X_SSTD =} $9292,
    {X_CR =} $A401,{X_EM =} $A402,{X_WB =} $A403,{X_DZR =} $A404,{X_FL35 =} $A405,{X_SCT =} $A406,
    {X_GC =} $A407,{X_Cn =} $A408,{X_Sa =} $A409,{X_Sh =} $A40A,{X_SDR =} $A40C,

    // malfored dates fake tags
    $C001, $C002, $C003
);
{$endregion}
{
const EXIF_EnumOrder = [
    X_IW, X_IL, X_BPS, X_C, X_PI, X_FO, X_DN, X_ID, X_MA, X_MO, X_SO,
    X_O,  X_SPP,  X_RPS,  X_SBC, X_XR, X_YR, X_PC, X_RU, X_TF, X_S, X_DT,
    X_A, X_WP, X_PCR, X_TR, X_JPEGP, X_TO, X_TL, X_YCCC, X_YCCSS, X_YCCP,
    X_RBW, X_SOf, X_GPSI,  X_EO,  X_II,  X_ET,  X_FN,  X_EP, X_ISOSR, X_OEC,
    X_EV, X_DTO, X_DTD, X_CC, X_CBPP, X_SSV, X_AV, X_BV, X_EBV, X_MAV,
    X_SD, X_MM, X_LS, X_F, X_FL, X_MN, X_UC, X_FPV, X_CS, X_EIW, X_EIL,
    X_IO, X_FE, X_SFR, X_FPXR, X_FPYR, X_FPRU, X_SL, X_EI, X_SM, X_FS,
    X_ST, X_CFARPD, X_CFAP, X_BL, X_Co, X_IPTCNA, X_ICP, X_SS, X_SST,
    X_SSTO, X_SSTD, X_CR, X_EM, X_WB, X_DZR, X_FL35, X_SCT, X_GC, X_CN,
    X_SA, X_SH, X_SDR
];
}
function GetTagName( abr : string ) : string; overload;
function GetTagName( x :EXIF_Enum ) : string; overload;
function GetTagAbbr( x : EXIF_ENum) : string;
function GetTagDesc( tag : word) : string;    overload;
function GetTagID( X : EXIF_Enum ) : word;
function GetTagEnum( abr : string ) : word;
function GetMP3TagName(abr : string) : string; overload;
function GetMP3TagName(m : MP3_Enum) : string; overload;
function GetJPGExifEnum(abr : string; var ex : Exif_Enum) : boolean;

function GetXMPValue(variablename : string) : string;

function ReadJpeg(filename : string; tags : THashedStringList; useAbbreviations : boolean = false) : string;
function ReadMP3(filename : string; tags : THashedStringList) : string;
function ReadMP3Full(filename : string; tags : THashedStringList) : string;

type TJPEGSegment = class(TObject)
    private
        LittleEndian : boolean;
        SegType : byte;
        SegSize : word;
        SegData : Pointer;
        function HI_LO(inw : word) : word; overload;
        function HI_LO(inl : integer) : integer; overload;
        function HI_LO(inl : cardinal) : cardinal; overload;
    public
        procedure GetHeader(var f : file);
        procedure GetSegment(var f : file);
        function GetType : byte;
        function GetSize : word;
        function GetDataByte(i : integer) : byte;
        function GetDataWord(i : integer) : word;
        function GetDataLong(i : integer) : integer;
        function GetDataLongWord(i : integer) : longword;

        function GetDataString(offset: integer; length : integer; skip8 : boolean = false): string;
        procedure SetLittleIndian(value : boolean);
        function GetTagDesc( tag : word ) : string; overload;
        function GetTagName( abr : string ) : string; overload;
        function GetTagName( x :EXIF_Enum ) : string; overload;
        function GetTagAbbr( x : EXIF_Enum) : string;
        function GetTagID( X : EXIF_Enum ) : word;
        function GetTagEnum( abr : string ) : word;
end;


{////////////////////}
{//}IMPLEMENTATION{//}
{////////////////////}

uses system.character, SysUtils, StrUtils, DateUtils, Dialogs, Classes, UnitUtil, Windows, xmldoc,
xmldom, XMLIntf, MSXML2_TLB, ComObj, TypInfo, System.Generics.Collections,
  UnitDebug, IOUtils, UnitVideo;

var NameToAbbr : THashedSTringList;
class function TTags.ReadTags(filename : string; tags : THashedStringList) : boolean;
var ext : string;
begin
    ext := lowercase( TPath.GetExtension(filename) );
    if (ext = '.mp3') then begin
        UnitFileTags.ReadMP3(filename, tags);
    end else if StringIn(ext, ['.jpg','.jpeg']) then begin
        UnitFileTags.ReadJpeg(filename, tags)
    end else if UnitVideo.FileSupported(filename) then begin
        UnitVideo.GetVideoTags(filename, tags)
    end;
    result := tags.count > 0;
end;
class function TTags.GetTagFromName(varName : string; var tag : string) : boolean;
var s : string;
begin
    // detect when a variable name is the same as the variable tag
    result := varname <> '';
    if varname[1]='?' then begin
        tag := varname;
        EXIT;
    end;

    s := NameToAbbr.Values[varname];
    result := s <> '';

    if result then begin
        tag := '?' + s + '?';
    end;
end;

{$region 'Constants and Stuff'}
const
    MSK_B1_VERSION = $18;
    MSK_B1_LAYER = $06;
    MSK_B1_CRC = $01;
    MSK_B2_BITRATE = $F0;
    MSK_B2_SAMPLE = $0C;
    MSK_B3_CHANNEL = $C0;

    {
    MP3_V1 = $18;
    MP3_V2 = $08;
    MP3_V3 = $00;
    MP3_Layer3 = $02;
    MP3_Layer2 = $04;
    MP3_Layer1 = $06;
    }

var
    MP3_Bitrates : array[0..1, 0..15] of word = (
        (0,32,40,48,56,64,80,96,112,128,160,192,224,256,320,0),
        (0, 8,16,24,32,40,48,56, 64, 80, 96,112,128,144,160,0)
    );
    MP3_Sample : array[0..3,0..2] of integer = (
        (32000, 16000, 8000),
        (0,0,0), {not use}
        (48000, 24000, 12000),
        (44100, 22050, 11025)
    );
type TMP3Segment = class(TObject)
    private
        header : array[0..3] of byte;

        version, layer, bitrate,
        sample, channel, filep : integer;
    public
        function IsHeader(h : array of byte) : boolean;
        function GetHeader(filename : string) : boolean;
end;


var
    //AbrHash : THashedStringList;
    AbrHash : TDictionary<string, word>;
    XMPvariables : TDictionary<string, string>;

CONST
    APP0 = $E0;
    APP14 = $E0;


    SOF0 = $C0;            {start of FRAME}
    SOF1 = $C1;
    SOF2 = $C2;             {processive}
    SOF9 = $C9;
    SOF10 = $CA;

    SOI  = $D8;            {start of image}
    EOI  = $D9;            {end   of image}
    SOS  = $DA;            {start of scan }
    APP1_EXIF = $E1;
    APP13_IPTC = $ED;


const MaxID3Genre=147;
var  ID3Genre: array[0..MaxID3Genre] of string = (
    'Blues', 'Classic Rock', 'Country', 'Dance', 'Disco', 'Funk', 'Grunge',
    'Hip-Hop', 'Jazz', 'Metal', 'New Age', 'Oldies', 'Other', 'Pop', 'R&B',
    'Rap', 'Reggae', 'Rock', 'Techno', 'Industrial', 'Alternative', 'Ska',
    'Death Metal', 'Pranks', 'Soundtrack', 'Euro-Techno', 'Ambient',
    'Trip-Hop', 'Vocal', 'Jazz+Funk', 'Fusion', 'Trance', 'Classical',
    'Instrumental', 'Acid', 'House', 'Game', 'Sound Clip', 'Gospel',
    'Noise', 'AlternRock', 'Bass', 'Soul', 'Punk', 'Space', 'Meditative',
    'Instrumental Pop', 'Instrumental Rock', 'Ethnic', 'Gothic',
    'Darkwave', 'Techno-Industrial', 'Electronic', 'Pop-Folk',
    'Eurodance', 'Dream', 'Southern Rock', 'Comedy', 'Cult', 'Gangsta',
    'Top 40', 'Christian Rap', 'Pop-Funk', 'Jungle', 'Native American',
    'Cabaret', 'New Wave', 'Psychadelic', 'Rave', 'Showtunes', 'Trailer',
    'Lo-Fi', 'Tribal', 'Acid Punk', 'Acid Jazz', 'Polka', 'Retro',
    'Musical', 'Rock & Roll', 'Hard Rock', 'Folk', 'Folk-Rock',
    'National Folk', 'Swing', 'Fast Fusion', 'Bebob', 'Latin', 'Revival',
    'Celtic', 'Bluegrass', 'Avantgarde', 'Gothic Rock', 'Progressive Rock',
    'Psychedelic Rock', 'Symphonic Rock', 'Slow Rock', 'Big Band',
    'Chorus', 'Easy Listening', 'Acoustic', 'Humour', 'Speech', 'Chanson',
    'Opera', 'Chamber Music', 'Sonata', 'Symphony', 'Booty Bass', 'Primus',
    'Porn Groove', 'Satire', 'Slow Jam', 'Club', 'Tango', 'Samba',
    'Folklore', 'Ballad', 'Power Ballad', 'Rhythmic Soul', 'Freestyle',
    'Duet', 'Punk Rock', 'Drum Solo', 'Acapella', 'Euro-House', 'Dance Hall',
    'Goa', 'Drum & Bass', 'Club-House', 'Hardcore', 'Terror', 'Indie',
    'BritPop', 'Negerpunk', 'Polsk Punk', 'Beat', 'Christian Gangsta Rap',
    'Heavy Metal', 'Black Metal', 'Crossover', 'Contemporary Christian',
    'Christian Rock', 'Merengue', 'Salsa', 'Trash Metal', 'Anime', 'Jpop',
    'Synthpop'
  );
{$ENDREGION}

function GetMP3TagName(abr : string) : string;
var m : MP3_Enum;
begin
    result := '';
    if abr='' then exit;

    abr := uppercase(abr);
    if (abr[1] = '?') then begin
        abr := midstr(abr, 2, length(abr)-2);
    end;

    m := MP3_Enum(GetEnumValue(TypeInfo(MP3_Enum),abr));

    result := GetMP3TagName(m);
end;
function GetMP3TagName(m : MP3_Enum) : string; overload;
begin
   case m of
        I_AL: result := 'Album';
        I_AR: result := 'Artist';
        I_C: result := 'Comment';
        I_G: result := 'Genre';
        I_TI: result := 'Title';
        I_TR: result := 'Track';
        I_Y: result := 'Year';
        M_B: result := 'Bitrate';
        M_S: result := 'Sample';
        M_C: result := 'Channels';
    end;
end;
function GetGenreName(b : byte) : string;
begin
    result := 'Unknown';
    if (b < MaxID3Genre) then begin
        result := ID3Genre[b];
    end;
end;
function GetJPGExifEnum(abr : string; var ex : Exif_Enum) : boolean;
begin
    result := false;
    if abr='' then exit;

    abr := uppercase(abr);
    if (abr[1] = '?') then begin
        abr := midstr(abr, 2, length(abr)-2);
    end;

    ex := EXIF_Enum( GetEnumValue(TypeInfo(EXIF_ENum), abr) );
    result := true;
end;


function ReadMP3(filename : string; tags : THashedStringList) : string;
type ID3V1 = packed record
     TAG : array[0..2] of ansichar;
     TITLE : array[0..29] of ansichar;
     ARTIST : array[0..29] of ansichar;
     ALBUM : array[0..29] of ansichar;
     YEAR : array[0..3] of ansichar;
     COMMENT : array[0..29] of ansichar;
     GENRE : byte;
end;
    // Remove trailing garbage from the strings
    function TrimTrailingNulls(s : string ) : string;
    begin
        if s <> '' then begin
            while (s[length(s)] = #0) or (s[length(s)] = ' ') do begin
                delete(s,length(s),1);
                if s = '' then break;
            end;
        end;
        result := s;
    end;
var f : file;
    buf : ID3V1;
    fs : Cardinal;
    s : string;
begin
    //read the last 128 bytes of the file
    assign(f, filename); filemode := fmOpenRead;
    reset(f,1);

    fs := FileSize(f);
    if fs < 128 then begin
    	close(f);
    	EXIT;
    end;

    seek(f, (fs-128));
    blockread(f,buf,128);
    close(f);

    // check for TAG - literally
    // strip the tags including the V1.1 Track number
    s := buf.tag[0] + buf.tag[1] + buf.tag[2];
    {$WARN IMPLICIT_STRING_CAST OFF}
    if (S = 'TAG') then begin
        tags.values[GetMP3TagName(I_Ti)] := TrimTrailingNulls(copy(pansichar(@buf.TITLE), 1, 30));
        tags.Values[GetMP3TagName(I_AR)] := TrimTrailingNulls(pansichar(copy(pansichar(@buf.ARTIST), 1, 30)));
        tags.Values[GetMP3TagName(I_AL)] := TrimTrailingNulls(pansichar(copy(pansichar(@buf.ALBUM), 1, 30)));
        tags.Values[GetMP3TagName(I_y)] := TrimTrailingNulls(pansichar(copy(pansichar(@buf.YEAR), 1, 4)));
        tags.Values[GetMP3TagName(I_C)] := TrimTrailingNulls(pansichar(@buf.COMMENT));

        if (Byte(buf.COMMENT[28]) = 0) then begin
            if (Byte(buf.COMMENT[29]) <> 0)
             (*
              and (not (buf.COMMENT[29]=' '{ $20}))
              *)
                 then begin
                tags.Values[GetMP3TagName(I_Tr)] := IntToStr(byte(buf.COMMENT[29]));
            end;
        end;
        tags.Values[GetMP3TagName(I_G)] := GetGenreName(buf.GENRE);
    end;
    {$WARN IMPLICIT_STRING_CAST ON}
end;
function ReadMP3Full(filename : string; tags : THashedStringList) : string;
var  ms : TMP3Segment;
begin
    ms := TMP3Segment.Create;
    if not ms.GetHeader(filename) then EXIT;
    tags.values[GetMP3TagName(M_B)] := IntToStr(ms.bitrate);
    tags.Values[GetMP3TagName(M_S)] := IntToStr(ms.sample);
    tags.Values[GetMP3TagName(M_C)] := IntToStr(ms.channel);
    tags.values['headerpos:'] := IntToStr(ms.filep);
    ReadMP3(Filename, tags);

    if (LongerDirectoryExists('debug\')) then begin
        tags.SaveToFile('debug\'+ extractfilename(filename) + '-log.txt');
    end;
end;


//--------------------------------------------------------
// TMP3Segment
//--------------------------------------------------------

function TMP3Segment.IsHeader(h : array of byte) : boolean;
var s : string;
begin
    result := false;

    // all bits set section
    if ((h[0] and $FF) <> $FF) or
       ((h[1] and $E0) <> $E0) then EXIT;

    version := (h[1] and MSK_B1_VERSION) shr 3;
    layer   := (h[1] and MSK_B1_LAYER)   shr 1;
    bitrate := (h[2] and MSK_B2_BITRATE) shr 4;
    sample  := (h[2] and MSK_B2_SAMPLE)  shr 2;
    channel := (h[3] and MSK_B3_CHANNEL) shr 6;

    // reserved or bad numbers
    if version = 1 then EXIT;
    if layer   = 0 then EXIT;
    if bitrate = $0F then EXIT;
    if channel = $03 then EXIT;
    if sample > 2 then EXIT;
    // layer should be 1 = Layer 3

    case version of
    0:   bitrate := MP3_Bitrates[0, bitrate];
    3,2: bitrate := MP3_Bitrates[0, bitrate];
    end;

    sample := MP3_Sample[version, sample];
    if channel = 3 then
        channel := 1
    else
        channel := 2;

    s := IntToStr(sample) + IntToStr(version) + IntToStr(layer)
        + IntToStr(bitrate) + IntToStr(channel);
    result := true;
end;
function TMP3Segment.GetHeader(filename : string) : boolean;
var f : file;
    procedure ScanForNextHeader;
    begin
        while not (IsHeader(self.header) or EOF(f)) do begin
            if (header[1] = $FF) then begin
                header[0] := header[1];
                header[1] := header[2];
                header[2] := header[3];
                blockread(f, header[3], 1);
            end else if (header[2] = $FF) then begin
                header[0] := header[2];
                header[1] := header[3];

                blockread(f, header[2], 2);
            end else if (header[3] = $FF) then begin
                header[0] := header[3];

                blockread(f, header, 3);
            end else begin
                blockread(f, header, 4);
            end;
        end;
    end;

var id3 : array[0..9] of byte;
    size : cardinal;
    fs : integer;
begin
    // here's the beef
    // If it's not an ID3, rewind and test from the begining
    // (were the first header is very likely to be)
    // Otherwise jump directly to the MP3 header

    assignfile(f, filename); filemode := fmOpenRead;
    reset(f,1);

    result := false;
    fs := FileSize(f);
    if fs < 128 then begin
    	close(f);
    	EXIT;
    end;

    blockread(f, id3, sizeof(id3));

    if (id3[0] = Ord('I')) and
        (id3[1] = Ord('D')) and
        (id3[2] = Ord('3')) then begin
        move(id3[6], size, 4);
        size := (id3[9] and $ff) +
	        ((id3[8] and $ff) shl 7) +
            ((id3[7] and $ff) shl 14) +
            ((id3[6] and $ff) shl 21);
        Seek(f, size + 10);
    end else begin
        seek(f,0);
    end;

    blockread(f, self.header, 4);
    ScanForNextHeader;
    filep := FilePos(f);
    result := not EOF(f);
    closefile(f);
end;


//
// JPEG stuff
//
function ReadJpeg(filename : string; tags : THashedStringList; useAbbreviations : boolean = false) : string;
var f      : file;
    sg : TJPEGSegment;

    s : string;
    i : integer;
    ifd_pointer : integer;
const TIFF_OFFSET = 6;
    function GetData(offset : integer; format : integer; count : integer) : string;
    var tsize : integer;
        c1, c2 : cardinal;
    const
        Format2Size : array[1..12] of integer = (
            1,1,2,4,8,1,1,2,4,8,4,8
        );
    begin
        tsize := 1;
        try
            tsize := Format2Size[format];
        except
        end;

        if count > (MaxInt/tsize) then EXIT;
        if (tsize * count) > 4 then begin
            // find data given offset
            //result := sg.GetDataString(offset + TIFF_OFFSET, tsize * count );
        end else begin
            // offset is the data
            result := IntToHex(offset, 8);
        end;
        case format of
            1 : {$region 'Byte'}
                begin // byte
                    result := IntToStr(Byte(offset));
                end;
                {$endregion}
            2,7 : {$region 'String'}
            begin // string
                if (tsize * count) > 4 then begin
                    // find data given offset
                    {$WARNINGS OFF}
                    result := sg.GetDataString(offset + TIFF_OFFSET, tsize * count );
                    {$WARNINGS ON}
                end else begin
                    // offset is the data
                    // chop off any unused data
                    result := PAnsiChar(@offset);
                    setlength(result, (tsize * count));
                end;
            end;
            {$endregion}
            3 : {$region 'unsigned short'}
            begin // unsigned short
                result := IntToStr( sg.HI_LO(
                    Word( sg.HI_LO(Cardinal(offset)) )
                ));
            end;
            {$endregion}
            4 : {$region 'unsigned long'}begin // unsinged long
                result := IntToStr(Cardinal(offset))
            end;
            {$endregion}
            5 : {$region 'ulong/ulong'}begin // ulong / ulong
                c1 := sg.GetDataLongWord(offset + TIFF_OFFSET);
                c2 := sg.GetDataLongWord(offset + TIFF_OFFSET + 4);
                if ((c2 * 1) = 0) then begin
                    result := '0.0';
                end else begin
                    result := FloatToStr(c1 / c2);
                end;
                if length(result) > 4 then setlength(result, 4);
            end;
            {$endregion}
            6 : {$region 'signed byte'}begin // signed byte
                result := IntToStr(ShortInt(offset));
            end;
            {$endregion}
            //7 : // undefined 8bit something or other
            8 : {$region 'signed word'}begin // singed word
                result := IntToStr(SmallInt(offset));
            end;
            {$endregion}
            9 : {$region 'signed long'}begin // signed long
                result := IntToStr(Integer(offset));
            end;
            {$endregion}
            10 : {$region 'slong/slong'}begin // slong / slong
                c1 := sg.GetDataLongWord(offset + TIFF_OFFSET);
                c2 := sg.GetDataLongWord(offset + TIFF_OFFSET + 4);
                if (c2 * 1) = 0 then begin
                	result := '0.0';
                end else begin
                 	result := FloatToStr(c1/c2);
                    	if length(result) > 4 then setlength(result, 4);
                end;

            end;
            {$endregion}
        end;
    end;
    function GetRationalString(offset, tformat, tag : integer) : string;
    var u1, u2 : longword;
        s1, s2 : cardinal;
        x1, x2 : string;
    begin
        // cheezy greatist common denominator for
        // powers of 10 [GRIN]
        case tformat of
        5:
            begin
                u1 := sg.GetDataLongWord(offset + TIFF_OFFSET);
                u2 := sg.GetDataLongWord(offset + TIFF_OFFSET + 4);
                x1 := IntToStr(u1);
                x2 := IntToSTr(u2);
                if (x1[length(x1)] = '0') and (x1[length(x1)] = '0') then begin
                    setlength(x1, length(x1) - 1);
                    setlength(x2, length(x2) - 1);
                end;
                result := x1 + '/' + x2;
            end;
        10:
            begin
                s1 := sg.GetDataLong(offset + TIFF_OFFSET);
                s2 := sg.GetDataLong(offset + TIFF_OFFSET + 4);
                x1 := IntToStr(s1);
                x2 := IntToSTr(s2);
                if (x1[length(x1)] = '0') and (x1[length(x1)] = '0') then begin
                    setlength(x1, length(x1) - 1);
                    setlength(x2, length(x2) - 1);
                end;
                result := IntToStr(s1) + '/' + IntToSTr(s2)
            end;
        end;
    end;
    function HandleDirectory(ifd_pointer : integer) : integer;
    var dir_entry : integer;
        tag : integer;
        tformat : integer;
        tcount : integer;
        toffset : integer;
        i : integer;
        td : TDateTime;
        procedure FormatData;
        var s : string;
            malformed : boolean;
        begin
            if not (tformat in [1,2,3,4,5,6,7,8,9,10]) then EXIT;

            malformed := false;
            case  (tag) of
            $829A, $292 {X_ET, X_CBPP}:
                s := GetRationalString(toffset, tformat, tag);
            $829D {X_FN} :
                begin
                    // x.x format
                    s := GetData(toffset, tformat, tcount);
                    s := LeftStr(s, 3);

                    if pos(FormatSettings.DecimalSeparator,s) = 0 then begin
                        s := s + FormatSettings.DecimalSeparator + '0';
                    end;
                end;
            $8827 {X_ISOSR} :
                begin
                    s := GetData(toffset, tformat, tcount);
                    while length(s) < 3 do s := '0' + s;
                end;
            $9101 {X_CC} :
                s := IntToHex(toffset, (tcount * 2));
            $9286 {X_UC} :
                s := sg.GetDataString(toffset + TIFF_OFFSET, tcount, true);
            {X_DT} $132,{X_DTO} $9003, {X_DTD} $9004 :
                begin
                    // I don't know the following formats are times
                    //{X_SST}$9290,{X_SSTO} $9291, {X_SSTD}$9292 : begin
                    s := GetData(toffset, tformat, tcount);
                    try
                        td := RecodeDateTime(td,
                            SafeInt(copy(s,1,4)){year},
                            SafeInt(copy(s,6,2)) {month},
                            SafeInt(copy(s,9,2)) {day},
                            SafeInt(copy(s,12,2)) {hour},
                            SafeInt(copy(s,15,2)) {minute},
                            SafeInt(copy(s,18,2)) {sec},
                            0
                        );
                        s := DateTimeToSTr(td);
                   except
                        malformed := true;
                   end;
                end;
            else
                begin
                    s := GetData(toffset, tformat, tcount) ;
                    if tag = $EA1C then begin
                        // UPDATE: Geez angry guy, chill.
                        // F*CK YOU MICROSOFT AND YOUR BULLSH*T PATENT
                        // THAT SCREWS UP AN OPEN STANDARD FORMAT
                        s := 'BAD MS FORMAT'
                    end;
                end;
            end;
            if malformed then begin
                tags.Values[sg.GetTagDesc(tag)+'Malformed'] := s;
            end else begin
                tags.Values[sg.GetTagDesc(tag)] := s;
            end;
        end;
    begin
        // IDF0
        // 0-4 = number of directories
        // XX = tag type
        // YY = data format
        // ZZZZ = number of items in directory
        // AAAA = data or offset of data if YY format is 5 (which
        // is 8 bytes long)
        dir_entry := sg.GetDataWord(ifd_pointer);
        inc(ifd_pointer, 2);
        for i := 0 to (dir_entry - 1) do begin
            tag       := sg.GetDataWord(ifd_pointer);

            inc(ifd_pointer, 2);
            tformat   := sg.GetDataWord(ifd_pointer);
            inc(ifd_pointer, 2);
            tcount := sg.GetDataLong(ifd_pointer);
            inc(ifd_pointer, 4);
            toffset := sg.GetDataLong(ifd_pointer);
            inc(ifd_pointer, 4);
            //  Sub IFD
            //  Interoperability IFD
            //  Makernote IFD
            // just a regular tag
            if (tag = GetTagID(X_EO) {//$8769}) then begin
                tags.Values['SubIFD'] :=  '-------------';
                if toffset > sg.SegSize then BREAK;
                HandleDirectory(toffset + TIFF_OFFSET);
            end else if (tag = GetTagID(X_IO {//$A005})) then begin
                tags.Values['InterIFD'] := '-------------';
                if toffset > sg.SegSize then BREAK;
                HandleDirectory(toffset + TIFF_OFFSET);
            end else if (tag = GetTagID(X_GPSI)) then begin
                {tags.Values['GPSIFD'] := '-------------';
                if toffset > sg.SegSize then BREAK;
                HandleDirectory(toffset + TIFF_OFFSET);}
            end else if (tag = GetTagID(X_MN)  {//$927C}) then begin
                {
                tags.Values['MakernoteIFD'] := '-------------';
                HandleDirectory(toffset + TIFF_OFFSET);
                // some start with an ASCII string identifying the
                // camera, some start as noram IDFs, but each has
                // it's own tags/data/ and bugs
                // I choose to ignore this mess altogether.
                }
            end else begin
                FormatData;
            end;
        end;
        // next IFD1 pointer
        i := sg.GetDataLong(ifd_pointer);
        result := i + TIFF_OFFSET;
    end;
var k : integer;
    io : integer;
    header : string;
    namestr : string;
    readXMP : boolean;
    ms : TStringStream;
const XMP_HEADER = 'http://ns.adobe.com/xap/1.0/';

    procedure getAdobeTags;
    var
        id : string;
        b : byte;
        offset : integer;
        iptc_type : cardinal;
        iptc_id : word;
        iptc_name_len : byte;
        iptc_name : string;
        iptc_data_size : cardinal;

        iptc_seg_header : word;
        iptc_seg_type : byte;
        iptc_seg_size : word;
        i : integer;
    begin
        offset := 0;
        id := sg.GetDataString(0, 14);
        inc(offset, 14);

        tags.Values['App13Id'] := id;
        //(Type)           4 bytes  Photoshop uses '8BIM' from ver 4.0 on
        //(ID)             2 bytes  a unique identifier, e.g., "\004\004" for IPTC
        //(Name)             ...    a Pascal string (padded to make size even)
        //(Size)           4 bytes  actual size of resource data
        //(Data)             ...    resource data, padded to make size even
        iptc_type := sg.GetDataLongWord(offset);
        inc(offset, 4);

        if iptc_type = $3842494D then begin
            iptc_id := sg.GetDataWord(offset);
            inc(offset, 2);

            if iptc_id <> $0404 then EXIT;

            iptc_name_len := sg.GetDataByte(offset);
            inc(offset, 1);
            if (iptc_name_len = 0) then begin
                inc(offset, 1);
            end else begin
                iptc_name := sg.GetDataString(offset, iptc_name_len);
                inc(offset, iptc_name_len);
            end;

            iptc_data_size := sg.GetDataWord(offset);
            inc(offset, 4);


            iptc_seg_header := sg.GetDataWord(offset);
            if iptc_seg_header <>$1c02 then EXIT;
            inc(offset, 2);

            iptc_seg_type := sg.GetDataByte(offset);
            inc(offset, 1);


             
        end;

    end;
    procedure GetJPEGStandardtags;
    begin
        tags.Values[JFIF_PRECISION] := IntToStr(sg.GetDataByte(0) * sg.GetDataByte(5));
        tags.Values[JFIF_HEIGHT] := IntToStr(sg.GetDataWord(1));
        tags.Values[JFIF_WIDTH] := IntToStr(sg.GetDataWord(3));
    end;
    procedure GetEXIFData;
    begin
        // Start of TIFF header
        // bytes 0-1 = II or MM
        // bytes 2-3 = $2400(for II) or $0024(for MM)
        // bytes 4-7 = long int pointing to first IFD
        //   usually $08000000(II) or $00000008(mm) }
        //   Relative to byte 6
        sg.SetLittleIndian(sg.GetDataWord(TIFF_OFFSET) = $4949); {'II'=little}
        i := sg.GetDataLong(4+TIFF_OFFSET);
        ifd_pointer := i + TIFF_OFFSET;

        // IFD0 (points to IFD1)
        // IFD1
        tags.Values['IFD0'] := '----------';
        //i := HandleDirectory(ifd_pointer);
        HandleDirectory(ifd_pointer);
        // don't care about the thumbnail stuff just yet
        //tags.Values['IFD1'] := '----------';
        //HandleDirectory(i);
        sg.SetLittleIndian(false);
    end;
    procedure GetXMPData;
    var
        xdoc : IXMLDOMDocument;
        nodes1, nodes2 : IXMLDOMNodeList;
        n1, n2 : IXMLDOMNode;
        i, k : integer;
    begin
        header := sg.GetDataString(0,length(XMP_HEADER));
        if header = XMP_HEADER then begin
            s := sg.GetDataString(
                length(XMP_HEADER)+1,
                sg.SegSize - (length(XMP_HEADER)+1));
            xdoc := CreateOleObject('Microsoft.XMLDOM') as IXMLDomDocument;
            xdoc.loadXML(s);
            {find all namespaces used}
            nodes1 := xdoc.selectNodes('//rdf:Description/@xmlns:*');
            for i := 0 to nodes1.length - 1 do begin
                n1 := nodes1.item[i];
                namestr := n1.NodeName;
                namestr := RightStr(namestr,length(namestr)-length('xmlns:'));
                {all nodes in that namespace}
                nodes2 := xdoc.selectNodes('//'+namestr+':* | //@'+namestr+':*');
                for k := 0 to nodes2.length - 1 do begin
                    n2 := nodes2.item[k];
                    try
                        // turn DOMAIN:VariableName into xmp_Domain_VariableName
                        namestr := n2.nodeName;
                        namestr := replacestr(namestr, ':', '_');
                        tags.Values['?xmp_'+namestr+'?'] := n2.text;

                        XMPvariables.AddOrSetValue('?xmp_'+lowercase(namestr)+'?',n2.text);
                    except
                    end;
                end;
            end;
        end;
    end;
begin
    {: open file :}
    sg := TJPEGSegment.Create;
    tags.Clear;
    {$I-}
    assignfile(F, filename); filemode := fmOpenRead;
    reset(f,1);
    {$I+}
    io := IOResult;
    if (io <> 0) then begin
        raise Exception.Create('ReadJpeg: Can''t open file: ' + filename + ' ('+IntToSTr(io)+')');
    end;
    if FileSize(f) = 0 then begin
        close(f);
    	EXIT;
    end;

    XMPvariables.Clear;
    readXMP := false;
    repeat
        try
            sg.getSegment(f);
        except
            BREAK;
        end;
        if (sg.segtype in [SOI,EOI]) then CONTINUE;
        case sg.SegType of
            APP0:
                s := sg.GetDataString(0,4);
            {dont ask why, but some don't use SOF0}
            SOF0, SOF1, SOF2, SOF9, SOF10:
                GetJPEGStandardtags;
            APP1_EXIF:
                begin
                    // bytes 0 - 5 = E x i f #0 #0  the EXIF header
                    header := sg.GetDataString(0,4);
                    if header = 'Exif' then begin
                        GetEXIFData;
                    end else begin
                        GetXMPData;
                        readXMP := true;
                    end;
                end;
            APP13_IPTC:
                begin
                    //getAdobeTags;
                end;
            SOS: BREAK;
            EOI: BREAK;
        end;
    until false;
    try
        close(f);
    except
    end;

    if readXMP then begin
        XMPvariables.TrimExcess;
    end;
    if (LongerDirectoryExists('debug\')) then begin
        try
            tags.SaveToFile('debug\'+ExtractFileName(filename) + '-log.txt');
        except on e : Exception do
            begin
                //just ignore really long filenames
            end;
        end;
    end;
end;

function GetXMPValue(variablename : string) : string;
begin
    variablename := lowercase(variablename);
    XMPvariables.TryGetValue(variablename, result);
end;

function TJPEGSegment.GetTagEnum( abr : string ) : word;
begin
    result := GetTagEnum(abr);
end;
function TJPEGSegment.GetTagID( X : EXIF_Enum ) : word;
begin
    result := UnitFileTags.GetTagID(x);
end;
function TJPEGSegment.GetTagAbbr( x : EXIF_ENum) : string;
begin
    result := UnitFileTags.GetTagAbbr(x);
end;

function TJPEGSegment.GetTagName( x :EXIF_Enum ) : string;
begin
    result := UnitFileTags.GetTagName(x);
end;
function TJPEGSegment.GetTagName( abr : string ) : string;
begin
    result := UnitFileTags.GetTagName(abr);
end;
function TJPEGSegment.GetTagDesc( tag : word) : string;
begin
    result := UnitFileTags.GetTagDesc(tag);
end;

procedure TJPEGSegment.SetLittleIndian(value : boolean);
begin
    self.LittleEndian := value;
end;
{::::::::::::::::::::::::::::::::::}
{: Change a HI-LO word to LO-HIGH :}
{::::::::::::::::::::::::::::::::::}
function Swap4Bytes(L  : integer) : integer;
begin
    result := swap(L shr 16) or
           (longint(swap(L and $ffff)) shl 16)
end;

function TJPEGSegment.HI_LO(inw : word) : word;
begin
    if self.LittleEndian then begin
        result := inw;
    end else begin
        result := swap(inw);
    end;
end;
function TJPEGSegment.HI_LO(inl : integer) : integer;
begin
    if self.LittleEndian then begin
        result := inl;
    end else begin
        result := Swap4Bytes(inl);
    end;
end;

{$R-}
function TJPEGSegment.HI_LO(inl : cardinal) : cardinal;
begin
    if self.LittleEndian then begin
        result := inl;
    end else begin
        result := Swap4Bytes(inl);
    end;
end;
{$R+}


// read only the header and not the following data
procedure TJPEGSegment.GetHeader(var f : file);
var id : packed array[0..1] of byte;
begin
    // Read the segment ID, skip any padding bytes
    blockread(f,id,2);
    while (id[1] = $FF) do begin
        blockread(f, id[1], 1);
    end;
    if (id[0] <> 255) then begin
        raise EParserError.Create('TJPEGSegment: Not a JPEG header');
    end;

    // read the size of the following data, if any
    // and read the data
    self.SegType := id[1];
    if (self.SegType in [SOI, EOI]) then begin
        self.SegSize := 0;
    end else begin
        blockread(f, self.SegSize, 2);
        segsize := self.HI_LO(segsize) - 2;
    end;
end;
// read both the header and the following data chunk
procedure TJPEGSegment.GetSegment(var f : file);
begin
    if (SegSize <> 0) then begin
        FreeMem(self.SegData, self.SegSize);
        SegData := nil;
    end;

    self.GetHeader(f);

    if (SegSize <> 0) then begin
        GetMem(SegData, SegSize);
        blockread(f, self.SegData^, self.segsize);
    end;
end;

function TJPEGSegment.GetType : byte;
begin
    result := self.SegType;
end;
function TJPEGSegment.GetSize : word;
begin
    result := self.SegSize;
end;


function TJPEGSegment.GetDataByte(i : integer) : byte;
var x : integer;
begin
    x := Integer(self.SegData) + i;
    move(Pointer(x)^, result, 1);
end;
function TJPEGSegment.GetDataWord(i : integer) : word;
var x : integer;
begin
    x := Integer(self.SegData) + i;
    move(Pointer(x)^, result, 2);
    result := hi_lo(result);
end;
function TJPEGSegment.GetDataLong(i : integer) : integer;
var x : integer;
begin
    x := Integer(self.SegData) + i;
    move(Pointer(x)^, result, 4);
    result := hi_lo(result);
end;

function TJPEGSegment.GetDataLongWord(i : integer) : cardinal;
var x : integer;
begin
    x := Integer(self.SegData) + i;
    move(Pointer(x)^, result, 4);
    result := hi_lo(Cardinal(result));
end;
function TJPEGSegment.GetDataString(offset : integer; length : integer; skip8 : boolean = false): string;
var x :integer;
    y : integer;
    function SkipNulls(c : char) : string;
    begin
        result := '';
        if c <> #0 then begin
            result := c;
        end;
    end;

begin
    result := '';
    if (offset+length) > self.SegSize  then begin
        EXIT;
    end;

    y := 0;
    if skip8 then y := 8;

    for x := y to length - 1 do begin
        result := result + SkipNulls(Chr(self.GetDataByte(offset + x)));
    end;
end;

//-----------------------------------------------------------------
// All these conversion routines are for having an enumerable list
// of tags to generate in a loop
// Abr -> Name is for UnitMetaVariables
// EXIF_Enum -> Name and EXIF_Enum -> Abr are for UnitFrmConfig
//-----------------------------------------------------------------


function GetTagEnum( abr : string ) : word;
var s : string;
    i : word;
begin
    abr := lowercase(abr);
    if abr[1] = '?' then begin
        abr := midstr(abr,2,length(abr)-2);
    end;
    if AbrHash.TryGetValue(abr, i) then begin
        result := i;
    end else begin
        result := $FFFF;
    end;
end;
function GetTagID( X : EXIF_Enum ) : word;
begin
    result := 0;
    case (x) of
    X_IW : result :=  $100 ;
    X_IL : result := $101;
    X_BPS : result :=$102;
    X_C : result := $103;
    X_PI : result := $106;
    X_FO : result := $10A;
    X_DN : result :=  $10D;
    X_ID : result :=  $10E;
    X_Ma : result := $10F;
    X_Mo : result := $110;
    X_SO : result :=  $111;
    X_O : result := $112;
    X_SPP : result := $115;
    X_RPS : result := $116  ;
    X_SBC : result :=  $117  ;
    X_XR : result :=  $11A    ;
    X_YR : result :=  $11B     ;
    X_PC : result :=  $11C      ;
    X_RU : result :=  $128       ;
    X_TF : result :=  $12D   ;
    X_S : result :=       $131     ;
    X_DT : result :=       $132     ;
    X_A : result :=   $13B ;
    X_WP : result :=  $13E       ;
    X_PCr : result :=  $13F       ;
    X_TR : result :=  $156    ;
    X_JPEGP : result :=  $200       ;
    X_TO : result :=  $201      ;
    X_TL : result :=  $202       ;
    X_YCCC : result :=  $211      ;
    X_YCCSS : result :=  $212      ;
    X_YCCP : result :=  $213        ;
    X_RBW : result :=  $214     ;
    X_SOf : result :=  $273      ;
    X_GPSI : result :=  $8825     ;
    X_EO : result :=  $8769        ;
    X_II : result :=  $0001         ;
    X_ET : result :=  $829A          ;
    X_FN : result :=  $829D      ;
    X_EP : result :=  $8822       ;
    X_ISOSR : result :=  $8827          ;
    X_OEC : result :=  $8828        ;
    X_EV : result :=  $9000          ;
    X_DTO : result :=  $9003          ;
    X_DTD : result :=  $9004           ;
    X_CC : result :=  $9101        ;
    X_CBPP : result :=  $9102       ;
    X_SSV : result :=  $9201         ;
    X_AV : result :=  $9202           ;
    X_BV : result :=  $9203            ;
    X_EBV : result :=  $9204            ;
    X_MAV : result :=  $9205;
    X_SD : result :=  $9206 ;
    X_MM : result :=  $9207  ;
    X_LS : result :=  $9208   ;
    X_F : result :=  $9209     ;
    X_FL : result :=  $920A     ;
    X_MN : result :=  $927C      ;
    X_UC : result :=  $9286  ;
    X_FPV : result :=  $A000  ;
    X_CS : result :=  $A001    ;
    X_EIW : result :=  $A002    ;
    X_EIL : result :=  $A003     ;
    X_IO : result :=  $A005       ;
    X_FE : result :=  $A20B   ;
    X_SFR : result :=  $A20C   ;
    X_FPXR : result :=  $A20E   ;
    X_FPYR : result :=  $A20F    ;
    X_FPRU : result :=  $A210    ;
    X_SL : result :=  $A214      ;
    X_EI : result :=  $A215      ;
    X_SM : result :=  $A217       ;
    X_FS : result :=  $A300        ;
    X_ST : result :=  $A301    ;
    X_CFARPD : result :=  $828D      ;
    X_CFAP : result :=  $828E;
    X_BL : result :=  $828F     ;
    X_Co : result :=  $8298     ;
    X_IPTCNA : result :=  $83BB      ;
    X_ICP : result :=  $8773    ;
    X_SS : result :=  $8824     ;
    X_SST : result :=  $9290    ;
    X_SSTO : result :=  $9291        ;
    X_SSTD : result :=  $9292        ;
    X_CR : result :=  $A401     ;
    X_EM : result :=  $A402     ;
    X_WB : result :=  $A403     ;
    X_DZR : result := $A404     ;
    X_FL35 : result :=$A405     ;
    X_SCT : result := $A406     ;
    X_GC : result :=  $A407     ;
    X_Cn : result :=  $A408     ;
    X_Sa : result :=  $A409     ;
    X_Sh : result :=  $A40A     ;
    X_SDR : result := $A40C;

    //
    X_DTOM : result := $C001;
    X_DTDM : result := $C002;
    X_DTM  : result := $C003;
    end;
end;
function GetTagDesc( tag : word ) : string;
begin
    result := IntToHex(tag,4) + ' not found';
    case tag of
    $100     :  result := 'ImageWidth';               // I_IW
    $101     :  result := 'ImageLength';              // I_IL
    $102     :  result := 'BitsPerSample';            // I_BPS
    $103     :  result := 'Compression';              // I_C
    $106     :  result := 'PhotometricInterpretation';// I_PI
    $10A     :  result := 'FillOrder';                // I_FO
    $10D     :  result := 'DocumentName';             // I_DN
    $10E     :  result := 'ImageDescription';         // I_ID
    $10F     :  result := 'Make';                     // I_Ma
    $110     :  result := 'Model';                    // I_Mo
    $111     :  result := 'StripOffsets';             // I_SO
    $112     :  result := 'Orientation';              // I_O
    $115     :  result := 'SamplesPerPixel';          // I_SPP
    $116     :  result := 'RowsPerStrip';             // I_RPS
    $117     :  result := 'StripByteCounts';          // I_SBC
    $11A     :  result := 'XResolution';              // I_XR
    $11B     :  result := 'YResolution';              // I_YR
    $11C     :  result := 'PlanarConfiguration';      // I_PC
    $128     :  result := 'ResolutionUnit';           // I_RU
    $12D     :  result := 'TransferFunction';         // I_TF
    $131     :  result := 'Software';                 // I_S
    $132     :  result := 'DateTime';                 // I_DT
    $13B     :  result := 'Artist';                   // I_A
    $13E     :  result := 'WhitePoint';               // I_WP
    $13F     :  result := 'PrimaryChromaticities';    // I_PC
    $156     :  result := 'TransferRange';            // I_TR
    $200     :  result := 'JPEGProc';                 // I_JPEGP
    // this is for internal use only
    $201     :  result := 'ThumbnailOffset';          // I_TO
    $202     :  result := 'ThumbnailLength';          // I_TL
    $211     :  result := 'YCbCrCoefficients';        // I_YCCC
    $212     :  result := 'YCbCrSubSampling';         // I_YCCSS
    $213     :  result := 'YCbCrPositioning';         // I_YCCP
    $214     :  result := 'ReferenceBlackWhite';      // I_RBW
    $273     :  result := 'StripOffset';              // I_SO
    $8825    :  result := 'GPSInfo';                  // I_GPSI
    $8769    :  result := 'ExifOffset';               // I_EO
    // this is for internal use only
    //$0001    :  result := 'InteroperabilityIndex';    // I_II
    // this is for internal use only
    $829A    :  result := 'ExposureTime';             // I_ET
    $829D    :  result := 'FNumber';                  // I_FN
    $8822    :  result := 'ExposureProgram';          // I_EP
    $8827    :  result := 'ISOSpeedRatings';          // I_ISOSR
    $8828    :  result := 'OECF';                     // I_OEC
    $9000    :  result := 'ExifVersion';              // I_EV
    $9003    :  result := 'DateTimeOriginal';         // I_DTO
    $9004    :  result := 'DateTimeDigitized';        // I_DTD
    $9101    :  result := 'ComponentsConfiguration';  // I_CC
    $9102    :  result := 'CompressedBitsPerPixel';   // I_CBPP
    $9201    :  result := 'ShutterSpeedValue';        // I_SSV
    $9202    :  result := 'ApertureValue';            // I_AV
    $9203    :  result := 'BrightnessValue';          // I_BV
    $9204    :  result := 'ExposureBiasValue';        // I_EBV
    $9205    :  result := 'MaxApertureValue';         // I_MAV
    $9206    :  result := 'SubjectDistance';          // I_SD
    $9207    :  result := 'MeteringMode';             // I_MM
    $9208    :  result := 'LightSource';              // I_LS
    $9209    :  result := 'Flash';                    // I_F
    $920A    :  result := 'FocalLength';              // I_FL
    $927C    :  result := 'MakerNote';                // I_MN
    // this is for internal use only
    $9286    :  result := 'UserComment';              // I_UC
    $A000    :  result := 'FlashPixVersion';          // I_FPV
    $A001    :  result := 'ColorSpace';               // I_CS
    $A002    :  result := 'ExifImageWidth';           // I_EIW
    $A003    :  result := 'ExifImageLength';          // I_EIL
    $A005    :  result := 'InteroperabilityOffset';   // I_IO
    // this is for internal use only
    $A20B    :  result := 'FlashEnergy';              // I_FE
    $A20C    :  result := 'SpatialFrequencyResponse'; // I_SFR
    $A20E    :  result := 'FocalPlaneXResolution';    // I_FPXR
    $A20F    :  result := 'FocalPlaneYResolution';    // I_FPYR
    $A210    :  result := 'FocalPlaneResolutionUnit'; // I_FPRU
    $A214    :  result := 'SubjectLocation';          // I_SL
    $A215    :  result := 'ExposureIndex';            // I_EI
    $A217    :  result := 'SensingMethod';            // I_SM
    $A300    :  result := 'FileSource';               // I_FS
    $A301    :  result := 'SceneType';                // I_ST
    $828D    :  result := 'CFARepeatPatternDim';      // I_CFARPD
    $828E    :  result := 'CFAPattern';               // I_CFAP
    $828F    :  result := 'BatteryLevel';             // I_BL
    $8298    :  result := 'Copyright';                // I_C
    $83BB    :  result := 'IPTC/NAA';                 // I_IPTCNA
    $8773    :  result := 'InterColorProfile';        // I_ICP
    $8824    :  result := 'SpectralSensitivity';      // I_SS
    $9290    :  result := 'SubSecTime';               // I_SST
    $9291    :  result := 'SubSecTimeOriginal';       // I_SSTO
    $9292    :  result := 'SubSecTimeDigitized';      // I_SSTD
    $A401    :  result := 'CustomRendered';           // I_CR
    $A402    :  result := 'ExposureMode';             // I_EM
    $A403    :  result := 'WhiteBalance';             // I_WB
    $A404    :  result := 'DigitalZoomRatio';         // I_DZR
    $A405    :  result := 'FocalLength35';            // I_FL
    $A406    :  result := 'SceneCaptureType';         // I_SCT
    $A407    :  result := 'GainControl';              // I_GC
    $A408    :  result := 'Contrast';                 // I_Cn
    $A409    :  result := 'Saturation';               // I_Sa
    $A40A    :  result := 'Sharpness';                // I_Sh
    $A40C    :  result := 'SubjectDistanceRange';     // I_SDR

    $C001    :  result := 'DateTimeOriginalMalformed';
    $C002    :  result := 'DateTimeDigitizedMalformed';
    $C003    :  result := 'DateTimeMalformed';

    end;
end;
function GetTagAbbr( x : EXIF_ENum) : string;
begin
    result := lowercase( GetEnumName(TypeInfo(EXIF_Enum), integer(x)) );
    EXIT;
end;

function GetTagName( x :EXIF_Enum ) : string;
var i : word;
begin
    result := '';
    i := GetTagID(x);
    if i = 0 then EXIT;
    result := GetTagDesc(i);
end;
function GetTagName( abr : string ) : string;
var i : word;
begin
    result := '';
    if AbrHash.TryGetValue(lowercase(abr), i) then begin
        result := GetTagDesc(i);
    end;
end;

var
    AbrI : EXIF_Enum;
    wd : word;
    i : integer;

    x : EXIF_Enum;
	m : MP3_Enum;
initialization
begin
    // Create an ABR to Enum hash
    AbrHash := TDictionary<string, word>.Create(integer(high(EXIF_ENUM)));
    for Abri := low(EXIF_FileID) to high(EXIF_FileID) do begin
        AbrHash.AddOrSetValue( GetTagAbbr(AbrI), EXIF_FileID[Abri] );
    end;
    AbrHash.TrimExcess;

    XMPvariables := TDictionary<string,string>.create;


    // TODO: research why X_CC was skipped

    // create a Name to VarTag hash for JPEG and MP3
    NameToAbbr := THashedStringList.create;
    for x := low(EXIF_Enum) to high(EXIF_ENum) do begin
        // skip internal structure tags or problematic tags
        if x in [X_JPEGP, X_EO, X_II, X_MN, X_IO, X_IW, X_IL,
            X_GPSI, X_CC ] then CONTINUE;

        NameToAbbr.Values[UnitFileTags.GetTagName(x)] := UnitFileTags.GetTagAbbr(x);
    end;
    NameToAbbr.values[JFIF_PRECISION] := 'j_b';
	NameToAbbr.values[JFIF_WIDTH] := 'j_w';
	NameToAbbr.values[JFIF_HEIGHT] := 'j_h';


    for m := low(mp3_enum) to high(mp3_enum) do begin
    	NameToAbbr.Values[UnitFileTags.GetMP3TagName(m)] := GetEnumName(TypeInfo(MP3_Enum), integer(m));
    end;
end;
end.

