unit UnitFrameCrumb;
///
/// Display a "Folder|More" combination that grows to the size of the
/// folder name. A size limit is impossed for long names and elipses
/// are used for a truncated name. A hint is generated to show the full
/// name on hovers.
///
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, Buttons;

type
  TFrameCrumb = class(TFrame)
    btnFolder: TSpeedButton;
    btnMore: TSpeedButton;
    btnDrives: TSpeedButton;
    procedure btnFolderMouseEnter(Sender: TObject);
    procedure btnFolderMouseLeave(Sender: TObject);
    procedure btnMoreMouseEnter(Sender: TObject);
    procedure btnMoreMouseLeave(Sender: TObject);
  private
    { Private declarations }
    fFolder : string;
  public
    { Public declarations }
    procedure SetFolder(fullname : string; LastFolderCheck : boolean = false);
    function GetFolder : string;
    procedure ShrinkTo(x : integer);
  end;

implementation

uses UnitFrameFolderDropdown, StrUtils;

{$R *.dfm}

{ TFrameCrumb }

function TextWidth(c : TSpeedButton; s : string) : integer;
var bm : TBitmap;
begin
    bm := TBitmap.Create;
    bm.Canvas.Font := c.Font;
    result := bm.Canvas.TextWidth(s);
    bm.free;
end;

procedure TFrameCrumb.btnFolderMouseEnter(Sender: TObject);
begin
    btnMore.Flat := false;
end;
procedure TFrameCrumb.btnFolderMouseLeave(Sender: TObject);
begin
    btnMore.Flat := true;
end;
procedure TFrameCrumb.btnMoreMouseEnter(Sender: TObject);
begin
    btnFolder.Flat := false;
end;
procedure TFrameCrumb.btnMoreMouseLeave(Sender: TObject);
begin
    btnFolder.Flat := true;
end;

function TFrameCrumb.GetFolder: string;
begin
    result := fFolder;
end;
procedure TFrameCrumb.SetFolder(fullname: string; LastFolderCheck : boolean = false);
var res : integer;
    rec : TSearchRec;
    s, drive : string;
    over : integer;
    x : integer;

const limit = 30;
begin
    fFOlder := ExcludeTrailingPathDelimiter(fullname);
    drive := IncludeTrailingPathDelimiter(ExtractFileDrive(fFolder));

    if drive = fullname then begin
        if ansistartstext('\\',drive) then begin
            btnFolder.Caption := 'Share: '+ ExtractFileName( ExcludeTrailingPathDelimiter(drive));
        end else begin
            btnFolder.Caption := 'Drive '+drive;
        end;
    end else begin
        btnFolder.Caption := ExtractFileName(fFolder);
    end;
    if btnFolder.Caption = '' then btnFolder.Caption := fFolder;

    over := length(btnFolder.caption);
    if over > limit then begin
        s := btnFolder.caption;
        x := (over - limit) div 2;

        if x > 6 then begin
            s := leftstr(btnFolder.Caption, limit) + '...';

            btnFolder.hint := btnFolder.caption;
            btnFOlder.caption := s;
        end;
    end;

    if LastFolderCheck then begin

        {$i-}
        res := findfirst(ExcludeTrailingPathDelimiter(fullname) + '\*.', faDirectory, rec);
        {$i+}
        btnMore.Visible := false;
        while res = 0 do begin
            if ((rec.Attr and fadirectory) > 0) and
                (rec.Name <> '.') and (rec.Name <> '..') then begin
                btnMore.Visible := true;
                BREAK;
            end;
            res := findnext(rec);
        end;
    end;


    btnFolder.Width := TextWidth(btnFolder, btnFolder.Caption) + 10;

    self.Width := 1;
    self.AutoSize := true;
    self.AutoSize := false;
    self.Width := self.width;
end;

procedure TFrameCrumb.ShrinkTo(x: integer);
var wd : integer;
begin
    self.autosize := false;
    self.Width := 10;
    self.AutoSize := true;
    self.autosize := false;
    self.width := self.width +10;
    
	if x > self.width then EXIT;


	wd := 0;
	if self.btnMore <> nil then inc(wd, self.btnMore.Width);
    if self.btnDrives <> nil then inc(wd, self.btnDrives.width);

    self.Width := x;
    self.btnFolder.width := x - wd;
    self.btnMore.Align := alLeft;
    self.AutoSize := true;
    self.autosize := false;

end;

end.
