unit UnitFrameDropDown;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Buttons, Vcl.ExtCtrls,
  Vcl.Menus, System.Generics.Collections;


type TDropDownSelectedEvent = procedure(caption : string; index : integer) of object;
type TDropDownPopupEvent = procedure() of object;
type
  TFrameDropDown = class(TFrame)
    pnlCurrentItem: TPanel;
    btnShow: TSpeedButton;
    pmDropDown: TPopupMenu;
    procedure pnlCurrentItemMouseEnter(Sender: TObject);
    procedure pnlCurrentItemMouseLeave(Sender: TObject);
    procedure pnlCurrentItemClick(Sender: TObject);
    procedure btnShowClick(Sender: TObject);
  private
    { Private declarations }
    fSelectedIndex : integer;
    fSelectedCaption  : string;
    fOnSelected : TDropDownSelectedEvent;
    fOnPopup : TDropDownPopupEvent;
    procedure OnMenuSelected(Sender : TObject);
  public
    { Public declarations }
    procedure setOnSelected(OnSelected : TDropDOwnSelectedEvent);
    procedure setOnPopup(OnPopup : TDropDownPopupEvent);
    procedure setItems(items : TList<string>);
    procedure setCurrent(caption : string);
  end;

implementation

{$R *.dfm}

uses UnitUtil;
// public interface
procedure TFrameDropDown.setOnSelected(OnSelected : TDropDOwnSelectedEvent);
begin
    fOnSelected := onSelected;
end;
procedure TFrameDropDown.setOnPopup(OnPopup : TDropDownPopupEvent);
begin
    fOnPopup := OnPopup;
end;

procedure TFrameDropDown.setItems(items : TList<string>);
var i : integer;
    s : string;
    mi : TMenuItem;
begin
    pmDropDown.Items.Clear;
    for s in items do begin
        mi := TMenuItem.Create(pmDropDown);
        mi.Caption := s;
        mi.OnClick := self.OnMenuSelected;
        pmDropDown.Items.Add(mi);
    end;
end;
procedure TFrameDropDown.setCurrent(caption : string);
begin
    pnlCurrentItem.Caption := caption;
    pnlCurrentItem.Hint := caption;
    pnlCurrentItem.Invalidate;
    fSelectedCaption := caption;
end;

// user events
procedure TFrameDropDown.btnShowClick(Sender: TObject);
var p : TPoint;
begin
    if Assigned(fOnPopup) then begin
        fOnPopup();
    end;
    p := pnlCurrentItem.ClientToScreen(point(pnlCurrentItem.left, pnlCurrentItem.top + pnlCurrentItem.Height));
    pmDropDown.Popup(p.x, p.y);
end;
procedure TFrameDropDown.pnlCurrentItemClick(Sender: TObject);
begin
    btnShow.Click;
end;
procedure TFrameDropDown.pnlCurrentItemMouseEnter(Sender: TObject);
var c : TColor;
begin
    c := ColorToRGB(cl3DLight);
    c := UnitUtil.dimcolor(c,0.7);
    pnlCurrentItem.Color := c;
end;
procedure TFrameDropDown.pnlCurrentItemMouseLeave(Sender: TObject);
begin
    pnlCurrentItem.Color :=   cl3DLight;
end;
procedure TFrameDropDown.OnMenuSelected(Sender : TObject);
var mi : TMenuITem;
begin
    mi := TMenuITem(sender);

    setCurrent(mi.Caption);
    fSelectedIndex := pmDropDown.Items.IndexOf(mi);

    if assigned(fOnSelected) then begin
        fOnSelected(fSelectedCaption, fSelectedIndex);
    end;
end;

end.
