unit UnitFrameFileTags;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls, System.IOUtils, System.StrUtils,
  IniFiles, Winapi.CommCtrl, Vcl.ComCtrls, Vcl.Buttons;


type TTagSelected = procedure(tagVar : string) of object;
type
  TframFileTags = class(TFrame)
    sbMain: TScrollBox;
    gpnlTags: TGridPanel;
    cbxFilter: TComboBoxEx;
    pnlTop: TPanel;
    btnClear: TSpeedButton;
    timFilter: TTimer;
    procedure btnClearClick(Sender: TObject);
    procedure cbxFilterKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbxFilterSelect(Sender: TObject);
    procedure timFilterTimer(Sender: TObject);
  private
    { Private declarations }
    fOnSelected : TTagSelected;
    ftopControlIdx : integer;
    ftags : THashedStringList;
    flastFile : string;
    procedure OnClick(Sender : TObject);
  public
    { Public declarations }
     procedure extractFrom(filename : string);
     procedure setOnSelected(OnSelected : TTagSelected);
     procedure deInit;
  end;

implementation

uses
    UnitUtil,  UnitFileTags, UnitVideo, UnitFrmFiletags;

{$R *.dfm}

const supported_files : array[0..10] of string =
  ('.jpeg','.jpg','.mp3','.avi','.mpg','.mpg',
                   '.mov','.mp4','.m4v','.asf','.wmv');
const row_height = 24;

procedure TframFileTags.extractFrom(filename : string);
var
    i, cnt : integer;
    ext : string;
    ri : TRowITem;
    rb  : TRadioButton;
    first : boolean;
    tagName : string;
    function getTags(filename : string; tags : THashedStringList) : boolean;
    var ext : string;
    begin
        ext := lowercase( TPath.GetExtension(filename) );
        if (ext = '.mp3') then begin
            UnitFileTags.ReadMP3(filename, tags);
        end else if StringIn(ext, ['.jpg','.jpeg']) then begin
            UnitFileTags.ReadJpeg(filename, tags)
        end else begin
            UnitVideo.GetVideoTags(filename, tags)
        end;
        result := tags.count > 0;
    end;

begin
    flastFile := filename;
    cnt := gpnlTags.ControlCount;
    gpnltags.ControlCollection.BeginUpdate;
    for i := cnt-1 downto 0 do begin
        gpnlTags.RemoveControl(gpnlTags.Controls[i]);
    end;
    gpnltags.ControlCollection.EndUpdate;

    ext := lowercase( TPath.GetExtension(filename) );
    if not StringIn(ext, supported_files) then EXIT;

    if assigned(ftags) then ftags.Free;
    ftags := THashedStringList.Create;
    if not TTags.ReadTags(filename, ftags) then begin
        EXIT;
    end;
    ftags.Sort;
    cnt := ftags.Count;

    gpnlTags.RowCollection.BeginUpdate;
    gpnltags.ControlCollection.BeginUpdate;
    gpnlTags.RowCollection.Clear;

    first := true;
    for i := 0 to cnt-1 do begin
        tagName := ftags.Names[i];
        if StringIn(tagName, ['IFD0','SubIFD','InterIFD']) then CONTINUE;
        if ContainsTerm('not found', tagName) then CONTINUE;

        if cbxFilter.Text <> '' then begin
            if not ContainsText(tagName, cbxFilter.Text) then CONTINUE;
        end;

        ri := gpnlTags.RowCollection.Add;


        ri.SizeStyle := ssAbsolute;
        ri.Value := row_height;


        rb := TRadioButton.Create(self);
        rb.Parent := gpnlTags;
        rb.Caption := ftags.Names[i];
        rb.Align := alLeft;
        rb.Width := gpnlTags.ClientWidth;
        rb.OnClick := self.OnClick;
        if first then begin
            rb.Checked := true;
            first := false;
        end;
    end;
    gpnlTags.Height := cnt * row_height;

    gpnlTags.Invalidate;

    gpnlTags.RowCollection.EndUpdate;
    gpnltags.ControlCollection.EndUpdate;

    sbmain.VertScrollBar.Range := gpnltags.Height;
    sbmain.VertScrollBar.Position := gpnltags.Height;
    sbmain.VertScrollBar.Position := 0;
    gpnlTags.top := 0;
    gpnlTags.Left := 5;
    sbmain.Invalidate;
    self.Invalidate;
end;

procedure TframFileTags.OnClick(Sender : TObject);
var s : string;
begin
    if assigned(fOnSelected) then begin
        if TTags.GetTagFromName( TRadioButton(sender).Caption, s ) then begin
            fOnSelected( s );
        end;
    end;
end;
procedure TframFileTags.setOnSelected(OnSelected : TTagSelected);
begin
    fOnSelected := onSelected;
end;

procedure TframFileTags.timFilterTimer(Sender: TObject);
begin
    timFilter.Enabled := false;
    self.extractFrom(flastFile);
end;

procedure TframFileTags.btnClearClick(Sender: TObject);
begin
    cbxFilter.text := '';
    timFilter.Enabled := false;
    timFilter.Enabled := true;
end;

procedure TframFileTags.cbxFilterKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    timFilter.Enabled := false;
    timFilter.Enabled := true;
end;

procedure TframFileTags.cbxFilterSelect(Sender: TObject);
begin
    timFilter.Enabled := false;
    timFilter.Enabled := true;
end;

procedure TframFileTags.deInit;
begin
    if assigned(ftags) then FreeAndNil(ftags);
end;

end.
