unit UnitFrameSteps;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, StdCtrls, ExtCtrls, Contnrs, Buttons;

type TScriptType = (
    stAddBeginningEnding,
    stAddPosition,
    stAddBeforeAfter,
    stReplaceNameExtension,
    stReplaceCharacters,
    stReplaceNumbers,
    stRemoveWords,
    stRemoveBeginningEnding,
    stRemoveSubstring,
    stRemoveBeforeAfter
);

type TScriptStep = class(TComponent)
    private
        FScriptType : TScriptType;
        fenabled : Boolean;

    public
        constructor Create;

    published

    property ScriptType : TScriptType  read FScriptType write FScriptType;
    property Enabled : Boolean read fenabled write fenabled;
end;

type
  TFrameStep = class(TFrame)
    pnlMiddle: TPanel;
    pnlTop: TPanel;
    pnlSpacer: TPanel;
    pnlBottom: TPanel;
    btnExpand: TSpeedButton;
    btnUp: TSpeedButton;
    btnDown: TSpeedButton;
    btnDelete: TSpeedButton;
    btnEnable: TSpeedButton;
    pnlTitle: TFlowPanel;

    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnExpandClick(Sender: TObject);
    procedure btnEnableClick(Sender: TObject);
    procedure pnlTopMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    saveheight : integer;
    fscriptstep : TScriptStep;
    expanded : boolean;
    procedure SetFocusScrollbox;
    procedure SetEnableGlyph;
  public
    { Public declarations }
    procedure MoveStep(inc : integer);
    procedure SetScriptStep(ss : TScriptStep);
    property ScriptStep : TScriptStep read fScriptStep write SetScriptStep;
    procedure BuildTitle;
  end;

type TFrameList = class(TObjectList)
    protected
        stepi : integer;
        scrollbox : TScrollbox;

    public
        procedure SetScrollbox(scrl : TScrollBox);
        function NewFrameStep(scrl : TScrollBox) : TFrameStep;
        procedure RefreshList;
        procedure AddInsert(t: TEdit);

end;
var FrameList : TFrameList;

implementation

uses UnitFrmMain, UnitFrmScriptActions, UnitFrmDummyPreview;



{$R *.dfm}

constructor TScriptStep.Create;
begin
    FrmDummyAutoPreview.InformOfKeypress;
    fenabled := true;
end;


procedure TFrameStep.btnEnableClick(Sender: TObject);
begin
    self.ScriptStep.Enabled := not self.ScriptStep.Enabled;


    self.SetEnableGlyph;
    self.BuildTitle;
end;

procedure TFrameStep.btnExpandClick(Sender: TObject);
const MIN_HEIGHT =43;
begin
    if self.Height = MIN_HEIGHT then begin
        // EXPAND
        expanded := true;
        TMyToplinePanel(pnlTop).RoundBottom := false;
        btnExpand.Glyph := FrmScriptActions.imgCollapse.Picture.Bitmap;
        self.Height := self.saveheight;

        //AnimateWindow(self.handle,400,AW_VER_POSITIVE);
        pnlMiddle.visible := true;

        btnEnable.Visible := true;
        btnUp.Visible := true;
        btnDown.Visible := true;
        btnDelete.Visible := true;
    end else begin
        // COLLAPSE
        expanded := false;
        TMyToplinePanel(pnlTop).RoundBottom := true;
        btnExpand.Glyph := FrmScriptActions.imgExpand.Picture.Bitmap;
        pnlMiddle.visible := false;
        //pnlBottom.Visible := false;
        //pnlSpacer.Visible := false;
        self.saveheight := self.height;
        self.Height := MIN_HEIGHT;

        btnEnable.Visible := false;
        btnUp.Visible := false;
        btnDown.Visible := false;
        btnDelete.Visible := false;

    end;

    self.BuildTitle;

    self.refresh;
    FrameList.scrollbox.ScrollInView(self);
end;

{ FrameList }


function TFrameList.NewFrameStep(scrl : Tscrollbox): TFrameStep;
var fs : TFrameStep;
    i : integer;
begin
    SetScrollbox(scrl);

    inc(stepi);
    fs := TFrameStep.Create(ScrollBox);
    fs.Name := fs.name + IntToStr(stepi);
    fs.Parent := scrollbox;
    fs.Align := altop;
    //fs.btnExpand.Click;
    fs.btnExpand.Glyph := FrmScriptActions.imgCollapse.Picture.Bitmap;
    fs.btnUp.Glyph := FrmScriptActions.imgUp.Picture.bitmap;
    fs.btnDown.Glyph := FrmScriptActions.imgDown.Picture.bitmap;
    fs.btnDelete.glyph := FrmScriptActions.imgDelete.Picture.Bitmap;
    fs.btnEnable.Glyph := FrmScriptActions.imgBlank.Picture.Bitmap;

    fs.pnlBottom.Free;
    fs.pnlBottom := TMyPanelBottom.Create(fs);
    fs.pnlBottom.Align := alBottom;
    fs.pnlBottom.caption := 'HELLOO';
    fs.pnlBottom.Parent := fs;
    fs.pnlBottom.Height := 7;

    fs.btnExpand.Parent := nil;
    fs.btnUp.Parent := nil;
    fs.btnDown.Parent := nil;
    fs.btnEnable.Parent := nil;
    fs.btnDelete.Parent := nil;


    fs.pnlTop.Free;
    fs.pnlTop := TMyToplinePanel.Create(fs);
    fs.pnlTop.Top := 0;
    fs.pnlTop.Align := alTop;
    fs.pnlTop.Parent := fs;
    fs.pnlTop.Height := 26;
    fs.pnlTop.OnMouseUp  := fs.pnlTopMouseUp;
    TMyToplinePanel(fs.pnlTop).RoundBottom := true;

    fs.pnlTitle.free;
    fs.pnlTitle := TTransFlowPanel.Create(fs);
     fs.pnlTitle.FlowStyle := fsLeftRightTopBottom;
        fs.pnlTitle.parent := fs.pnlTop;
        fs.pnlTitle.Height := 30;
        //fs.pnlTitle.Align := alClient;
        //fs.pnlTitle.Alignment := taLeftJustify;
        fs.pnlTitle.margins.Left := 4;
        fs.pnlTitle.Padding.left := 4;
        fs.pnlTitle.BevelOuter := bvNone;
        fs.pnlTitle.BevelKind := bkNone;
        //fs.pnlTitle.OnClick := self.ToggleScriptDetailsClick;
        fs.pnlTitle.ShowHint := false;
        fs.pnlTitle.Left := 40;
        fs.pnlTitle.Width := fs.pnlTop.Width - 140;
        fs.pnlTitle.Top := fs.pnlTop.Padding.top + 2;
        fs.pnlTitle.Height := fs.pnlTop.Height - (fs.pnlTitle.Top + 2);
        fs.pnlTitle.Anchors := [akLeft, akRight, akTop];

    fs.btnExpand.Parent := fs.pnlTop;
    fs.btnUp.Parent := fs.pnlTop;
    fs.btnDown.Parent := fs.pnlTop;
    fs.btnEnable.Parent := fs.pnlTop;
    fs.btnDelete.Parent := fs.pnlTop;
    fs.btnDelete.Left := fs.pnlTop.Left + fs.pnlTop.Width - (fs.btnDelete.Width+2);
    fs.btnDown.Left := fs.pnlTop.Left + fs.pnlTop.Width - ((fs.btnDelete.Width+2)*2);
    fs.btnUp.Left := fs.pnlTop.Left + fs.pnlTop.Width - ((fs.btnDelete.Width+2)*3);
    fs.btnEnable.Left := fs.pnlTop.Left + fs.pnlTop.Width - ((fs.btnDelete.Width+2)*4);

    //fs.AutoSize := true;
    self.Add(fs);
    //fs.Width := fs.Width - 1;
    //fs.btnExpand.Click;

    RefreshList;

    result := fs;
end;


procedure TFrameList.SetScrollbox(scrl: TScrollBox);
begin
    self.scrollbox := scrl;
end;

procedure TFrameStep.MoveStep(inc: integer);
var pos : integer;
  i: Integer;
  fs : TFrameStep;
begin
    pos := FrameList.IndexOf(self);
    if ((pos + inc) < 0) or
        ((pos + inc) >= FrameList.count) then EXIT;

    FrameList.move( pos, pos + inc);

    FrameList.RefreshList;
end;


procedure TFrameStep.pnlTopMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    if x > btnEnable.left then begin
        btnEnable.Visible := true;
        btnUp.Visible := true;
        btnDown.Visible := true;
        btnDelete.Visible := true;
    end;
end;

procedure TFrameStep.SetEnableGlyph;
begin
    btnEnable.Glyph := FrmScriptActions.imgBlank.Picture.Bitmap;
    if self.ScriptStep.Enabled  then begin
        btnEnable.Glyph := FrmScriptActions.imgCheck.Picture.Bitmap;
    end;
end;

procedure TFrameStep.SetFocusScrollbox;
begin
    if FrameList.Scrollbox.CanFocus and FrameList.Scrollbox.visible then FrameList.Scrollbox.SetFocus;
end;

procedure TFrameStep.SetScriptStep(ss: TScriptStep);
begin
    fscriptstep := ss;
    self.SetEnableGlyph;
    
end;

procedure TFrameList.RefreshList;
var i : integer;
    fs : TFrameStep;
begin
   for i := 0 to FrameList.Count - 1  do begin
        fs := TFrameStep(FrameList.items[i]);
        fs.Align := albottom;
        fs.Align := altop;
    end;
end;

procedure TFrameStep.btnUpClick(Sender: TObject);
begin
    MoveStep(-1);
    SetFocusScrollbox
end;

procedure TFrameStep.BuildTitle;
    function GetText(wc : TControl) : string;
    begin
        if wc is TLabel then
            result := TLabel(wc).caption
        else if wc is TCheckbox then
            result := TCheckbox(wc).Caption
        else if wc is TEdit then
            //result := '''' + TEdit(wc).text + ''''
            result := '''' + TEdit(wc).text + ''''
        else if wc is TComboBox  then
            result := TComboBox(wc).text
        else if wc is TRadioButton then begin
            if TRadioButton(wc).checked then begin
                result := TRadioButton(wc).Caption;
            end;
        end;

    end;
    function GetSentence() : string;
    var ssa : TScriptStepAdd;
        ssbe : TScriptStepAddBeginningEnding;
        ssp : TScriptStepAddPosition;
        ssba : TScriptStepAddBeforeAfter;
        ssne : TScriptStepReplaceNameExtension;
        ssrc : TScriptStepReplaceCharacters;
        ssrw : TScriptStepRemoveWords;
        ssrmc : TScriptStepRemoveCharacters;
        ssrms : TScriptStepRemoveSubstring;
        ssrba : TScriptStepRemoveBeforeAfter;
        ssrn : TScriptStepReplaceNumbers;

        ssr : TScriptStepReplace;
        ssrm : TScriptStepRemove;
        procedure Append(wc : TControl);   overload;
        var lbl : TLabel;
            p : TTransPanel;
            t : string;
        begin
            p := TTransPanel.Create(pnlTitle);
            p.Padding.Right := 4;
            p.padding.Left := 2;
            p.padding.top := 6;
            p.padding.bottom := 4;
            p.parent := pnlTitle;
            //p.DoubleBuffered := true;

            lbl := TLabel.Create(p);
            lbl.Parent := p;
            if not fScriptStep.Enabled  then begin
                lbl.Font.Color := clGrayText;
            end else begin
                lbl.Font.Color := clWindowText;
            end;

            lbl.Transparent := true;
            //lbl.OnClick := self.ToggleScriptDetailsClick;

            p.Alignment := taCenter;
            p.AutoSize := true;
            p.BevelEdges := [];
            p.BevelKind := bkFlat;
            p.BevelOuter := bvNone;
            t := GetText(wc);

            lbl.Caption := t;
            if wc is TEdit then begin
                lbl.Font.Style := lbl.font.style + [fsBold]
            end else if wc is TComboBox then begin
                lbl.Font.Style := lbl.font.style + [fsUnderline];
            end;

            p.Invalidate;
            result := result + t + ' ';
        end;
        procedure Append(s : string);   overload;
        var
            l : TLabel;
        begin
            l := TLabel.Create(pnlTitle);
            l.caption := s;
            Append(l);
        end;

    begin
        result := '';

        if fScriptStep is TScriptStepAdd  then begin
            case fScriptStep.ScriptType  of
                stAddBeginningEnding: Append('[Add to Beginning/Ending]  ');
                stAddPosition: Append('[Add at Position]  ');
                stAddBeforeAfter: Append('[Add Before/After]  ');
            end;

            if (Expanded) then begin
                result := 'Add to ...';
            end else begin
                ssa := TScriptStepAdd(fScriptStep);

                Append(ssa.lblAdd);
                Append(ssa.txtAdd);
                Append(ssa.lblToThe);
                Append(ssa.cbxAddPart);

                if (fscriptstep is TScriptStepAddBeginningEnding)  then begin
                    ssbe := TScriptStepAddBeginningEnding(fscriptstep);
                    Append(ssbe.lblAtThe);
                    Append(ssbe.cbxAddAtThe);
                    if ssbe.cbOnCollision.checked then begin
                        Append(ssbe.cbOnCollision);
                    end;
                end else if (fscriptstep is TScriptStepAddPosition)  then begin
                    ssp := TScriptStepAddPosition(fscriptstep);
                    Append(ssp.lblAtCharacterPosition);
                    Append(ssp.txtAddPosition); 
                end else if (fscriptstep is TScriptStepAddBeforeAfter)  then begin
                    ssba := TScriptStepAddBeforeAfter(fscriptstep);
                    Append(ssba.lblPlaceIt);
                    Append(ssba.cbxAddPlace);
                    Append(ssba.txtAfter);  
                end;
            end;
        end else if fScriptStep is TScriptStepReplace then begin
            case fScriptStep.ScriptType  of
                stReplaceNameExtension: Append('[Replace Name/Phrase]  ');
                stReplaceCharacters: Append('[Replace Characters]  ');
                stReplaceNumbers: Append('[Replace Numbers]  ');
            end;

            if (Expanded) then begin
                result := 'Replace ...';

            end else begin
                ssr := TScriptStepReplace(fScriptStep);
                if ssr is TScriptStepReplaceNameExtension then begin
                    ssne := TScriptStepReplaceNameExtension(ssr);
                    Append(ssne.lblReplaceThe);
                    Append(ssne.cbxReplacePart);
                    // Phrase
                    if ssne.cbxReplacePart.ItemIndex = 2 then begin
                        Append(ssne.txtReplacePhrase);
                    end;
                    Append(ssne.lblWith);
                    Append(ssne.txtReplaceWith);
                    if ssne.cbReplaceOnly.checked  then begin
                        Append(ssne.cbReplaceOnly);
                        Append(ssne.txtRP2X);
                        Append(ssne.lblThru);
                        Append(ssne.txtRP2Y);
                    end;

                end else if ssr is TScriptStepReplaceCharacters then begin
                    ssrc := TScriptStepReplaceCharacters(ssr);

                    Append(ssrc.lblReplaceEach);
                    Append(ssrc.txtReplaceChar1);
                    Append(ssrc.lblWithChar);
                    Append(ssrc.txtReplaceChar2);
                    Append(ssrc.lblInThe);
                    APpend(ssrc.cbxReplaceCharPart);
                end else if (ssr is TScriptStepReplaceNumbers) then begin
                    ssrn := TScriptStepReplaceNumbers(ssr);

                    Append(ssrn.lblReplaceNumbers);
                    Append(ssrn.cbxNumbers);
                    Append(ssrn.lblOfTheName);
                end;
            end;
        end else if fScriptStep is TScriptStepRemove then begin
            case fScriptStep.ScriptType  of
                stRemoveWords: Append('[Remove Words/Numbers]  ');
                stRemoveBeginningEnding: Append('[Remove Beginning/Ending]  ');
                stRemoveSubstring: Append('[Remove Substring]  ');
                stRemoveBeforeAfter: Append('[Remove Before/After]  ');
            end;


            if (expanded) then begin
                result := 'Remove ...';

            end else begin
                ssrm := TScriptStepRemove(fScriptStep);
                if (ssrm is TScriptStepRemoveWords)  then begin
                    ssrw := TScriptStepRemoveWords(ssrm);
                    Append(ssrw.lblRemoveWords);
                    Append(ssrw.cbxRemovePhrase);
                    if ssrw.cbxRemovePhrase.ItemIndex <> 2 then begin
                        Append(ssrw.txtRemovePhrase);
                    end;

                    Append(ssrw.lblFromThe);
                    Append(ssrw.cbxRemovePhrasePart);

                end else if (ssrm is TScriptStepRemoveCharacters)  then begin
                    ssrmc := TScriptStepRemoveCharacters(ssrm);
                    Append(ssrmc.lblRemoveChars);
                    Append(ssrmc.txtR2X);
                    Append(ssrmc.lblCharsFromThe);
                    Append(ssrmc.cbxRemoveCharPosition);
                    Append(ssrmc.lblOfThe);
                    Append(ssrmc.cbxRemoveCharactersPart);
                    if ssrmc.cbAndStore.checked  then begin
                        Append(ssrmc.cbAndStore);
                    end;

                end else if (ssrm is TScriptStepRemoveSubstring)  then begin
                    ssrms := TScriptStepRemoveSubstring(ssrm);
                    Append(ssrms.lblRemoveSubstring);
                    Append(ssrms.cbxRemoveType);
                    Append(ssrms.lblChars);
                    Append(ssrms.txtR3X);
                    Append(ssrms.lblCharThru);
                    Append(ssrms.txtR3Y);
                    if ssrms.cbAndMove.checked then begin
                        Append(ssrms.cbAndMove);
                        Append(ssrms.txtR3Z);
                    end;
                    if ssrms.cbAndStoreRM.checked  then begin
                        Append(ssrms.cbAndStoreRM);
                    end;

                end else if (ssrba is TScriptStepRemoveBeforeAfter)  then begin
                    ssrba := TScriptStepRemoveBeforeAfter(ssrm);

                    Append(ssrba.lblRemoveEverything);
                    Append(ssrba.cbxBeforeAfter);
                    Append(ssrba.lblThePhrase);
                    Append(ssrba.txtRemove2Phrase);
                    if ssrba.cbAndStoreRM2.checked  then begin
                        Append(ssrba.cbAndStoreRM2);
                    end;
                end;
            end;

        end;
    end;
var i : integer;
begin
    pnlTitle.Visible := false;
    for i := (pnlTitle.ControlCount - 1) downto 0 do begin
        pnlTitle.Controls[i].Free;
    end;
    pnlTitle.Visible := true;
    pnlTop.Hint := GetSentence;
    pnlTitle.Caption := ' ';
    if not Expanded then begin
        pnlTitle.Caption := ' ';
    end;
    //pnlTitle.Repaint;

end;

procedure TFrameStep.btnDeleteClick(Sender: TObject);
begin
    FrameList.Delete(FrameList.IndexOf(self));
    FrameList.RefreshList;
    SetFocusScrollbox
end;

procedure TFrameStep.btnDownClick(Sender: TObject);
begin
    MoveStep(+1);
    SetFocusScrollbox
end;


procedure TFrameList.AddInsert(t: TEdit);
var p : TPanel;
    sb : TSpeedButton;
begin
    p := TPanel.Create(t.Parent);
    sb := TSpeedButton.Create(p);

    p.ParentBackground := false;
    p.Parent := t.parent;
    p.Width := t.Width +10;
    p.Height := t.Height +4;
    p.top := t.top;
    p.left := t.left;
    p.Visible := t.Visible;
    p.BevelEdges := [beRight, beBottom];
    //p.Anchors := [akLeft, akTop, akRight];
    p.BevelKind := bkFlat;
    p.BevelOuter := bvNone;
    p.Margins.Top := 0;
    p.Padding.Left := 1;
    p.Padding.Bottom := 1;
    p.padding.top := 1;
    p.ShowHint := true;

    sb.Parent := p;
    sb.Align := alRight;
    sb.Glyph := frmmain.SpeedButton34.Glyph;
    sb.Flat := true;
    sb.OnClick := frmmain.InsertButtonClick;
    sb.Hint := frmmain.SpeedButton34.Hint;
    sb.ShowHint := true;

    t.Parent := p;
    t.Align := alClient;
    t.ShowHint := true;
    t.Hint := '';
    t.OnClick :=   FrmScriptActions.TextClick;
    t.OnMouseDown := FrmScriptActions.TextMouseDown;
    //t.OnMouseUp  := self.TextMouseUp;
    t.PopupMenu := FrmScriptActions.poRightClick;
    t.OnContextPopup := FrmScriptActions.TextContextPopup;
    t.HideSelection := false;
end;

initialization
begin
    FrameList := TFrameList.Create;
    
end;

end.
