unit UnitFrameTextfile;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, StdCtrls, Grids, ComCtrls, ExtCtrls, Buttons, Menus;

type
  TFrameTextfield = class(TFrame)
    Panel51: TPanel;
    Panel8: TPanel;
    rbTMatch: TRadioButton;
    rbTFind: TRadioButton;
    pnlCustomData: TPanel;
    SpeedButton1: TSpeedButton;
    txtTFind: TEdit;
    odTBrowse: TOpenDialog;
    lblTextFilename: TLabel;
    txtTName: TEdit;
    btnTBrowse: TSpeedButton;
    GroupBox1: TGroupBox;
    Panel3: TPanel;
    txtTSkip: TEdit;
    udTSkip: TUpDown;
    Label7: TLabel;
    txtTDelim: TEdit;
    cbTDelim: TCheckBox;
    Label3: TLabel;
    btnUseTab: TButton;
    cbxSearchWith: TComboBox;
    procedure SpeedButton1Click(Sender: TObject);
    procedure btnTBrowseClick(Sender: TObject);
    procedure txtTSkipChange(Sender: TObject);
    procedure rbTMatchClick(Sender: TObject);
//    procedure txtTNamexChange(Sender: TObject);
    procedure txtTNameChange(Sender: TObject);
    procedure BtnExampleScriptClick(Sender: TObject);
    procedure txtTFindChange(Sender: TObject);
    procedure txtTFindClick(Sender: TObject);
    procedure btnUseTabClick(Sender: TObject);
    procedure cbxSearchWithChange(Sender: TObject);
    procedure cbTDelimClick(Sender: TObject);
    procedure rbTFindClick(Sender: TObject);
  private
    //lastFileChange : boolean;
    { Private declarations }
    procedure ImportTextfileData;
    procedure GuessInitialImportSettings;
  public
    { Public declarations }
  end;

implementation

uses UnitTextFileFormat, UnitFrmMain, UnitFrmDummyPreview, UnitUtil, math, strutils,
  UnitWildcards
  , UnitColumnManager, UnitFrmDummyShowPreview, UnitFrmScriptActions,
  UnitMetaVariables;

{$R *.dfm}

const TEXT_INDEX = 6;

procedure TFrameTextfield.GuessInitialImportSettings;
var sl : TStringList;
    s1, s2 : string;

    wc : TRegExpWildcards;
    idx1, idx2 : cardinal;
    tff : TTextFileFormat;
    i,j : integer;
    FilenamesFoundInOrder : boolean;

    function CharCount(s : string; c : char) : integer;
    var i : integer;
    begin
        result := 0;
        for i := 1 to length(s) do begin
            if s[i] = c then inc(result);
        end;
    end;

    procedure UseCSVcomma;
    begin
        rbTFind.checked := true;
        self.txtTDelim.text := ',';
        self.cbTDelim.Checked := true;
    end;
    procedure UseCSVtab;
    begin
        rbTFind.checked := true;
        self.txtTDelim.text := 'TAB';
        self.cbTDelim.Checked := true;
    end;
begin
    if not FileExists(txtTName.text) then EXIT;

    sl := TStringList.Create;
    wc := TRegExpWildcards.Create;
    sl.LoadFromFile(txtTname.text);
    tff := TTextFileFormat.create;
    if sl.Count > 2 then begin
        s1 := sl.Strings[1];
        s2 := sl.Strings[2];



        self.cbTDelim.Checked := false;
        self.txtTDelim.text := ',';
        rbTMatch.checked := true;


        // 1 or more double quote surrounded items with commas between
        wc.SetSearchString(s1);
        wc.SetSearchPattern('"[^"]*"(,"[^"]*")*');
        if wc.FindFirst(idx1,idx2) then begin
            wc.SetSearchString(s2);
            if wc.FindFirst(idx1,idx2) then begin
                UseCSVcomma;
            end;
        end;

        // 1 or more double quote surrounded items with tabs between
        wc.SetSearchString(s1);
        wc.SetSearchPattern('"[^"]*"(\t"[^"]*")*');
        if wc.FindFirst(idx1,idx2) then begin
            wc.SetSearchString(s2);
            if wc.FindFirst(idx1,idx2) then begin
                UseCSVtab;
            end;
        end;

        // 2 or more items with tabs between
        wc.SetSearchString(s1);
        wc.SetSearchPattern('[^\t]*(\t[^t]*)+');
        if wc.FindFirst(idx1,idx2) then begin
            wc.SetSearchString(s2);
            if wc.FindFirst(idx1,idx2) then begin
                UseCSVtab;
            end;
        end;

        // 2 or more items with commas between
        wc.SetSearchString(s1);
        wc.SetSearchPattern('[^,]*(,[^,]*)+');
        if wc.FindFirst(idx1,idx2) then begin
            wc.SetSearchString(s2);
            if wc.FindFirst(idx1,idx2) then begin
                UseCSVcomma;
            end;
        end;


        // search for each file matched by line
        // or just found anywhere
        tff.LoadFromFile(txtTname.text);

        // look for the first file index and ensure it's actually not a folder
        j := FrmMain.FirstFileIndex;
        if not FrmDummyShowPreview.isfolder(FrmMain.lvFileList.Items[j]) then begin
            for i := j+0 to j+min(10, FrmMain.lvFileList.Items.count - 1) do begin
                if not (FrmMain.lvFilelist.Items[i].Checked) then continue;

                s1 := UnitColumnManager.GetName(i);
                FilenamesFoundInOrder := tff.FindLine(s1) = i;
                if not FilenamesFoundInOrder then BREAK;
            end;

            if FilenamesFoundInOrder then begin
                rbTFind.Checked := true;
                txtTFind.Text := '?cfn?';
                EXIT;
            end;

            for i := j+0 to j+min(10, FrmMain.lvFileList.Items.count - 1) do begin
                FilenamesFoundInOrder := tff.FindLine(s1) <> -1;
                if not FilenamesFoundInOrder then break;
            end;
            if FilenamesFoundInOrder then begin
                rbTFind.Checked := true;
                txtTFind.Text := '?cfn?';
                EXIT;
            end;
        end;

        rbTMatch.Checked := true;
    end;

    tff.free;
    sl.free;
    wc.free;
end;

procedure TFrameTextfield.rbTFindClick(Sender: TObject);
begin
    FrmMain.TrackPageChanges(rbTFind, TEXT_INDEX);
    FrmDummyAutoPreview.InformOfKeypress;
end;

procedure TFrameTextfield.rbTMatchClick(Sender: TObject);
begin
    FrmMain.TrackPageChanges(rbTMatch, TEXT_INDEX);
    FrmDummyAutoPreview.InformOfKeypress;
end;

procedure TFrameTextfield.ImportTextfileData;
var i, j {, skip} : integer;
    mi : TMenuItem;
//    rec : RenameRecord;
begin
    FrmMain.extfile1.Enabled := false;

     if FileExists(txtTName.text) then begin
        TextFileFormat.SkipedLines := SafeInt(txtTSkip.text);
        TextFileFormat.LoadFromFile(txtTName.text);
        TextFileFormat.Delim := ',';
        if txtTDelim.text <> '' then begin
            if lowercase(txtTDelim.Text) = 'tab' then begin
                TextFileFormat.Delim := #9;
            end else begin
                TextFileFormat.Delim := txtTDelim.text[1];
            end;
        end;

        FrmMain.extfile1.Clear;
        FrmMain.extfile1.Enabled := false;

        FrmMain.extfile1.Enabled := true;


        if cbTDelim.Checked then begin
            if TextFileFormat.FieldCount = 0 then begin
                mi := FrmMain.poInsert.CreateMenuItem;
                mi.Caption := 'Field 1';
                mi.Hint := '?text1?';
                mi.OnClick := FrmMain.InsertPopupItemClick;
                FrmMain.extfile1.Add(mi);
            end;

            for i := 0 to TextFileFormat.FieldCount  - 1 do begin
                    mi := FrmMain.poInsert.CreateMenuItem;
                    mi.Caption := 'Field ' + IntToSTr(i+1);
                    mi.Hint := '?text'+IntToStr(i+1)+'?';
                    mi.OnClick := FrmMain.InsertPopupItemClick;
                    FrmMain.extfile1.Add(mi);
            end;
            for i := 0 to TextFileFormat.FieldCount  - 1 do begin
                for j := 0 to 1 do begin
                end;
            end;


        end else begin
            mi := FrmMain.poInsert.CreateMenuItem;
            mi.Caption := 'Field 1';
            mi.Hint := '?text1?';
            mi.OnClick := FrmMain.InsertPopupItemClick;
            FrmMain.extfile1.Add(mi);
        end;
    end;


end;



procedure TFrameTextfield.BtnExampleScriptClick(Sender: TObject);
begin
    FrmMain.pcTemplate.ActivePageIndex := 3;
    FrmMain.btnClearOptions.click;
    FrmScriptActions.NewScriptControl(FrmMain.FlowPanel1, stReplaceNameExtension );
    FrmScriptActions.AlterStep(stReplaceNameExtension, 'txtReplaceWith', '?text1?');
    FrmScriptActions.AlterStep(stReplaceNameExtension, 'cbxReplacePart', '0');
end;

procedure TFrameTextfield.btnTBrowseClick(Sender: TObject);
begin
    if odTBrowse.InitialDir = '' then
        odTBrowse.InitialDir := ExtractFilePath(Application.ExeName);

    if odTBrowse.Execute then begin
        txtTName.Text := odTBrowse.FileName;
        self.GuessInitialImportSettings;
    end;
end;

procedure TFrameTextfield.btnUseTabClick(Sender: TObject);
begin
    txtTDelim.Text := 'TAB';
end;



procedure TFrameTextfield.cbxSearchWithChange(Sender: TObject);
begin
    pnlCustomData.Visible := cbxSearchWith.ItemIndex = 2;

    self.ImportTextfileData;
    FrmDummyAutoPreview.InformOfKeypress;
end;

procedure TFrameTextfield.SpeedButton1Click(Sender: TObject);
begin
    FrmMain.InsertButtonClick( SpeedButton1 );
end;

procedure TFrameTextfield.txtTFindChange(Sender: TObject);
begin
    self.ImportTextfileData;
    FrmDummyAutoPreview.InformOfKeypress;
end;

procedure TFrameTextfield.txtTFindClick(Sender: TObject);
begin
    FrmScriptActions.TextClick(sender);
end;

procedure TFrameTextfield.txtTNameChange(Sender: TObject);
var notempty : boolean;
begin
    if not FrmMain.Visible  then EXIT;

    if Assigned(FrmMain) then begin
        notempty  := txtTName.Text <> '';
        FrmMain.TrackPageChanges(notempty, TEXT_INDEX);
        if Assigned(FrmDummyAutoPreview) then begin
            FrmDummyAutoPreview.InformOfKeypress;
            self.ImportTextfileData;
        end;
    end;
end;

{procedure TFrameTextfield.txtTNamexXChange(Sender: TObject);
var cb : TCheckbox;
begin
    cb := TCheckBox.Create(self);
    cb.checked := txtTName.Text <> '';
    FrmMain.TrackPageChanges(cb, TEXT_INDEX);
    FrmDummyAutoPreview.InformOfKeypress;
    cb.free;

    self.ImportTextfileData;
end;
 }
procedure TFrameTextfield.txtTSkipChange(Sender: TObject);
var cb : TCheckbox;
begin
    cb := TCheckBox.Create(self);
    cb.Checked := not cbTDelim.checked;
    //Frmmain.TrackPageChanges(cb, TEXT_INDEX);
    FrmDummyAutoPreview.InformOfKeypress;
    cb.Free;

    FrmDummyAutoPreview.InformOfKeypress;
    self.ImportTextfileData;
end;

procedure TFrameTextfield.cbTDelimClick(Sender: TObject);
begin
    Frmmain.TrackPageChanges(cbTDelim, TEXT_INDEX);
    FrmDummyAutoPreview.InformOfKeypress;
end;


end.
