unit UnitFrmDummyThumbExtract;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls;

type
  TFrmDummyThumbExtract = class(TForm)
    Image1: TImage;
    btnStop: TButton;
    btnHide: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure btnHideClick(Sender: TObject);
  private
    activated : boolean;
    pic : TPicture;
    stopped : boolean;
    hidepressed : boolean;
    cache : string;
    { Private declarations }
    function InCacheAs(filename : string) : string;
    procedure SaveToCache(filename : string; bit : TBitmap);
    function FilenameToHashname(filename : string; ExistsAs : string = '') : string;
  public
    { Public declarations }
    function GetThumbBitmap(filename : string) : TBitmap;
    procedure UpdateCachedBitmap(oldname, newname : string);
    function WasHidePressed() : boolean;
  end;

var
  FrmDummyThumbExtract: TFrmDummyThumbExtract;

implementation

uses UnitFrmMain, UnitFrmConfig, UnitUtil, jpeg, UnitFormScaling;

{$R *.dfm}

{ TFrmDummyThumbExtract }

function TFrmDummyThumbExtract.GetThumbBitmap(filename: string): TBitmap;
var s : string;
begin
    try
        if stopped then begin
            result := nil;
            EXIT;
        end;
        application.ProcessMessages;
        s := self.InCacheAs(filename);
        if (s <> '') then begin
            image1.Picture.Bitmap.LoadFromFile(s);
            result := image1.Picture.Bitmap;
        end else begin
            pic.LoadFromFile(filename);
            Image1.canvas.Brush.Color := clWhite;
            Image1.canvas.Rectangle(Image1.ClientRect);
            image1.Canvas.StretchDraw(image1.Canvas.ClipRect, pic.graphic);
            result := Image1.Picture.Bitmap;
            if not frmconfig.cbNoICache.checked then
                self.SaveToCache(filename, Image1.Picture.Bitmap);
        end;
    except on e : exception do begin
        result := nil;
    end;
    end;

end;

procedure TFrmDummyThumbExtract.FormShow(Sender: TObject);
begin
    // center the form in the middle of the filelist
    self.Top := frmmain.Top  + GetFileList.Top - trunc(self.height/2)  + trunc(GetFileList.Height / 2);
    self.Left := frmmain.Left + GetFileList.Left - trunc(self.width/2)  + trunc(GetFileList.Width / 2);

    Application.ProcessMessages;
    self.stopped := false;
    self.hidepressed := false;

    FrmMain.Enabled := false;
end;

procedure TFrmDummyThumbExtract.FormActivate(Sender: TObject);
begin
    if not activated then begin
        self.FormShow(self);
    end;
    activated := true;
end;

procedure TFrmDummyThumbExtract.FormCreate(Sender: TObject);
begin
    pic := TPicture.Create;
    cache := IncludeTrailingPathDelimiter(
        ExtractFilePath(Application.ExeName)
    ) + 'Icache\';
    ForceDirectories(cache);
    TFormScaling.RegisterSelf(self);
end;

procedure TFrmDummyThumbExtract.btnStopClick(Sender: TObject);
begin
    self.stopped := true;
    self.ModalResult := mrok;
end;


procedure TFrmDummyThumbExtract.FormHide(Sender: TObject);
begin
    FrmDummyThumbExtract.Image1.Update;
    sleep(50);
    Application.ProcessMessages;

    FrmMain.Enabled := true;
end;

procedure TFrmDummyThumbExtract.btnHideClick(Sender: TObject);
begin
    self.hidepressed := true;
    self.Hide;
    GetFileList.Items.EndUpdate;
end;

function TFrmDummyThumbExtract.WasHidePressed: boolean;
begin
    result := self.hidepressed;
end;

function TFrmDummyThumbExtract.InCacheAs(filename: string): string;
var  hash : string;
begin
    result := '';

    if FileExists(filename) then begin

        hash := FilenameToHashname(filename);
        if FileExists( hash ) then begin
            result := hash;
        end;

    end;
end;

function TFrmDummyThumbExtract.FilenameToHashname(
  filename: string; ExistsAs : string = ''): string;
var s, name, location : string;
    i, j : integer;
    var f : file;
begin
    s := ExtractFilename(
     ExcludeTrailingPathDelimiter(ExtractFilePath(filename))
    );
    name := ExtractFileName(filename);


    if ExistsAs <> '' then begin
        location := ExistsAs;
    end else begin
        location := filename;
    end;
    i := FileAge(location);
    try
        AssignFile(f, location);
        FileMode := fmOpenRead;
        reset(f);
        j := FileSize(f);
    finally
        closefile(f);
    end;


    result := cache + IntToStr(i mod 100) + '\'+ IntToStr(i) +'-'+IntToStr(j)  + '-' + s + '-' + name + '.bmp';
end;

procedure TFrmDummyThumbExtract.SaveToCache(filename: string; bit: TBitmap);
var s : string;
begin
    s := self.FilenameToHashname(filename);
    ForceDirectories(extractfilepath(s));
    bit.SaveToFile(s);
end;

procedure TFrmDummyThumbExtract.UpdateCachedBitmap(oldname,
  newname: string);
var s : string;
begin
    if LongerDirectoryExists(ExcludeTrailingPathDelimiter(newname)) then EXIT;

    s := self.FilenameToHashname(oldname, newname);
    if (FileExists(s)) then begin
        RenameFile(s, self.FilenameToHashname(newname))
    end;
end;

end.
