unit UnitFrmGenerateBatch;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TfrmGenerateBatch = class(TForm)
    txtName: TLabeledEdit;
    Label1: TLabel;
    btnSave: TButton;
    Button2: TButton;
    Memo1: TMemo;
    txtCommand: TLabeledEdit;
    procedure txtNameChange(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }

  public
    { Public declarations }
    function Execute : boolean;
    procedure SetDefaults;
    procedure GenerateScript(fullname : string; makeundo : Boolean = false);
  end;

var
  frmGenerateBatch: TfrmGenerateBatch;

implementation

uses UnitUtil, UnitFrmDummyShowPreview, UnitFrmMain, UnitColumnManager, UnitFormScaling;

{$R *.dfm}

{ TfrmGenerateBatch }


procedure TfrmGenerateBatch.btnSaveClick(Sender: TObject);
begin
    ModalResult := mrOK;
end;

procedure TfrmGenerateBatch.Button2Click(Sender: TObject);
begin
    ModalResult := mrCancel;
end;

function TfrmGenerateBatch.Execute: boolean;
begin
    btnSave.Enabled := false;
	self.ShowModal;
    result := ModalResult = mrOK;
    if result then begin
        self.GenerateScript(
            IncludeTrailingPathDelimiter(unitutil.GetINIPathOnly) + txtName.text+'.bat'
        );
    end;
end;

procedure TfrmGenerateBatch.FormCreate(Sender: TObject);
begin
TFormScaling.RegisterSelf(self);
end;

procedure TfrmGenerateBatch.SetDefaults;
begin
    txtCommand.text := 'rename';
end;

procedure TfrmGenerateBatch.GenerateScript(fullname : string; makeundo : Boolean = false);
    function DoInclude(i : Integer) : Boolean;
    begin
        Result := (FrmDummyShowPreview.GetChecked(i)) and
            (UnitColumnManager.GetNewName(i) <> '') and
            (UnitColumnManager.GetNewName(i) <> UnitColumnManager.GetName(i));
    end;
var i, x : integer;
	sl : TStringList;
    oldn, newn, base : string;
begin
    sl := TStringList.Create;
    FrmDummyShowPreview.ShowPreviewList(x);
    base := IncludeTrailingPathDelimiter(FrmMain.GetPath);
    for i := 0 to frmmain.lvFileList.Items.Count -1 do begin
        oldn := UnitColumnManager.GetName(i);
        newn := UnitColumnManager.GetNewName(i);
        if DoInclude(i) then begin
            if makeundo then begin
                sl.Add(txtCommand.Text + ' "'+base+newn+'" "'+base+oldn+'.RM_TEMPNAME"' )
            end else begin
                sl.Add(txtCommand.Text + ' "'+base+oldn+'" "'+base+newn+'.RM_TEMPNAME"' )
            end;
        end;
    end;
    for i := 0 to frmmain.lvFileList.Items.Count -1 do begin
        oldn := UnitColumnManager.GetName(i);
        newn := UnitColumnManager.GetNewName(i);

        if DoInclude(i) then begin
            if makeundo then begin
                sl.Add(txtCommand.Text + ' "'+base+oldn+'.RM_TEMPNAME" "'+base+oldn+'"' )
            end else begin
                sl.Add(txtCommand.Text + ' "'+base+newn+'.RM_TEMPNAME" "'+base+newn+'"' )
            end;
        end;
    end;
    if sl.Count > 0 then begin
        sl.SaveToFile(
            fullname,
            TEncoding.Unicode
        );
    end;
    sl.free;
end;



procedure TfrmGenerateBatch.txtNameChange(Sender: TObject);
begin
	btnsave.Enabled := txtName.Text <> '';
end;

end.
