unit UnitFrmInstance;
{
    Purpose:
        Recieve data from a previous instance;

    Notes:
        This form is conditionally created for only the first instance
        of this program.        
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs;


const MYMESSAGE = WM_USER + 5;
const MYCLASS = 'RenameMaster2.FrmDummyInstance';
type
  TFrmDummyInstance = class(TForm)
    procedure FormCreate(Sender: TObject);
  private
    mycommandline : string;
    params : array[0 .. 31] of string;
    pcount : integer;

    { Private declarations }

     procedure ParseCommandLine;
  public
    procedure CreateParams(var Params: TCreateParams); override;
    function ParamCount : integer;
    function ParamStr(index : integer): string;
    { Public declarations }
    procedure GetDataMessage(var Msg: TWMCopyData); message WM_COPYDATA;
    procedure ManualSet(comandline : string);
  end;

var
  FrmDummyInstance: TFrmDummyInstance;

procedure FrmInstance_SendData;
procedure FrmInstance_Init;
procedure FrmInstance_DeInit;

implementation

uses UnitFrmMain, UnitWildcards, StrUtils, UnitUtil;

{$R *.dfm}
var mutex : THandle;
{ TFrmDummyInstance }



procedure FrmInstance_Init;
begin
    mutex := Windows.CreateMutex(nil, true, pchar(lowercase(ExtractFileName(Application.exename))));
    if (Windows.GetLastError = 0) then begin
          Application.CreateForm(TFrmDummyInstance, FrmDummyInstance);
    end else begin
        Application.Terminate;
        Application.ProcessMessages;
        
        //Application.CreateForm(TFrmDummyInstance, FrmDummyInstance);

        UnitFrmInstance.FrmInstance_SendData;

        Application.ProcessMessages;
        //Halt(0);
        // for some reason - not killing over immediately causes problems
        // in the other instance
    end;
end;

procedure FrmInstance_DeInit;
begin
    if mutex <> 0 then begin
        Windows.CloseHandle(mutex);
    end;
end;

procedure FrmInstance_SendData;
var
  cd : TCopyDataStruct;
  s, param : string;
  h : THandle;
  i : integer;
  tf : TextFile;
begin

    // rebuild a parsable version of the command line
    for i := 0 to ParamCount do begin
        param := ParamStr(i);
        if not ContainsTerm(' ', param) then begin
            s := s + param + ' ';
        end else begin
            s := s + '"' + param + '" ';
        end;
    end;
    s := s + #0;

    try

        //ParamCount;
        cd.dwData := 0;
        cd.cbData := length(s) * 2;
        cd.lpData := StrAlloc(cd.cbData);
        StrPCopy(PChar(cd.lpData), s);

        // Send copy data message
        // set result according to result returned from other instance
        h := FindWindow(MYCLASS, nil);


        if (h <> 0) then begin
            i := SendMessage(
                h,
                WM_COPYDATA,
                0,
                integer(@cd)
            );

        end;

    finally
    end;
end;

procedure TFrmDummyInstance.ManualSet(comandline : string);
begin
    self.mycommandline := comandline;
    self.ParseCommandLine;
    FrmMain.SetCommandLine;
end;
procedure TFrmDummyInstance.GetDataMessage(var Msg: TWMCopyData);
var s : string;
begin
    s := PChar(MSg.CopyDataStruct.lpData);
    self.ManualSet(s);
end;

procedure TFrmDummyInstance.CreateParams(var Params: TCreateParams);
begin
    inherited;

    StrCopy(Params.WinClassName, MYCLASS);
end;

function TFrmDummyInstance.ParamCount: integer;
begin
    result := pcount - 1; // the first paramater is not included
end;

function TFrmDummyInstance.ParamStr(index: integer): string;
begin
    result := trim(params[index]);
end;

procedure TFrmDummyInstance.FormCreate(Sender: TObject);
begin
    self.mycommandline := Windows.GetCommandLine;
    self.ParseCommandLine;
end;

procedure TFrmDummyInstance.ParseCommandLine;
var s : string;
    i, j : cardinal;
    c : char;
    wc : TWildcards;
begin
    s := trim(mycommandline);
    s := StringReplace(s, #0,'',[rfReplaceAll]);
    s := StringReplace(s, #9,'',[rfReplaceAll]);

    wc := TWildcards.Create;
    wc.SetSearchString(s);


    self.pcount := 0;
    while (s <> '') do begin
        // skip spaces

        while (s <> '') and ((s[1] = ' ') or (s[1]=#0)) do
            delete(s,1,1);

        if (s = '') then begin
            break;
        end;

        wc.SetSearchString(s);
        // determine if the next chunk is quote delimited or space delimeted
        if (s[1] = '"') then begin
            c := '"';
            wc.SetSearchPattern('"*"' );
        end else begin
            c := ' ';
            wc.SetSearchPattern('* ');
        end;

        if wc.FindFirst(i,j) then begin
            params[pcount] := AnsiDequotedStr(copy(s,i, i+j-1), '"');
            delete(s,i,i+j-1);
            inc(pcount);
        end else begin
            params[pcount] := AnsiDequotedStr(s, '"');
            inc(pcount);
            s := '';
        end;
    end;

    mycommandline := '';
    for i := 0 to pcount-1 do begin
        mycommandline := mycommandline + params[i] + #0;
    end;
end;

end.
