unit UnitFrmUndo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Vcl.ComCtrls, System.Generics.Collections;

type
  TfrmUndo = class(TForm)
    Label1: TLabel;
    btnYes: TButton;
    btnCancel: TButton;
    lvNames: TListView;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnYesClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    oldNames : TStringList;
    newNames : TStringList;
    newNamesHand : TDictionary<string, boolean>;
    tempTag: string;
  public
    { Public declarations }
    procedure SetTempTag(tag : string);
    //procedure RemoveTagFromNewName(tag : string);
    procedure ShowForm;
    procedure AddUndo(oldName, newName : string);
    procedure ClearAll;
    function UndoCount() : integer;
    function GetUndoOldName(index : integer) : string;
    function GetUndoNewName(index : integer) : string;
    function IsNewName(name : string) : boolean;
    procedure UndoNow(savetofile : string = '');
  end;

var
  frmUndo: TFrmUndo;

implementation

uses UnitFrmError, UnitUtil, UnitUndoRename, System.Math, UnitFormScaling;

{$R *.dfm}

procedure TfrmUndo.FormCreate(Sender: TObject);
begin
    oldNames := TStringList.Create;
    newNames := TStringList.Create;
    newNamesHand := TDictionary<string,boolean>.create;

    TFormScaling.RegisterSelf(self);
end;
procedure TfrmUndo.FormDestroy(Sender: TObject);
begin
    oldNames.Free;
    newNames.Free;
end;

//
// For interactive use, the actual local undo states are never used.
// When cancelled, the last state is restored. States are only stored when a
// full rename occurs with no errors.
//
// For backwards compatibility, "UndoNow" may be called during an error,
// during batch mode, and so on.
//

procedure TfrmUndo.FormShow(Sender: TObject);
var
    s : string;
    i : integer;
    li : TListItem;
begin
    UndoRename.LoadLastState;
    lvNames.Clear;
    for i := 0 to Min(oldNames.Count-1, 100) do begin
        li := lvNames.Items.Add;
        li.Caption := ExtractFileName(oldNames[i]);
        li.SubItems.Add(ExtractFileName(newNames[i]));
    end;
end;
procedure TfrmUndo.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    if self.ModalResult = mrCancel then begin
        UndoRename.CancelLastLoad;
    end;
end;

procedure TfrmUndo.btnYesClick(Sender: TObject);
begin
    self.ModalResult := mrYes;
    try
        btnyes.Enabled := false;
        btnCancel.Enabled := false;
        btnyes.Repaint;
        btnCancel.Repaint;

        Application.ProcessMessages;

        self.UndoNow;
    finally
        btnYes.Enabled := true;
        btnCancel.Enabled := true;
        Application.ProcessMessages;
    end;
end;
procedure TfrmUndo.btnCancelClick(Sender: TObject);
begin
    self.ModalResult := mrCancel;
end;


//--------------------------------------------
// public interface
//--------------------------------------------
procedure TFrmUndo.ShowForm;
begin
    self.ShowModal;
end;
procedure TFrmUndo.AddUndo(oldName, newName : string);
begin
    oldNames.Add(oldName);
    newNames.add(newName);
    newNamesHand.Add(newName, true);
end;

procedure TFrmUndo.ClearAll;
begin
    oldNames.Clear;
    newNames.Clear;
    newNamesHand.Clear;
end;

function TFrmUndo.UndoCount() : integer;
begin
    result := oldNames.Count;
end;
function TFrmUndo.GetUndoOldName(index : integer) : string;
begin
    result := oldNames[index];
end;
function TFrmUndo.GetUndoNewName(index : integer) : string;
begin
    result := newNames[index];
end;
function TfrmUndo.IsNewName(name: string): boolean;
var b : boolean;
begin
    result := newNamesHand.TryGetValue(name, b) or newNamesHand.TryGetValue(name + self.tempTag, b);
end;
procedure TfrmUndo.SetTempTag(tag: string);
begin
    self.tempTag := tag;
end;

procedure TFrmUndo.UndoNow(savetofile : string = '');
var tf : textfile;
    fs : TStreamWriter;
    {$I-}
    procedure DoRename(oldfile: string; newfile: string);
    var f: file;
        i, j : integer;
        s : string;
    begin
        try
            i := 0;
            if (savetofile <> '') then begin
                //writeln(tf, 'ren "' + oldfile + '" "' + ExtractFileName(newfile) + '" >> rm_error_log.txt');
                s := 'fso.MoveFile "' + oldfile + '", "' + newfile + '"';
                fs.WriteLine(s);
            end else begin
                if not Windows.MoveFile(pchar(oldfile), pchar(newfile)) then begin
                    j := Windows.GetLastError;
                end;
            end;
        finally
            i := IOResult;
            if i <> 0 then begin
                // TODO: We need some error handling here!
            end;
        end;
    end;
    {$I+}

var i : longint;
    s : string;
begin

    if (savetofile <> '') then begin
        fs := TStreamWriter.Create(UnitUtil.GetINIPathOnly  + savetofile, false, TEncoding.Unicode);
        fs.WriteLine('Set fso = CreateObject("Scripting.FileSystemObject")');
    end;

    //
    // 3 file states may exist
    // - file successfully renamed to newname without the temp tag
    // - file failed and has a newname and a temp tag
    // - file still has original name (nothing to undo)

    for i := (oldNames.Count - 1) downto 0 do begin

        s := newNames[i];
        if FileExists(s) then begin
            // undo a failed rename
            DoRename(
                s,
                oldNames[i] + tempTag + '2'
            );
        end else begin
            // undo a completley successfull rename
            s := StringReplace(s, tempTag, '',[rfreplaceall]);
            if FileExists(s) or LongerDirectoryExists(s) then begin
                DoRename(
                    s,
                    oldNames[i] + tempTag + '2'
                );
            end;
        end;
    end;
    for i := (oldNames.Count - 1) downto 0 do begin
        DoRename(
            oldNames[i] + tempTag + '2',
            oldNames[i]
        );
    end;

    if (savetofile <> '') then begin
        fs.Free;
    end;

    self.ClearAll;
end;

end.
