unit UnitMetaVariables;
{
    Purpose:
        Receive a string an replace metavariables with values

    Updates:
        Fix for formatting file dates
        ---------------
        EXIF tags section would consume all other tags.
        
        ---------------

        New faster method for replacing meta-variables
        New UnitFileTag metavariables support.


        ---------------
        updated to account for comma formated file sizes in the record

        -----------
        New ?cf? current folder
        faster ReplaceMetavariables

        -----------
        Use the system's format for a short date
}

{///////////////}
{//}interface{//}
{///////////////}
 {
 Then Rename Record contains all the info needed to do meta-variable
 replacements on a string.
 }
 type RenameRecord = record
    FullName : string;
    name : string;
    shortname : string;
    columnname : string;
    ext : string;
    listposition : longint;
    listtotal: longint;
    selectedposition : longint;
    selectedtotal: longint;
    foldertotal: longint;
    incrementby : longint;
    changedcount : LongInt;
    filedate : string;
    filesize : string;

    Created, Modified : integer; {FileDate}

    ScriptChanged : Boolean;
 end;

 {
 All meta-variables
 The FrmMetaVariable will generate these for the user
 }

 const VAR_REGEXP = '?reg?';
 const VAR_CUR_FOLDER = '?cf?';
 const VAR_PAR_FOLDER = '?pf?';
 Const VAR_CUR_DATE_SHORT = '?cds?';
 Const VAR_CUR_DATE_MEDIUM = '?cdm?';
 Const VAR_CUR_DATE_LONG = '?cdl?';
 const VAR_CUR_FILENAME = '?cfn?';
 const VAR_CUR_FILENAME_NOEXT = '?cfo?';

 Const VAR_DATE_SHORT = '?ds?';
 Const VAR_DATE_MEDIUM = '?dm?';
 Const VAR_DATE_LONG = '?dl?';

 const VAR_DATE_MODIFIED = '?dmo?';
 const VAR_DATE_CREATED = '?dc?';

 Const VAR_TYPE = '?t?';

 Const VAR_COUNT = '?c?';
 Const VAR_SELECTED_COUNT = '?sc?';
 Const VAR_FOLDER_COUNT = '?fc?';
 Const VAR_SIZE_COMMAS = '?zc?';
 Const VAR_SIZE_GIGS_2DEC = '?zg2d?';
 Const VAR_SIZE_GIGS_COMMAS = '?zgc?';
 Const VAR_SIZE_GIGS = '?zgn?';


 Const VAR_SIZE_MEGS_COMMAS = '?zmc?';
 Const VAR_SIZE_MEGS_2DEC = '?zm2d?';
 Const VAR_SIZE_MEGS = '?zmn?';

 Const VAR_SIZE_KILO_COMMAS = '?zkc?';
 Const VAR_SIZE_KILO_2DEC = '?zk2d?';
 Const VAR_SIZE_KILO = '?zkn?';

 Const VAR_SIZE = '?zn?';

 Const VAR_NUMBER_13 = '?n13?';
 Const VAR_NUMBER_12 = '?n12?';
 Const VAR_NUMBER_11 = '?n11?';
 Const VAR_NUMBER_10 = '?n10?';
 Const VAR_NUMBER_9 = '?n09?';
 Const VAR_NUMBER_8 = '?n08?';
 Const VAR_NUMBER_7 = '?n07?';
 Const VAR_NUMBER_6 = '?n06?';
 Const VAR_NUMBER_5 = '?n05?';
 Const VAR_NUMBER_4 = '?n04?';
 Const VAR_NUMBER_3 = '?n03?';
 Const VAR_NUMBER_2 = '?n02?';
 Const VAR_NUMBER_1 = '?n01?';
 Const VAR_NUMBER_0 = '?n00?';

 const VAR_TEMP = '?temp?';
//'--
//'-- NOTE: only used to generate a number like those above
//'--
const VAR_NUMBER_BASE = '?n';
const VAR_NUMBER_SUFFIX = '?';
const TEXT_NUMBER_BASE = '?text';
const TEXT_NUMBER_SUFFIX = '?';

// Specify which kind of counting to use
// The position in the list or a specified starting number
// For a starting number, the selected position is used to renumber

procedure SetCountingListPosition;
procedure SetCountingAt(pos: longint);
procedure IncrementBy(inc : longint);

//
// Given S and the files properties in Rec, replace all meta-variables
// with their appropriet representaion
//
function ReplaceMetaVariables(s: string; rec : RenameRecord; ForceJpeg : boolean = false; ForceMP3 : boolean = false) : string;

//
// returns true if string contains a regular expression
//
function HasRegularExpressions(s: string) : boolean;

procedure SetAsteriskChar(c : char);
procedure SetQuestionChar(c : char);
procedure SetBackslashChar(c : char);
procedure SetPipeChar(c : char);
procedure SetSlashChar(c : char);
procedure SetColonChar(c : char);
procedure SetGreaterChar(c : char);
procedure SetLessChar(c : char);
procedure SetQuoteChar(c : char);
procedure SetPeriodChar(c : char);
procedure SetTemp(t: string);

procedure ClearTextFields;
procedure SetTextFields(field : integer; s : string);
procedure SetNoCleanOnce;
procedure ClearNoCleanOnce;


function GetFriendlyName(metavar : string) : string;
function IsStaticVariable(s : string) : boolean;
{///////////////////////}
{\\}  implementation {//}
{///////////////////////}

uses
    DateUtils, StrUtils, Dialogs, SysUtils, math, UnitUtil, UnitFrmConfig, UnitFileTags, INIFiles,
    UnitToken, Classes, Windows, Variants, UnitVideo, UnitWildcards, UnitDebug,
    System.Character;
var
    UseListPosition: boolean;
    StartCountingAt: longint;
    Increment : longint;

    JSeg : TJPEGSegment;
    Asterisk : string;
    Question : string;
    Pipe : string;
    Backslash : string;
    Slash : string;
    Colon : string;
    Greater, Less, Quote : String;
    Period : string;
    MyMetaHash : THashedStringList;
    temp : string;
    TextFields : TStringList;

procedure ClearTextFields;
begin
    TextFields.Clear;
end;
procedure SetTextFields(field : integer; s : string);
begin
    textfields.Values[IntToStr(field)] := s;
end;
procedure SetTemp(t: string);
begin
    temp := t;
end;
procedure SetPeriodChar(c : char);
begin
    Period := c;
end;
procedure SetGreaterChar(c : char);
begin
    Greater := c;
end;
procedure SetLessChar(c : char);
begin
    Less := c;
end;
procedure SetQuoteChar(c : char);
begin
    Quote := c;
end;

procedure SetAsteriskChar(c : char);
begin
    Asterisk := c;
end;
procedure SetQuestionChar(c : char);
begin
    Question := c;
end;
procedure SetBackslashChar(c : char);
begin
    Backslash := c;
end;
procedure SetPipeChar(c : char);
begin
    Pipe := c;
end;
procedure SetSlashChar(c : char);
begin
    Slash := c;
end;
procedure SetColonChar(c : char);
begin
    Colon := c;
end;

procedure SetCountingListPosition;
begin
    UseListPosition := true;
end;
procedure SetCountingAt(pos: longint);
begin
    UseListPosition := false;
    StartCountingAt := pos;
end;
procedure IncrementBy(inc : longint);
begin
    Increment := inc;
end;

function GetMyMetaValue( meta : string ) : string;
begin
    if meta = '' then EXIT;
    
    result := MyMetaHash.values[lowercase(meta)];
end;
//
// Many dirty tricks where used he because I found no string utils
// comperable to VB's
//
procedure GetMetaPosition(s : string; var start : integer; var count : integer);
begin
    // find first ?
    // remove up to it
    start := Pos('?', s);
    Delete(s, 1, start);
    count := Pos('?', s) + 1;
end;
procedure GetMetaPositionEXIF(s : string; var start : integer; var count : integer);
begin
    s := lowercase(s);
    // find first ?
    // remove up to it
    start := Pos('?x', s);
    Delete(s, 1, start);
    count := Pos('?', s) + 1;
end;
procedure GetMetaPositionXMP(s : string; var start : integer; var count : integer);
begin
    s := lowercase(s);
    // find first ?
    // remove up to it
    start := Pos('?xmp_', s);
    Delete(s, 1, start);
    count := Pos('?', s) + 1;
end;
procedure GetMetaPositionMP3(s : string; var start : integer; var count : integer);
var x, y : integer;
begin
    s := lowercase(s);
    // find first ?
    // remove up to it
    x := Pos('?i', s);
    y := pos('?m', s);

    // find the smallest non-zero position
    if (x * y = 0) then begin
        start := max(x,y);
    end else begin
        start := min(x,y);
    end;

    Delete(s, 1, start);
    count := Pos('?', s) + 1;
end;
procedure GetMetaPositionJFIF(s : string; var start : integer; var count : integer);
begin
    s := lowercase(s);
    // find first ?
    // remove up to it
    start := Pos('?j', s);
    Delete(s, 1, start);
    count := Pos('?', s) + 1;
end;
procedure GetMetaPositionVideo(s : string; var start : integer; var count : integer);
begin
    s := lowercase(s);
    // find first ?
    // remove up to it
    start := Pos('?v', s);
    Delete(s, 1, start);
    count := Pos('?', s) + 1;
end;
procedure GetMetaPositionFileContents(s : string; var start : integer; var count : integer);
begin
      s := lowercase(s);
    // find first ?
    // remove up to it
    start := Pos('?fc', s);
    Delete(s, 1, start);
    count := Pos('?', s) + 1;
end;

function StripFormatFromTag(var s : string) : string;
var i : integer;
begin
    result := '';
    i := pos(':',s);
    if i > 0 then begin
        result := copy(s, i + 1, length(s) - (i+1));
        delete(s, i, length(s) - i);
    end;
end;
function FormatTag(tag, tagformat : string) : string;
var c : char;
    cnt : integer;
    dec, p : integer;
    i : integer;
    start : integer;
    MySettings: TFormatSettings;

    dt : TDateTime;

    s,op  : string;
    units, amount : integer;
    function DateTimeFix(FileTime: TDateTime): TDateTime;
    var
		LocalFileTime, Ft: TFileTime;
       	SystemTime: TSystemTime;
    begin
       	DateTimeToSystemTime(FileTime, SystemTime);
       	SystemTimeToFileTime(SystemTime, LocalFileTime);
       	LocalFileTimeToFileTime(LocalFileTime, Ft);
        FileTimeToSystemTime(ft, SystemTime);

       	result := SystemTimeToDateTime(SystemTime)
    end;
begin
    result := tag;
    tagformat := lowercase(tagformat);
    if tagformat = '' then EXIT;

    case tagformat[1] of
    'l' : begin
        try
            c := tagformat[2];
            cnt := StrToInt(copy(tagformat,3,length(tagformat)-2));
            while length(result) < cnt do begin
                result := c + result;
            end;
        except
            result := tag;
        end;
    end;
    'r' : begin
        try
            c := tagformat[2];
            cnt := StrToInt(copy(tagformat,3,length(tagformat)-2));
            while length(result) < cnt do begin
                result := result + c;
            end;
        except
            result := tag;
        end;
    end;
    // dxxx where xxx is a number
    'd' : begin
        p := pos('.', tag);
        try
            if (p = 0) then begin
                cnt := StrToInt(copy(tagformat,2,length(tagformat)-1));
                result := tag + '.';
                for i := 1 to cnt do begin
                    result := result + '0';
                end;
            end else begin
                cnt := StrToInt(copy(tagformat,2,length(tagformat)-1));
                dec := length(tag) - pos('.', tag);
                if (dec > cnt) then begin
                    delete(result, pos('.', tag), dec - cnt);
                end else begin
                    for i := 1 to cnt - dec do begin
                        result := result + '0';
                    end;
                end;
            end;
        except
            result := tag;
        end;
    end;
    // fxxxxxxx where xxxxxxx is the date format
    'f' : begin
        try
            if tag='' then begin
                result := '';
            end else begin
                dt := VarToDateTime(tag); // StrToDateTime screws up on VALID windows formats
                result := FormatDateTime(Copy(tagformat,2,length(tagformat)-1), dt);
            end;
        except  on e : exception do begin
            result := tag;
            try
                // UTC string - correct for local timezone
                if length(tag) >= 12 then begin
                    if tag[11]='T' then begin
                        dt := VarToDateTime( midstr(tag,1,10) +' '+ midstr(tag,12,8));
                        dt := DateTimeFix(dt);
                        result := FormatDateTime(Copy(tagformat,2,length(tagformat)-1), dt);
                    end;
                end;
            except
            end;
        end;
        end;
    end;

    // C[P/M][0-4]yyy[F]xxxxxx
    // plus/minus - unit, amount, format
    'c': begin
        try
            s := tagformat;
            s := UnitToken.clipOneChar(s);
            op := s[1];
            s := UnitToken.clipOneChar(s);
            units := strtoint(s[1]);
            s := UnitToken.clipOneChar(s);
            amount := StrToInt(UnitToken.TokenString(s,'f'));
            if op = 'm' then amount := amount * -1;


            dt := VarToDateTime(tag);
            case units of
            0: dt := IncMinute(dt, amount);
            1: dt := IncHour(dt, amount);
            2: dt := IncDay(dt, amount);
            3: dt := IncMonth(dt, amount);
            4: dt := IncYear(dt, amount);
            end;

            result := FormatDateTime(s, dt);
        except  on e : exception do begin

            result := tag;
        end;
        end;

    end;
    // HXXX where XXX is a number
    'h' : begin
        try
            cnt := StrToInt(copy(tagformat,2, length(tagformat)-1));
            result := IntToHex(StrToInt(tag), cnt);
        except
            result := tag;
        end;
    end;
    's' : begin
        try
            tagformat := rightstr(tagformat, length(tagformat)-1);
            i := StrToInt(UnitToken.TokenString(tagformat,'.'));
            cnt := StrToInt(tagformat);
            cnt := cnt - i + 1;
            result := MidStr(tag, i, cnt);
        except
            result := tag;
        end;
    end;
    't' : begin // numbers to letters
        try
            i := StrToInt(tag);
            result := '';


            start := 0;
            if (length(tagformat) > 1) then begin
                case tagformat[2] of
                '0': begin
                    start := 0;
                end;
                '1': begin
                     start := 1;
                end;
                end;
            end;



            if (i = 0) then begin
                result := 'A';
            end;
            while i <> 0 do begin
                if start = 1 then begin
                    i := i - 1;
                end;
                c := Chr((i mod 26) + 65);
                result := c  + result;

                i := i div 26;
                start := 1;
            end;

        except
            result := tag;
        end;
    end;
    end;
end;

var NoCleanOnce : boolean;
procedure SetNoCleanOnce;
begin
    NoCleanOnce := true;
end;
procedure ClearNoCleanOnce;
begin
    NoCleanOnce := False;
end;

function ReplaceMetaVariables(s: string; rec : RenameRecord; ForceJpeg : boolean = false; ForceMP3 : boolean = false) : string;
    function zeropad(s: string; count: integer): string;
    begin
        while length(s) < count do begin
            s := '0' + s
        end;

        result := s;
    end;
    function CleanReservedChars(s : string) : string;
    var i : integer;
    begin
        if NoCleanOnce then begin
            result := s;
            NoCleanOnce := false;
            EXIT;
        end;
        for i := 1 to length(s) do begin
            case s[i] of
            '*': s[i] := Asterisk[1];
            '\': s[i] := Backslash[1];
            '/': s[i] := Slash[1];
            ':': s[i] := colon[1];
            '|': s[i] := pipe[1];
            '?': s[i] := Question[1];
            '.': s[i] := Period[1];
            end;
        end;
        {
        // lazy, lazy, lazy - and slow
        s := stringreplace(s, '*', Asterisk, [rfreplaceall]);
        s := stringreplace(s, '\', Backslash, [rfreplaceall]);
        s := stringreplace(s, '/', Slash, [rfreplaceall]);
        s := stringreplace(s, ':', Colon, [rfreplaceall]);
        s := stringreplace(s, '|', Pipe, [rfreplaceall]);
        s := stringreplace(s, '?', Question, [rfreplaceall]);
        s := stringreplace(s, '.', Period, [rfreplaceall]);
        }
        result := s;
    end;
var t,t2, t3 : string;
    rc : string;
    filesize: Extended;
    filesizekb: Extended;
    filesizemb: Extended;
    filesizegb: Extended;
    giga : cardinal;
    mega : cardinal;
    kilo : cardinal;

    // temporary variables used for dirty tricks
    c: cardinal;
    v, sc: string;
    tags : THashedSTringList;
    meta, actualmeta, tagabr, tagname, tagvalue, tagformat, temps,
    formattedTag : string;
    start, count : integer;
    wc : TRegExpWildcards;
    idx1, idx2 : cardinal;

    fs : TFileStream;
    isUnicode : boolean;
    ansistr : AnsiString;
    contentlen : integer;
begin

    if frmconfig.cbDetectEnvVars.Checked then begin
        if pos('%',s) <> 0 then begin
        	wc := TRegExpWildcards.Create;
            wc.SetSearchPattern('%[^%]+%');
            wc.SetSearchString(s);
            if (wc.FindFirst(idx1, idx2)) then begin
                repeat
                	meta := copy(s,idx1, (idx2-idx1)+1);
                    tagvalue := CleanReservedChars(UnitUtil.ExpandEnvironment(meta));
                    s := stringreplace(s,meta,tagvalue,[rfIgnoreCase, rfReplaceAll]);
                until not wc.FindNext(idx1, idx2);
            end;
            wc.Free;
        end;
    end;

    // speed things up when no metavariables are used
    if (pos('?',s) = 0) then begin
        result := s;
        EXIT;
    end;
    rec.filesize := stringreplace(rec.filesize,',','',[rfreplaceall]);

    //--
    //-- Replaces all variables with data
    //-- Use the file's date/size/extenstion as stated in the mf.FileList
    //--

    // KLUDGE: use short filename for file reads
    if rec.shortname <> '' then begin
        rec.fullname := IncludeTrailingPathDelimiter(ExtractFilePath(rec.FullName)) + rec.shortname;
    end;
    {$REGION 'Video Tags'}
    t := s;
    if UnitVideo.FileSupported(rec.FullName) then begin
        if ContainsTerm('?v_', lowercase(s)) then begin
            temps := lowercase(rec.ext);
            tags := THashedStringList.Create;
            if UnitVideo.GetVideoTags(rec.FullName, tags) then begin

                
                GetMetaPositionVideo(t, start, count);
                while (start*count <> 0) do begin
                    meta := uppercase(copy(t,start, count));
                    actualmeta := meta;
                    tagformat := StripFormatFromTag(meta);
                    tagabr := copy(meta, 2, length(meta) -2); // remove ? and ?

                    if meta <> '' then begin
                        tagvalue := tags.values[meta];
                    end else begin
                        tagvalue := ':ERROR:';
                    end;

                    if (frmconfig.cbIgnoreEmpty.checked) and (tagvalue = '') then begin
                    	t := stringreplace(t, actualmeta,
                        stringreplace(actualmeta,'?','|',[rfReplaceALl]) ,
                        [rfreplaceall,rfignorecase]);
                	end else begin
                        formattedTag := FormatTag(tagvalue, tagformat);
                        formattedTag := CleanReservedChars(formattedTag);

                        t := stringreplace(t, actualmeta, formattedTag, [rfreplaceall,rfignorecase]);
                	end;
                    GetMetaPositionVideo(t, start, count);
                end;
                tags.free;
            end;
        end;
    end;
    {$ENDREGION}
    {$REGION 'MP3 tags'}
    if (lowercase(rec.ext) = 'mp3') then begin
        if (ContainsTerm('?i_', lowercase(s))){pos('?i_', lowercase(s)) > 0} or
            (ContainsTerm('?m_', lowercase(s))){pos('?m_', lowercase(s)) > 0} then begin
            tags := THashedStringList.Create;
            if (ContainsTerm('?m_', lowercase(s))){Pos('?m_', lowercase(s)) > 0)} then begin
                UnitFileTags.ReadMP3Full(rec.FullName, tags);
            end else begin
                UnitFileTags.ReadMP3(rec.FullName, tags);
            end;

            GetMetaPositionMP3(t, start, count);
            while (start <> 0) and (count <> 0) do begin
                meta := uppercase(Copy(t, start, count));
                actualmeta := meta;
                tagformat := StripFormatFromTag(meta);
                tagabr := copy(meta, 2, length(meta) -2); // remove ? and ?

                if (tagabr = 'I_TR') and (tags.Values[GetMP3TagName(I_Tr)]<>'') then begin
                    tagvalue :=ZeroPad(tags.Values[GetMP3TagName(I_Tr)],2);
                end else begin
                    tagname := GetMP3TagName(tagabr);
                    if tagname <> '' then begin
                        tagvalue := tags.values[tagname];
                    end else begin
                        tagvalue := ':ERROR:';
                    end;
                end;
                    if (frmconfig.cbIgnoreEmpty.checked) and (tagvalue = '') then begin
                    	t := stringreplace(t, actualmeta,
                        stringreplace(actualmeta,'?','|',[rfReplaceALl]) ,
                        [rfreplaceall,rfignorecase]);
                	end else begin
                        formattedTag := FormatTag(tagvalue, tagformat);
                        formattedTag := CleanReservedChars(formattedTag);
                		t := stringreplace(t, actualmeta, formattedTag, [rfreplaceall,rfignorecase]);
                	end;
                GetMetaPositionMP3(t, start, count);
            end;
            t := StringReplace(t, '|', '?', [rfReplaceAll]);
            tags.free;
        end;
    end;
    {$ENDREGION}
    {$REGION 'JPEG tags'}
    if (lowercase(rec.ext) = 'jpg') or (lowercase(rec.ext) ='jpeg') then begin

        if (ContainsTerm('?x_', lowercase(s)) {pos('?x_', lowercase(s)) > 0)} or
            (ContainsTerm('?j_', lowercase(s))){pos('?j_', lowercase(s)) > 0}) or
            (ContainsTerm('?xmp_', lowercase(s)))  then begin
            tags := THashedStringList.Create;
            UnitFileTags.ReadJPEG(
                rec.FullName,
                tags);
            // JFIF tags
            if (ContainsTerm('?j_', lowercase(s)))  then begin
                GetMetaPositionJFIF(t, start, count);
                while (start <> 0) and (count <> 0) and
                    (ContainsTerm('?j_', lowercase(t))) do begin
                    meta := Copy(t, start, count);
                    actualmeta := meta;
                    tagformat := StripFormatFromTag(meta);
                    tagabr := copy(meta, 2, length(meta) -2); // remove ? and ?
                    meta := lowercase(meta);
                    if meta = '?j_w?' then begin
                        tagvalue := tags.Values[JFIF_WIDTH];
                    end else if meta = '?j_h?' then begin
                        tagvalue := tags.Values[JFIF_HEIGHT];
                    end else if meta = '?j_b?' then begin
                        tagvalue := tags.Values[JFIF_PRECISION];
                    end else begin
                        tagvalue := ':ERROR:';
                    end;
                    if (frmconfig.cbIgnoreEmpty.checked) and (tagvalue = '') then begin
                    	t := stringreplace(t, actualmeta,
                        stringreplace(actualmeta,'?','|',[rfReplaceALl]) ,
                        [rfreplaceall,rfignorecase]);
                	end else begin
                		t := stringreplace(t, actualmeta, FormatTag(tagvalue, tagformat), [rfreplaceall,rfignorecase]);
                	end;
                    GetMetaPositionJFIF(t, start, count);
                end;
                t := StringReplace(t, '|', '?', [rfReplaceAll]);
            end;
            // EXIF tags
            // FIX: New "GetMeta" for EXIF only tags
            if (ContainsTerm('?x_', lowercase(s))) or ForceJPEG then begin
                GetMetaPositionEXIF(t, start, count);
                while (start <> 0) and (count <> 0) and
                    (ContainsTerm('?x_', lowercase(t))) do begin
                    meta := Copy(t, start, count);
                    actualmeta := meta;
                    tagformat := StripFormatFromTag(meta);
                    tagabr := copy(meta, 2, length(meta) -2); // remove ? and ?


                    // given an abreviation,
                    // get the real tag name
                    // and replace the meta with with the value of that tag
                    tagname :=  JSeg.GetTagName(tagabr);
                    tagvalue :=  tags.values[tagname];
                    if tagname = '' then begin
                        tagvalue := ':ERROR:';
                    end else begin
                        // date values should be in a format that
                        // can be converted at this point
                        tagvalue := FormatTag(tagvalue, tagformat);
                        tagvalue := CleanReservedChars(tagvalue);
                    end;

                    if (frmconfig.cbIgnoreEmpty.checked) and (tagvalue = '') then begin
                    	t := stringreplace(t, actualmeta,
                        stringreplace(actualmeta,'?','|',[rfReplaceALl]) ,
                        [rfreplaceall,rfignorecase]);
                	end else begin
                		//t := stringreplace(t, actualmeta, FormatTag(tagvalue, tagformat), [rfreplaceall,rfignorecase]);
                        t := stringreplace(t, actualmeta, tagvalue, [rfreplaceall,rfignorecase]);
                	end;
                    GetMetaPositionEXIF(t, start, count);
                end;
                t := StringReplace(t, '|', '?', [rfReplaceAll]);

            end;
            tags.free;

            if ContainsTerm('?xmp_', lowercase(s)) or ForceJPEG then begin
                GetMetaPositionXMP(t, start, count);

                while (start <>0) and (count<>0) and ContainsTerm('?xmp_', t) do begin
                    meta := copy(t, start, count);
                    actualmeta := meta;
                    tagformat := StripFormatFromTag(meta);
                    temps := t;
                    TokenString(temps,'?xmp_',false); {delete prefix}
                    tagname := TokenString(temps,'?',false);
                    tagValue := FormatTag(UnitFileTags.GetXMPValue(meta), tagFormat);
                    tagvalue := CleanReservedChars(tagvalue);
                    if (frmconfig.cbIgnoreEmpty.checked) and (tagvalue = '') then begin
                    	t := stringreplace(t, actualmeta,
                        stringreplace(actualmeta,'?','|',[rfReplaceALl]) ,
                        [rfreplaceall,rfignorecase]);
                	end else begin
                		t := stringreplace(t, actualmeta, tagvalue, [rfreplaceall,rfignorecase]);
                	end;
                    GetMetaPositionXMP(t, start, count);
                end;
                t := StringReplace(t, '|', '?', [rfReplaceAll]);
            end;
        end;
    end;
    {$ENDREGION}
    {$REGION 'FileContents Tags'}
    if ContainsTerm('?fcp', lowercase(s)) or ContainsTerm('?fcu', lowercase(s))  then begin
        GetMetaPosition(t,start,count);
        while (start*count <> 0) do begin
            meta := lowercase(Copy(t,start,count));
            actualmeta := meta;
            tagformat := StripFormatFromTag(meta);

            //?fc[u/]_A.B?
            if Length(meta)>=4 then
                isUnicode := meta[4] = 'u';
            fs := TFileStream.Create(rec.FullName, fmOpenRead);
            temps := ' ';
            fs.Seek(0, soFromBeginning);
            fs.Read(temps[1],1);
            if Ord(temps[1]) = TEncoding.Unicode.GetPreamble[0] then begin
                isUnicode := True;
            end;

            TokenString(meta,'_');
            temps := TokenString(meta,'.');
            idx1 := SafeInt(temps,1) - 1;
            temps := TokenString(meta,'?');
            idx2 := SafeInt(temps, 2) - 1;

            contentlen := 0;
            if idx2 > idx1 then
                contentlen := (idx2-idx1 + 1);
            if isUnicode then begin
                contentlen := contentlen * 2;
            end;

            if (idx1 > idx2) or (idx1 > fs.Size)
                or ((idx1+contentlen) > fs.size)
                or (contentlen = 0) then begin
                tagvalue := ':ERROR:';
            end else begin
                if isUnicode then begin
                    fs.Seek(idx1 * 2,soFromBeginning);

                    SetLength(temps, contentlen div 2);
                    fs.Read(temps[1], contentlen);
                    tagvalue := CleanReservedChars(temps);
                end else begin
                    fs.Seek(idx1,soFromBeginning);

                    setlength(ansistr, contentlen);
                    fs.Read(ansistr[1], contentlen);
                    tagvalue := CleanReservedChars(string(ansistr));
                end;
            end;
            t := stringreplace(t,
                actualmeta, FormatTag(tagvalue, tagformat),
                [rfreplaceall, rfignorecase]
            );
            fs.Free;

            GetMetaPositionFileContents(t, start, count);
        end;
    end;
    {$ENDREGION}
    //
    // calculate a GB, MB, and KB
    // and get the file size in each unit
    //
    giga := trunc(power(2,  30));
    mega := trunc(power(2, 20));
    kilo := trunc(power(2, 10));

    filesize := StrToint64( UnitUtil.StripNumberFormat(rec.filesize));
    filesizekb := (filesize) / kilo;
    filesizemb := (filesize) / mega;
    filesizegb := (filesize) / giga;

    //-- AutoNumber
    // use the appropriete position zeropadded to make all the numbers
    // numerically sortable and the same length
    //
    if (UseListPosition) then begin
        c := rec.listposition;
    end else begin
        try
            c := (StartCountingAt + (rec.selectedposition - 1) * increment );
        except
            c := 0;
        end;
    end;
    sc := inttostr(c);


    GetMetaPosition(t, start, count);
    while (start <> 0) and (count <> 0) do begin
        meta := copy(t, start, count);
        actualmeta := meta;
        tagformat := StripFormatFromTag(meta);
        v := GetMyMetaValue(meta);
        if (v = '') then BREAK;

        case StrToInt(v) of
           0: begin //-- Current Foldername
               tagvalue :=ExtractFileName(ExcludeTrailingPathDelimiter(ExtractFilePath(rec.FullName)));
           end;
            //-- Current Date
           1: begin
                tagvalue := CleanReservedChars( FormatDateTime('ddddd',date));
                if (tagformat <> '') then begin
                    if (lowercase(tagformat[1]) = 'f') or (lowercase(tagformat[1]) = 'c') then begin
                        tagvalue := FormatDateTime('ddddd',date);
                    end;
                end;
            end;
           2: begin
                tagvalue := CleanReservedChars( formatdatetime('dd-mmm-yyyy', date));
                if (tagformat <> '') then begin
                    if (lowercase(tagformat[1]) = 'f')  or (lowercase(tagformat[1]) = 'c') then begin
                        tagvalue := FormatDateTime('ddddd',date);
                    end;
                end;
           end;
           3: begin
                tagvalue := CleanReservedChars(FormatDateTime('dddddd', date));
                if (tagformat <> '') then begin
                    if (lowercase(tagformat[1]) = 'f')  or (lowercase(tagformat[1]) = 'c') then begin
                        tagvalue := FormatDateTime('ddddd',date);
                    end;
                end;
            end;

            //-- File Date
           4: begin
                if (tagformat = '') then begin
                    tagvalue := CleanReservedChars(FormatDateTime('ddddd', StrToDateTime(rec.filedate )));
                end else begin
                    tagvalue := rec.filedate;
                end;
            end;
           5: begin
                if (tagformat = '') then begin
                    tagvalue := FormatDateTime('dd-mmm-yyyy', StrToDateTime(rec.filedate));
                end else begin
                    tagvalue := rec.filedate;
                end;
           end;
           6: begin
                if (tagformat = '') then begin
                    tagvalue := CleanReservedChars(FormatDateTime('dddddd', StrToDateTime(rec.filedate)));
                end else begin
                    tagvalue := rec.filedate;
                end;

            end;
           7:  tagvalue := rec.ext;

           //-- File Counts
           8:    tagvalue := IntToStr(rec.listtotal );
           9:     tagvalue := IntToStr(rec.selectedtotal );
           10:     tagvalue := IntToStr(rec.foldertotal);
           // -- file sizes
           11:  {commas}begin
               tagvalue := Format('%.0n',[filesize]);
           end;
           12: {gig.00} tagvalue := format('%f',[filesizegb]);
           13:  {gig w commas}begin
               c := trunc(filesizegb);
                filesizegb := c;

                tagvalue := format('%.0n',[filesizegb]);
           end;
           14: {gig}tagvalue := inttostr(trunc(filesizegb));
           15: {megs commas}begin;
               c := trunc(filesizemb);
                filesizemb := c;
                rc := CleanReservedChars( Format('%.0n',[filesizemb]));
                tagvalue := rc;
            end;
           16: {megs.00}    tagvalue:= Format('%f',[filesizemb]);
           17: {megs}tagvalue :=  inttostr(trunc(filesizemb));
           18: {kb commas} begin
               c := trunc(filesizekb);
                filesizekb := c;
                rc := CleanReservedChars(Format('%.0n',[filesizekb]));
                tagvalue :=  rc
           end;
           19: {kb 2 decimal}begin
               rc := CleanReservedChars( Format('%f',[filesizekb]));
                tagvalue := rc ;
           end;
           20:  {kb}tagvalue := inttostr(trunc(filesizekb));
           21: begin
               rc := CleanReservedChars ( floattostr(filesize));
                tagvalue := rc;
            end;
           // -- file numbering
           22:     tagvalue := zeropad(sc,13);
           23:     tagvalue := zeropad(sc,12);
           24:     tagvalue := zeropad(sc,11);
           25:     tagvalue := zeropad(sc,10);
           26:     tagvalue := zeropad(sc,9);
           27:     tagvalue := zeropad(sc,8);
           28:     tagvalue := zeropad(sc,7);
           29:     tagvalue := zeropad(sc,6);
           30:     tagvalue := zeropad(sc,5);
           31:     tagvalue := zeropad(sc,4);
           32:     tagvalue := zeropad(sc,3);
           33:     tagvalue := zeropad(sc,2);
           34:     tagvalue := zeropad(sc,1);
           35:     tagvalue := sc;

           36: {Current Filename} begin
                tagvalue := rec.name;
           end;
           37: {current filename only - no ext} begin
               tagvalue := leftstr(rec.name, length(rec.name)-length(ExtractFileExt(rec.name)));
           end;
           38: {created}
           begin
               tagvalue := DateTimeToStr(FileDateToDateTime(rec.Modified));
           end;
           39: {modified}
           begin
                tagvalue := DateTimeToStr(FileDateToDateTime(rec.Created));
           end;
           40: {parent folder}
           begin
                t2 := ExcludeTrailingPathDelimiter(
                    ExtractFilePath(ExtractFilePath(rec.FullName))
                );
                t3 := ExtractFileName( t2 );
                t3 := leftstr(t2, length(t2)-length(t3));

                t3 := ExtractFileName( ExcludeTrailingPathDelimiter(t3) );

                tagvalue := t3;

           end;
           41: {temp variable}
            begin
                tagvalue := temp;
            end;
           42: {REG variable - ignore}
            begin

            end;
           43..43+99: {text field variables}
            begin
                tagvalue := TextFields.values[IntToStr(StrToInt(v)-43)];
            end
        else begin
            tagvalue := ':ERROR:';
        end;
        end;

        tagvalue := FormatTag(tagvalue, tagformat);
        tagvalue := CleanReservedChars(tagvalue);
        if (frmconfig.cbIgnoreEmpty.checked) and (tagvalue = '') then begin
            t := stringreplace(t, actualmeta,
            stringreplace(actualmeta,'?','|',[rfReplaceALl]) ,
            [rfreplaceall,rfignorecase]);
        end else begin
            t := stringreplace(t, actualmeta, tagvalue, [rfreplaceall,rfignorecase]);
        end;
        GetMetaPosition(t, start, count);

    end;
    t := StringReplace(t, '|', '?', [rfReplaceAll]);
    // for ###,###,###,##0
    // "%n" is used for thousand's separaters and the trailing .00 is removed
    // for 0.00
    // "%f" is used

    result := t;
end;

function IsStaticVariable(s : string) : boolean;
begin
    result := GetMyMetaValue(s) <> '';
end;
function GetFriendlyName(metavar : string) : string;
var
    ex : EXIF_Enum;
    function getAbbr(metavar : string) : string;
    var
        c : Char;
        lastWasUnderscore : boolean;
    begin
        result := '';
        lastWasUnderscore := false;
        if (Pos('_',metavar) <> 0) then TokenString(metavar,'_');
        for c in metavar do begin
            if TCharacter.IsUpper(c) or lastWasUnderscore then begin
                result := result + c;
            end;
            lastWasUnderscore := c = '_';
        end;
    end;
begin
    result := 'none';
    StripFormatFromTag(metavar);
    try
        case StrToInt(GetMyMetaValue(lowercase(metavar))) of
        0:      result := 'Cur Folder';
        1..3:   result := 'Cur Date';
        4..6:   result := 'File Date';
        7 :     result := 'Ext';
        8:      result := 'Total';
        9:      result := 'Selected';
        10:     result := 'Folder Cnt';
        11..21: result := 'Filesize';
        22..35: result := 'Counter';
        36..37: result := 'File';
        38:     result := 'Date Mod';
        39:     result := 'Date Create';
        40:     result := 'Parent';
        41:     result := 'Temp Var';
        42:     result := 'RegExp';
        43..(43+99):  result := 'TextField';
        else
        end;
    except
        if StartsText('?v',metavar) then begin
            result := 'Video';
        end else if StartsText('?i',metavar) or StartsText('?m',metavar) then begin
            result := UnitFileTags.GetMP3TagName(metavar);
        end else if StartsText('?xmp',metavar) then begin
            result := 'XMP ';
            result := result + lowercase(getAbbr(metavar));
        end else if StartsText('?x',metavar) then begin
            result := 'JPG EXIF';
            if not UnitFileTags.GetJPGExifEnum(metavar, ex) then EXIT;
            TokenString(metavar,'_');
            metavar := TokenString(metavar,'?');
            result := 'JPG ' + lowercase(metavar);
        end else if StartsText('?j',metavar) then begin
            result := 'JPG';
            if StartsText('?j_w',metavar) then begin
                result := 'JPG wd'
            end else if StartsText('?j_h',metavar) then begin
                result := 'JPG ht';
            end else if StartsText('?j_b',metavar) then begin
                result := 'JPG p';
            end;
        end else if StartsText('?f',metavar) then begin
            result := 'Contents';
        end;
    end;
end;
function HasRegularExpressions(s: string) : boolean;
begin
    result := (StringIn('?', s)) or (StringIn('*', s));
end;

procedure InitMyMetaHash;
var i, x : integer;
    procedure add(s : string);
    begin
        MyMetaHash.values[s] := IntToStr(i);
        inc(i)
    end;
begin
    i := 0;

    MyMetaHash := THashedStringList.Create;
    add('?cf?');
    add('?cds?');
    add('?cdm?');
    add('?cdl?');
    add('?ds?');
    add('?dm?');
    add('?dl?');
    add('?t?');
    add('?c?');
    add('?sc?');
    add('?fc?');
    add('?zc?');
    add('?zg2d?');
    add('?zgc?');
    add('?zgn?');
    add('?zmc?');
    add('?zm2d?');
    add('?zmn?');
    add('?zkc?');
    add('?zk2d?');
    add('?zkn?');
    add('?zn?');
    add('?n13?');
    add('?n12?');
    add('?n11?');
    add('?n10?');
    add('?n09?');
    add('?n08?');
    add('?n07?');
    add('?n06?');
    add('?n05?');
    add('?n04?');
    add('?n03?');
    add('?n02?');
    add('?n01?');
    add('?n00?');
    add('?cfn?');
    add('?cfo?');
    add(VAR_DATE_MODIFIED);
    add(VAR_DATE_CREATED);
    add(VAR_PAR_FOLDER);
    add(VAR_TEMP);
    Add(VAR_REGEXP);

    for x := 1 to 99 do begin
        Add('?text' +IntToStr(x)+ '?');
    end;

end;

initialization
begin
    Question := '''';
    Pipe := '''';
    Backslash := '-';
    Slash := '-';
    Colon := '''';
    Asterisk := '''';
    Period := '.';
    Increment := 1;
    
    JSeg := TJPEGSegment.Create;
    InitMyMetaHash;
    TextFields:= TStringList.Create;
end;
end.
