unit UnitRMScript;
{
    Purpose:
        Save or load an RM script

    Updates:
        Updated for new "This" item in REPLACE2 option

        Overrided INIFIle's Write/ReadString functions to keep
        leading and trailing space
}
interface
uses INIFiles, SysUtils {ExtractFilePath}, Forms {Application};
type TRMScript = class(TObject)
    private
        Filename : string;
        batchmode : boolean;
        UseFilter : boolean;
        UsePath : boolean;
        fversion : string;
    public
        procedure SetFilename(f : string);
        procedure ToggleFilter(b : boolean);
        procedure TogglePath(b : boolean);
        function GetUseFilter : boolean;
        function GetUsePath : boolean;
        procedure SaveSettings(renameoptions : boolean = true);
        procedure LoadSettings(renameoptions : boolean = true);
        function DefaultExists : boolean;
        procedure SetBatchMode(b : boolean);

        property version : string read fversion write fversion;
end;


type TMyINIFile = class(TMemINIFile)
    private
        mini : TMemIniFile;
        dirty : boolean;
    public
      constructor Create(filename : string);
      destructor Destroy; override;
        procedure WriteString(const Section, Ident, Value: String); override;
        function ReadString(const Section, Ident, Default: String): String; override;
        function ValueExists(const Section, Ident : string) : boolean; override;
end;
var RMScript : TRMSCript;
const DEFAULT_SCRIPT = 'default.rm-';
const NEW_DEFAULT_SCRIPT = 'default.rmscr';

{////////////////////}
{//}implementation{//}
{////////////////////}

uses UnitFrmMain, StrUtils, UnitUtil, UnitFrmAbout;


procedure TMyINIFile.WriteString(const Section, Ident, Value: String);
begin
    // we need to wrap strings with
    // leading/trailing spaces
    // strings that start or end with our sentinal values (quote & space)

    if (trim(value) <> value) or
        (leftstr(value,2) = '" ') or
        (rightstr(value,2) = ' "') then begin
        //inherited WriteString(section, ident, '"' + value + '"');

        inherited WriteString(Section, Ident,'');
        mini.writestring(section,ident+'_unicode', '"' + value + '"');
    end else begin
        //inherited WriteString(section, ident, value);
        inherited WriteString(Section, Ident,'');
        mini.writestring(section,ident+'_unicode', value);
    end;

    dirty := true;
end;
constructor TMyINIFile.Create(filename : string);
begin
    inherited Create(filename);

    dirty := false;
    mini := TMemIniFile.Create(filename, TEncoding.Unicode);
end;
destructor TMyINIFile.Destroy;
begin
    inherited Destroy;

    mini.UpdateFile;
    mini.Free;
end;


function TMyINIFile.ReadString(const Section, Ident, Default: String): string;
var s : string;
begin

	if self.ValueExists(section, ident+'_unicode') then begin
        result := mini.ReadString(section, ident +'_unicode', default);
    end else begin
    	result := inherited ReadString(section, ident, default);
    end;


    // strip if begins and ends with double quotes
    // strip if double quotes encodes trailing or leading spaces
    // or if original data contained our sentinal values and had
    // to be encoded
    if result <> '' then

    if  ((leftstr(result,1) = '"') and (rightstr(result,1) = '"')) and

        ((leftstr(result,2) = '" ') or (rightstr(result,2) = ' "') or
         (leftstr(result,3) = '"" ') or (rightstr(result,2) = ' ""'))

         then begin
        result := leftstr(result, length(result) -1);
        result := rightstr(result, length(result) -1);
    end;
end;
function TMyINIFile.ValueExists(const Section, Ident : string) : boolean;
begin
    result := inherited ValueExists(section, ident) or inherited valueexists(section, ident+'_unicode');
end;



procedure TRMScript.SetBatchMode(b: boolean);
begin
    self.batchmode := b;
end;

procedure TRMSCript.SetFilename(f : string);
begin
    Filename := f;
end;


procedure TRMScript.ToggleFilter(b: boolean);
begin
    UseFilter := b;
end;

procedure TRMScript.TogglePath(b: boolean);
begin
    UsePath := b;
end;

const SCRIPT_OPTIONS = 'ScriptOptions';
const ADD_TO = 'AddTo';
const ADD_TO2 = 'AddTo2';
const REPLACE = 'Replace';
const REPLACE2 = 'Replace2';
const REMOVE = 'Remove';
const REMOVE2 = 'Remove2';
const COUNTING = 'Counting';
const OVERRIDECASE = 'Case';
const WILDCARDS = 'Wildcards';
const TEXT = 'Text';

procedure TRMSCript.SaveSettings(renameoptions : boolean = true);
var ini : TMyIniFile;
begin
    ini := TMyIniFile.Create(Filename);
    if (self.UsePath) then begin
        ini.WriteString('Path', 'cbPath', frmMain.GetPath );
    end else begin
        ini.WriteString('Path', 'cbPath', '');
    end;

    ini.WriteString('Filter', 'txtFilter', frmMain.GetFilter );

    ini.WriteBool(SCRIPT_OPTIONS, 'UseFilter', self.UseFilter);
    ini.WriteBool(SCRIPT_OPTIONS, 'UsePath', self.UsePath);

    if assigned(frmabout) then
        ini.WriteString('Version','Current', frmabout.lblVersion.caption);

    if renameoptions  then begin
        ini.WriteBool(ADD_TO, frmMain.cbAEndOfName.name, frmMain.cbAEndOfName.Checked );

        ini.WriteBool(ADD_TO, frmMain.cbAEndOfExtension.Name, frmMain.cbAEndOfExtension.Checked );
        ini.WriteBool(ADD_TO, frmMain.cbANInsert.Name, frmMain.cbANInsert.checked );
        ini.WriteBool(ADD_TO, frmMain.cbAEInsert.Name, frmMain.cbAEInsert.checked );

        ini.WriteString(ADD_TO, frmMain.txtAEndOfName.name, frmMain.txtAEndOfName.Text );
        ini.WriteString(ADD_TO, frmMain.txtAEndOfExtension.Name, frmMain.txtAEndOfExtension.Text );

        ini.WriteString(ADD_TO, frmMain.txtANInsert.Name, frmMain.txtANInsert.text );
        ini.WriteString(ADD_TO, frmMain.txtAEInsert.Name, frmMain.txtAEInsert.text );
        ini.WriteInteger(ADD_TO, frmMain.udANPosition.Name, frmMain.udANPosition.position );
        ini.WriteInteger(ADD_TO, frmMain.udAEPosition.Name, frmMain.udAEPosition.position );

        ini.WriteBool(ADD_TO2, frmmain.cbA2Find.Name, frmmain.cbA2Find.checked);
        ini.WriteString(ADD_TO2, frmmain.txtA2Find.name, frmmain.txtA2Find.text);
        ini.WriteString(ADD_TO2, frmmain.txtA2Insert.Name, frmmain.txtA2Insert.text);
        ini.WriteInteger(ADD_TO2, frmmain.cbA2Insert.Name, frmmain.cbA2Insert.ItemIndex);

        ini.WriteBool(REPLACE, frmMain.cbRPNameWith.Name, frmMain.cbRPNameWith.Checked );
        ini.WriteBool(REPLACE, frmMain.cbRPExtensionWith.Name, frmMain.cbRPExtensionWith.Checked );
        ini.WriteBool(REPLACE, frmMain.cbRPThis.Name, frmMain.cbRPThis.Checked );
        ini.WriteString(REPLACE, frmMain.txtRPNameWith.Name, frmMain.txtRPNameWith.Text );
        ini.WriteString(REPLACE, frmMain.txtRPExtensionWIth.name, frmMain.txtRPExtensionWIth.text );
        ini.WriteString(REPLACE, frmMain.txtRPThis.Name, frmMain.txtRPThis.text );
        ini.WriteString(REPLACE, frmMain.txtRPWith.Name, frmMain.txtRPWith.Text );


        ini.WriteBool(REPLACE2, FrmMain.cbRP2Characters.Name, FrmMain.cbRP2Characters.Checked );
        ini.WriteInteger(REPLACE2, frmMain.upRP2X.Name, frmMain.upRP2X.Position );
        ini.WriteInteger(REPLACE2, FrmMain.upRP2Y.Name, FrmMain.upRP2Y.Position);
        ini.WriteString(REPLACE2, FrmMain.txtRP2With.Name, FrmMain.txtRP2With.text );
        ini.WriteString(REPLACE2, FrmMain.txtRP2This.name, FrmMain.txtRP2This.text );
        ini.WriteBool(REPLACE2, FrmMain.cbRP2CharacterList.Name, FrmMain.cbRP2CharacterList.checked);
        ini.WriteString(REPLACE2, FrmMain.txtRP2sc.Name, FrmMain.txtRP2sc.Text);
        ini.WriteString(REPLACE2, FrmMain.txtRP2scWith.Name, FrmMain.txtRP2scWith.Text);
        ini.WriteBool(REPLACE2, frmmain.cbEverything.Name, frmmain.cbEverything.checked);
        ini.WriteInteger(REPLACE2, frmmain.txtR2EverythingX.Name, SafeInt(frmmain.txtR2EverythingX.text) );
        ini.WriteInteger(REPLACE2, frmmain.txtR2EverythingY.Name, SafeInt(frmmain.txtR2EverythingY.text) );
        ini.WriteBool(ADD_TO2, frmmain.cbCollision.Name, frmmain.cbCollision.checked );
        ini.WriteString(ADD_TO2, frmmain.txtCollision.Name, frmmain.txtCollision.Text);
        ini.WriteBool(REPLACE2, frmmain.cbRPNumbers.name, frmmain.cbRPNumbers.checked);
        ini.WriteInteger(REPLACE2, frmmain.cbxRPNumbers.name,frmmain.cbxRPNumbers.ItemIndex);




        ini.WriteBool(REMOVE, frmMain.cbRMFromNameStart.Name, frmMain.cbRMFromNameStart.checked );
        ini.WriteBool(REMOVE, frmMain.cbRMFromNameEnd.Name, frmMain.cbRMFromNameEnd.Checked);
        ini.WriteBool(REMOVE, frmMain.cbRMFromExtensionStart.Name, frmMain.cbRMFromExtensionStart.checked);
        ini.WriteBool(REMOVE, frmMain.cbRMFromExtensionEnd.Name, frmMain.cbRMFromExtensionEnd.Checked);
        ini.WriteBool(REMOVE, frmMain.cbRMFromName.Name, frmMain.cbRMFromName.Checked);
        ini.WriteBool(REMOVE, frmMain.cbRMFromExtension.Name, frmMain.cbRMFromExtension.Checked);
        ini.WriteBool(REMOVE, frmMain.cbRMWordList.Name, frmMain.cbRMWordList.Checked);

        ini.WriteInteger(REMOVE, frmMain.udNS.Name, frmMain.udNS.Position );
        ini.Writeinteger(REMOVE, frmMain.udNE.Name, frmMain.udNE.Position );
        ini.Writeinteger(REMOVE, frmMain.udES.Name, frmMain.udES.Position );
        ini.Writeinteger(REMOVE, frmMain.udEE.name, frmMain.udEE.Position );

        ini.WriteString(REMOVE, frmMain.txtRMFromName.Name, frmMain.txtRMFromName.Text );
        ini.WriteString(REMOVE, frmMain.txtRMFromExtension.Name, frmMain.txtRMFromExtension.Text );
        ini.WriteBool(Remove2, FrmMain.cbRM2NumFromName.Name, FrmMain.cbRM2NumFromName.Checked  );
        ini.WriteBool(Remove2, FrmMain.cbRM2NumFromExt.Name, FrmMain.cbRM2NumFromExt.Checked  );
        ini.WriteBool(REMOVE2, FrmMain.cbEverything.Name, frmmain.cbEverything.Checked  );
        ini.WriteInteger(REMOVE2, FrmMain.udRM2X.Name, frmmain.udRM2X.Position );
        ini.WriteInteger(REMOVE2, FrmMain.udRM2Y.Name, frmmain.udRM2Y.Position );

        ini.WriteBool(REMOVE2, FrmMain.cbRM2Characters.name, FrmMain.cbRM2Characters.checked);
        ini.WriteInteger(REMOVE2, FrmMain.udR2CharX.Name, frmmain.udR2CharX.Position );
        ini.WriteInteger(REMOVE2, FrmMain.udR2CharY.Name, frmmain.udR2CharY.Position );
        ini.WriteBool(REMOVE2, FrmMain.cbRM2AndMove.name, FrmMain.cbRM2AndMove.checked);
        ini.WriteInteger(REMOVE2, FrmMain.udR2AndMove.Name, frmmain.udR2AndMove.Position );
    end;


    ini.WriteBool(COUNTING, frmMain.rbUsePositionInList.Name, frmMain.rbUsePositionInList.Checked );
    ini.WriteBool(COUNTING, frmMain.rbStartCountingAt.Name, frmMain.rbStartCountingAt.Checked );
    ini.WriteString(COUNTING, frmMain.txtStartCountingAt.Name, frmMain.txtStartCountingAt.Text );
    ini.WriteString(COUNTING, frmMain.txtIncrementBy.Name, frmMain.txtIncrementBy.Text );
    ini.WriteBool(COUNTING, FrmMain.cbClearCount.Name, FrmMain.cbClearCount.Checked);


    ini.WriteBool(OVERRIDECASE, frmmain.cbOverrideCase.Name, frmmain.cbOverrideCase.checked);
    ini.WriteBool(OVERRIDECASE, frmmain.rbCUppercase.Name, frmmain.rbCUppercase.Checked);
    ini.WriteBool(OVERRIDECASE, frmmain.rbCLowercase.Name, frmmain.rbCLowercase.Checked);
    ini.WriteBool(OVERRIDECASE, frmmain.rbCCapitalizeWords.Name, frmmain.rbCCapitalizeWords.Checked);
    ini.WriteBool(OVERRIDECASE, frmmain.rbCCapitalizeSentence.Name, frmmain.rbCCapitalizeSentence.Checked);
    ini.WriteBool(OVERRIDECASE, frmmain.cbCPreserverExtension.name, frmmain.cbCPreserverExtension.checked);
    ini.WriteBool(OVERRIDECASE, frmmain.rbCTitleCase.name, frmmain.rbCTitleCase.checked);
    ini.WriteBool(OVERRIDECASE, frmmain.cbCRoman.name, frmmain.cbCRoman.checked);
    ini.WriteBool(OVERRIDECASE, frmmain.cbSplit.Name, FrmMain.cbSplit.Checked);
    ini.WriteString(OVERRIDECASE, frmmain.txtSplit.name, frmmain.txtSplit.Text);


    ini.WriteBool(WILDCARDS, frmmain.rbWildcards.name, frmmain.rbWildcards.checked);
    ini.WriteBool(WILDCARDS, frmmain.rbRegExp.name, frmmain.rbRegExp.checked);

    ini.WriteString(TEXT, frmmain.FrameTextfield1.txtTName.name, frmmain.FrameTextfield1.txtTName.Text);
    ini.WriteInteger(TEXT, FrmMain.FrameTextfield1.udTSkip.name, FrmMain.FrameTextfield1.udTSkip.Position);
    ini.WriteBool(TEXT, FrmMain.FrameTextfield1.cbTDelim.name, FrmMain.FrameTextfield1.cbTDelim.checked);
    ini.WriteBool(TEXT, FrmMain.FrameTextfield1.rbTMatch.name, FrmMain.FrameTextfield1.rbTMatch.checked);
    ini.WriteBool(TEXT, FrmMain.FrameTextfield1.rbTFind.name, FrmMain.FrameTextfield1.rbTFind.checked);
    ini.WriteString(TEXT, FrmMain.FrameTextfield1.txtTFind.name, FrmMain.FrameTextfield1.txtTFind.text);
    ini.WriteString(TEXT, FrmMain.FrameTextfield1.txtTDelim.name, FrmMain.FrameTextfield1.txtTDelim.text);
    ini.WriteInteger(TEXT, FrmMain.FrameTextfield1.cbxSearchWith.Name, FrmMain.FrameTextfield1.cbxSearchWith.ItemIndex);

    ini.UpdateFile;
    ini.Free;
end;

function TRMScript.DefaultExists: boolean;
begin
    result := FileExists(
        IncludeTrailingPathDelimiter(
        ExtractFilePath(Application.ExeName)
        ) + DEFAULT_SCRIPT
    );
end;

function TRMScript.GetUseFilter: boolean;
begin
    result := self.UseFilter;
end;

function TRMScript.GetUsePath: boolean;
begin
    result := self.UsePath;
end;

procedure TRMSCript.LoadSettings(renameoptions : boolean = true);
var ini : TMyIniFile;
    s : string;
begin
    ini := TMyIniFile.Create(Filename);
    self.UseFilter := false;
    self.UsePath := false;
    if ini.ValueExists(SCRIPT_OPTIONS, 'UseFilter') then begin
        self.UseFilter := ini.ReadBool(SCRIPT_OPTIONS, 'UseFilter', true);
        self.UsePath := ini.ReadBool(SCRIPT_OPTIONS, 'UsePath', true);
        if (UsePath) then begin
            s := ini.ReadString('Path', 'cbPath',  ExtractFilePath(Application.EXEName) );
            if (s <> '') then begin
                frmMain.SetPathOnly(s, True);
            end;
        end;
        if (UseFilter) then begin
//            s := ini.ReadString('Filter', frmMain.txtFilter.Name, '*.*');
            s := ini.ReadString('Filter', 'txtFilter', '*.*');
            if (s <> '') then begin
                frmMain.SetFilter(s);
            end;
        end;
    end else begin

        if ini.ReadBool('ScriptOptions','UsePath',false) then begin
            s := ini.ReadString('Path', 'cbPath',  ExtractFilePath(Application.EXEName) );
            if (s <> '') then begin
                self.UsePath := true;
                frmMain.SetPathOnly(s);
            end;
        end;
        s := ini.ReadString('Filter', 'txtFilter', '*.*');
        if (s <> '') then begin
            frmMain.SetFilter(s);
            self.UseFilter := true;
        end;
    end;

    fversion := ini.ReadString('Version','Current','');

    if renameoptions  then begin
        //frmMain.cbAStartOfName.Checked :=    ini.ReadBool(ADD_TO, frmMain.cbAStartOfName.name , false );
        frmMain.cbAEndOfName.Checked    := ini.ReadBool(ADD_TO, frmMain.cbAEndOfName.name, false );
        //frmMain.cbAStartOfExtension.checked := ini.ReadBool(ADD_TO, frmMain.cbAStartOfExtension.Name, false );
        frmMain.cbAEndOfExtension.Checked := ini.ReadBool(ADD_TO, frmMain.cbAEndOfExtension.Name,  false);
        frmMain.cbANInsert.Checked := ini.ReadBool(ADD_TO, frmMain.cbANInsert.Name, false);
        frmMain.cbAEInsert.Checked := ini.ReadBool(ADD_TO, frmMain.cbAEInsert.Name, false);
        //frmMain.txtAStartOfName.Text := ini.ReadString(ADD_TO, frmMain.txtAStartOfName.name ,   '');
        frmMain.txtAEndOfName.Text := ini.ReadString(ADD_TO, frmMain.txtAEndOfName.name,  '');
        //frmMain.txtAStartOfExtension.Text := ini.ReadString(ADD_TO, frmMain.txtAStartOfExtension.name,  '');
        frmMain.txtAEndOfExtension.Text :=  ini.ReadString(ADD_TO, frmMain.txtAEndOfExtension.Name,  '');
        frmMain.txtANInsert.text := ini.ReadString(ADD_TO, frmMain.txtANInsert.Name, '');
        frmMain.txtAEInsert.text := ini.ReadString(ADD_TO, frmMain.txtAEInsert.Name, '');

        frmMain.udANPosition.position := ini.ReadInteger(ADD_TO, frmMain.udANPosition.name, 1);
        FrmMain.txtANPosition.Text := IntToSTr(frmMain.udANPosition.position);
        frmMain.udAEPosition.position := ini.ReadInteger(ADD_TO, frmMain.udAEPosition.name, 1);
        FrmMain.txtAEPosition.Text := IntToSTr(frmMain.udAEPosition.position);

        frmmain.cbA2Find.checked := ini.ReadBool(ADD_TO2, frmmain.cbA2Find.Name, false);
        frmmain.txtA2Find.text := ini.ReadString(ADD_TO2, frmmain.txtA2Find.name, '');
        frmmain.txtA2Insert.text := ini.ReadString(ADD_TO2, frmmain.txtA2Insert.Name, '');
        frmmain.cbA2Insert.ItemIndex := ini.ReadInteger(ADD_TO2, frmmain.cbA2Insert.Name, 0);
        frmmain.txtCollision.Text := ini.ReadString(ADD_TO2, frmmain.txtCollision.Name, '');
        frmmain.cbCollision.checked := ini.ReadBool(ADD_TO2, frmmain.cbCollision.Name, false);

        frmMain.cbRPNameWith.Checked := ini.ReadBool(REPLACE, frmMain.cbRPNameWith.Name,  false);
        frmMain.cbRPExtensionWith.Checked := ini.ReadBool(REPLACE, frmMain.cbRPExtensionWith.Name, false);
        frmMain.cbRPThis.Checked  := ini.ReadBool(REPLACE, frmMain.cbRPThis.Name, false);
        frmMain.txtRPNameWith.Text := ini.ReadString(REPLACE, frmMain.txtRPNameWith.Name, '');
        frmMain.txtRPExtensionWIth.text := ini.ReadString(REPLACE, frmMain.txtRPExtensionWIth.name, '');
        frmMain.txtRPThis.text := ini.ReadString(REPLACE, frmMain.txtRPThis.Name, '');
        frmMain.txtRPWith.Text := ini.ReadString(REPLACE, frmMain.txtRPWith.Name, '');


        FrmMain.cbRP2Characters.Checked := ini.ReadBool(REPLACE2, FrmMain.cbRP2Characters.Name,  false);
        frmMain.upRP2X.Position := ini.ReadInteger(REPLACE2, frmMain.upRP2X.Name,  1);
        FrmMain.upRP2Y.Position := ini.ReadInteger(REPLACE2, FrmMain.upRP2Y.Name, 1);
        FrmMain.txtRP2With.text := ini.ReadString(REPLACE2, FrmMain.txtRP2With.Name,  '');
        FrmMain.txtRP2This.text := ini.ReadString(REPLACE2, FrmMain.txtRP2This.name, '*' );

        FrmMain.cbRP2CharacterList.checked := ini.ReadBool(REPLACE2, FrmMain.cbRP2CharacterList.Name, false);
        FrmMain.txtRP2sc.Text := ini.ReadString(REPLACE2, FrmMain.txtRP2sc.Name, '');
        FrmMain.txtRP2scWith.Text := ini.ReadString(REPLACE2, FrmMain.txtRP2scWith.Name, '');
        frmmain.cbRPNumbers.checked := ini.ReadBool(REPLACE2, frmmain.cbRPNumbers.name, false);
        frmmain.cbxRPNumbers.ItemIndex := ini.ReadInteger(REPLACE2, frmmain.cbxRPNumbers.name, 0);


        frmmain.cbEverything.checked:= ini.ReadBool(REPLACE2, frmmain.cbEverything.Name, false);
        frmmain.udRM2X.position := ini.ReadInteger(REPLACE2, frmmain.txtR2EverythingX.Name, 1 );
        frmmain.txtR2EverythingX.Text := IntToStr(frmmain.udRM2X.position);
        frmmain.udRM2Y.position := ini.ReadInteger(REPLACE2, frmmain.txtR2EverythingY.Name, 10 );
        frmmain.txtR2EverythingY.Text := IntToStr(frmmain.udRM2Y.position);



        frmMain.cbRMFromNameStart.checked := ini.ReadBool(REMOVE, frmMain.cbRMFromNameStart.Name, false);
        frmMain.cbRMFromNameEnd.Checked := ini.ReadBool(REMOVE, frmMain.cbRMFromNameEnd.Name, false);
        frmMain.cbRMFromExtensionStart.checked := ini.ReadBool(REMOVE, frmMain.cbRMFromExtensionStart.Name, false);
        frmMain.cbRMFromExtensionEnd.Checked := ini.ReadBool(REMOVE, frmMain.cbRMFromExtensionEnd.Name, false);
        frmMain.cbRMFromName.Checked := ini.ReadBool(REMOVE, frmMain.cbRMFromName.Name, false);
        frmMain.cbRMFromExtension.Checked := ini.ReadBool(REMOVE, frmMain.cbRMFromExtension.Name, false);
        frmMain.cbRMWordList.Checked := ini.ReadBool(REMOVE, frmMain.cbRMWordList.Name, false);


        frmMain.udNS.Position := ini.ReadInteger(REMOVE, frmMain.udNS.name, 0);
        frmmain.txtRMFromNameStart.text := IntToStr(frmMain.udNS.position);
        frmMain.udNE.Position := ini.ReadInteger(REMOVE, frmMain.udNE.name, 0);
        frmmain.txtRMFromNameEnd.text := IntToStr(frmMain.udNE.position);
        frmMain.udES.Position := ini.ReadInteger(REMOVE, frmMain.udES.name, 0);
        frmmain.txtRMFromExtensionStart.text := IntToStr(frmMain.udES.position);
        frmMain.udEE.Position := ini.ReadInteger(REMOVE, frmMain.udEE.name, 0);
        frmmain.txtRMFromExtensionEnd.text := IntToStr(frmMain.udEE.position);

        frmMain.txtRMFromName.Text := ini.ReadString(REMOVE, frmMain.txtRMFromName.Name, '');
        frmMain.txtRMFromExtension.Text := ini.ReadString(REMOVE, frmMain.txtRMFromExtension.Name, '');

        frmMain.cbRM2NumFromName.Checked := ini.ReadBool(REMOVE2, frmMain.cbRM2NumFromName.Name, false);
        frmMain.cbRM2NumFromExt.Checked :=ini.ReadBool(REMOVE2, frmMain.cbRM2NumFromExt.Name, false);


        frmmain.cbEverything.Checked :=     ini.ReadBool(REMOVE2, FrmMain.cbEverything.Name, false);
        frmmain.udRM2X.Position := ini.ReadInteger(REMOVE2, FrmMain.udRM2X.Name,  1);
        frmmain.udRM2Y.Position := ini.ReadInteger(REMOVE2, FrmMain.udRM2Y.Name,  10);

        FrmMain.cbRM2Characters.checked := ini.ReadBool(REMOVE2, FrmMain.cbRM2Characters.name, false );
        frmmain.udR2CharX.Position := ini.ReadInteger(REMOVE2, FrmMain.udR2CharX.Name,  1);
        frmmain.udR2CharY.Position := ini.ReadInteger(REMOVE2, FrmMain.udR2CharY.Name,  10);
        FrmMain.cbRM2AndMove.checked := ini.ReadBool(REMOVE2, FrmMain.cbRM2AndMove.name, false);
        frmmain.udR2AndMove.Position := ini.ReadInteger(REMOVE2, FrmMain.udR2AndMove.Name,  1);
    end;

    frmMain.rbUsePositionInList.Checked := ini.ReadBool(COUNTING, frmMain.rbUsePositionInList.Name, false);
    frmMain.rbStartCountingAt.Checked := ini.ReadBool(COUNTING, frmMain.rbStartCountingAt.Name, true);
    frmMain.txtStartCountingAt.Text := ini.ReadString(COUNTING, frmMain.txtStartCountingAt.Name, '');
    frmMain.txtIncrementBy.Text := ini.ReadString(COUNTING, frmMain.txtIncrementBy.Name, '');
    frmMain.udIncrementBy.Position := SafeInt(frmMain.txtIncrementBy.text, 1);
    FrmMain.cbClearCount.Checked := ini.ReadBool(COUNTING, FrmMain.cbClearCount.Name, false);



    frmmain.cbOverrideCase.checked := ini.ReadBool(OVERRIDECASE, frmmain.cbOverrideCase.Name, false);
    frmmain.rbCUppercase.Checked := ini.ReadBool(OVERRIDECASE, frmmain.rbCUppercase.Name, true);
    frmmain.rbCLowercase.Checked := ini.ReadBool(OVERRIDECASE, frmmain.rbCLowercase.Name, false);
    frmmain.rbCCapitalizeWords.Checked := ini.ReadBool(OVERRIDECASE, frmmain.rbCCapitalizeWords.Name, false);
    frmmain.rbCCapitalizeSentence.Checked := ini.ReadBool(OVERRIDECASE, frmmain.rbCCapitalizeSentence.Name, false);
    frmmain.cbCPreserverExtension.checked := ini.ReadBool(OVERRIDECASE, frmmain.cbCPreserverExtension.name, false);
    frmmain.rbCTitleCase.Checked := ini.ReadBool(OVERRIDECASE, frmmain.rbCTitleCase.name, false);
    frmmain.cbCRoman.checked := ini.ReadBool(OVERRIDECASE, frmmain.cbCRoman.name, false);
    FrmMain.cbSplit.Checked := ini.ReadBool(OVERRIDECASE, frmmain.cbSplit.Name, false);
    frmmain.txtSplit.Text := ini.ReadString(OVERRIDECASE, frmmain.txtSplit.name, '');


    frmmain.rbWildcards.checked := ini.ReadBool(WILDCARDS, frmmain.rbWildcards.Name, true);
    frmmain.rbRegExp.checked := ini.ReadBool(WILDCARDS, frmmain.rbRegExp.Name, false);


    frmmain.FrameTextfield1.txtTName.Text := ini.ReadString(TEXT, frmmain.FrameTextfield1.txtTName.name,  '');
    FrmMain.FrameTextfield1.udTSkip.Position := ini.ReadInteger(TEXT,FrmMain.FrameTextfield1.udTSkip.name,  0);
    FrmMain.FrameTextfield1.cbTDelim.checked := ini.ReadBool(TEXT,FrmMain.FrameTextfield1.cbTDelim.name, true);
    FrmMain.FrameTextfield1.rbTMatch.checked := ini.ReadBool(TEXT, FrmMain.FrameTextfield1.rbTMatch.name,true);
    FrmMain.FrameTextfield1.rbTFind.checked := ini.ReadBool(TEXT, FrmMain.FrameTextfield1.rbTFind.name,  false);
    FrmMain.FrameTextfield1.txtTFind.text := ini.ReadString(TEXT, FrmMain.FrameTextfield1.txtTFind.name, '?cfn?');
    FrmMain.FrameTextfield1.txtTDelim.text := ini.ReadString(TEXT, FrmMain.FrameTextfield1.txtTDelim.name, ',');
    FrmMain.FrameTextfield1.cbxSearchWith.ItemIndex := ini.ReadInteger(TEXT, FrmMain.FrameTextfield1.cbxSearchWith.Name, 0);
    FrmMain.FrameTextfield1.cbxSearchWith.OnChange(self);


    // don't need to update, no writes
    ini.Free;
end;

begin
    RMScript := TRMScript.Create;
end.
