unit UnitSpecialPaths;
{
    Purpose:
        Return the paths to special folders used throught the program

    Updates:
        Win9x compatibility added for GetCommonStartupPath
}

interface

type TSpecialPaths = class(TObject)
    private
        function GetPath(CLSIDL : word) : string;
    public
        function GetStartupPath : string;
        function GetCommonStartupPath : string;
        function GetAltStartupPath : string;
        function GetStartRightStartup : string;
        function GetMyDocuments : string;
        function GetMyPictures : string;
        function GetSystemRoot : string;
        function GetDesktop : string;

end;
var SpecialPaths : TSpecialPaths;

{////////////////////}
{//}implementation{//}
{////////////////////}

uses ShellAPI, ShlObj, Windows, Forms, SysUtils, UnitUtil;

const CSIDL_MYPICTURES = $0027;

function TSpecialPaths.GetPath(CLSIDL : word) : string;
var p : PItemIDList;
    Path: array[0..MAX_PATH] of char;
begin
    SHGetSpecialFolderLocation(Application.Handle , CLSIDL, p);
    SHGetPathFromIDList(p, path);
    result := path;
end;



function TSpecialPaths.GetStartupPath : string;
begin
    result := self.GetPath(CSIDL_STARTUP);
end;

function TSpecialPaths.GetSystemRoot: string;

begin
    result := ExpandEnvironment('%SystemRoot%');
end;

function TSpecialPaths.GetCommonStartupPath : string;
begin
    result := self.GetPath(CSIDL_COMMON_STARTUP);

    // win9x compatibility
    if (result = '') then begin
        result := self.GetPath(CSIDL_STARTUP);
    end;
end;

function TSpecialPaths.GetDesktop: string;
begin
    result := self.GetPath(CSIDL_DESKTOP);
end;

function TSpecialPaths.GetAltStartupPath : string;
begin
    result := self.GetPath(CSIDL_ALTSTARTUP);
end;
function TSpecialPaths.GetStartRightStartup : string;
begin
    result := IncludeTrailingPathDelimiter(
         ExtractFilePath(Application.ExeName)) + 'Startup\';
end;

function TSpecialPaths.GetMyDocuments: string;
begin
    result := self.GetPath(CSIDL_PERSONAL);
end;

function TSpecialPaths.GetMyPictures: string;
begin
    result := self.GetPath(CSIDL_MYPICTURES);
end;

initialization
    SpecialPaths := TSpecialPaths.Create;
end.
