unit UnitStates;
{
    Purpose:
        Used to create a state machine where each state has
        a binary outcome.
}
interface

const EOF_STATE = -1;
type TState = class(TObject)
    private
        MatchChar : char;
        MatchState : integer;
        OtherState : integer;
        isStarting : boolean;
        isEnding : boolean;
    public

        procedure SetIsStarting(b : boolean);
        procedure SetIsEnding(b : boolean);
        procedure SetState(match : char; MatchStateIndex, OtherStateIndex : integer);
        procedure SetMatchState(index : integer);

        function GetMatchChar : char;
        function GetMatchStateIndex : integer;
        function GetOtherStateIndex : integer;
        function GetIsStarting : boolean;
        function GetIsEnding : boolean;

        function GetNextState(c : char) : integer;
        function IsMatch(c : char) : boolean;
end;

implementation

procedure TState.SetState(match : char; MatchStateIndex, OtherStateIndex : integer);
begin
    MatchChar := match;
    MatchState := MatchStateIndex;
    OtherState := OtherStateIndex;
end;

procedure TState.SetMatchState(index : integer);
begin
    MatchState := index;
end;

procedure TState.SetIsStarting(b : boolean);
begin
    IsStarting := b;
end;

procedure TState.SetIsEnding(b : boolean);
begin
    IsEnding := b;
end;

function TState.GetIsStarting : boolean;
begin
    result := IsStarting;
end;

function TState.GetIsEnding : boolean;
begin
    result := IsEnding;
end;

function TState.GetMatchChar : char;
begin
    result := MatchChar;
end;
function TState.GetMatchStateIndex : integer;
begin
    result := MatchState;
end;
function TState.GetOtherStateIndex : integer;
begin
    result := OtherState;
end;


function TState.GetNextState(c : char) : integer;
begin
    if (IsMatch(c)) then begin
        result := MatchState;
    end else begin
        result := OtherState;
    end;
end;


function TState.IsMatch(c : char) : boolean;
begin
    result := (c = MatchChar) or (MatchChar = '*') or (MatchChar = '?');
end;

end.
