unit UnitTextFileFormat;


interface

uses Classes;

type TTextFileFormat = class(TStringList)
    private
        fSkippedLines : integer;
        fFieldCount : integer;
        fDelim : Char;
        lastFile : string;
        procedure TokenCVS(line : string; var fields : TStringList);
        function Get(Index: Integer): string; override;
        procedure Put(Index: Integer; const Value: string);
        procedure SetSkippedLines(const Value: integer);
        procedure SetDelim(const Value: Char);

        function isUnicode(const FileName:string):boolean;
    public
        constructor Create;
        procedure LoadFromFile(const FileName: string); override;
        function GetField(line, field : integer) : string;

        function FindLine(search : string) : integer;
        function FindLine_ColumnInStr(columnindex : integer; str : string ) : integer;
        property SkipedLines : integer read fSkippedLInes write SetSkippedLines;
        property FieldCount : integer read fFieldCount;
        property Delim : Char read fDelim write SetDelim;
        property Strings[Index: Integer]: string read Get write Put; default;
end;

var TextFileFormat  : TTextFileFormat;

implementation


uses UnitToken, UnitUtil, System.SysUtils;

{ TTextFileFormat }

constructor TTextFileFormat.Create;
begin
    fDelim := ',';
    inherited create;
end;

function TTextFileFormat.FindLine_ColumnInStr(columnindex: integer;
  str: string): integer;
var i : integer;
    s : string;
begin
    result := -1;
    if (columnindex < 0) or (columnindex >= fFieldCount )  then EXIT;

    for i := 0 to self.Count - 1 do begin
        s := self.GetField(i, columnindex);
        if ContainsTerm(s, str) then begin
            result := i;
            EXIT;
        end;
    end;

end;

function TTextFileFormat.FindLine(search: string): integer;
begin
    for result := 0 to self.GetCount - 1 do
        if ContainsTerm(search, self.Get(result)) then EXIT;
    result := -1;
end;

function TTextFileFormat.Get(Index: Integer): string;
begin
    result := '';
    if (index >=0) and (index < self.count) then
        result := inherited get(Index);
    
end;

function TTextFileFormat.GetField(line, field: integer): string;
var
    sl : TStringList;
begin
    sl := TStringList.Create;
    self.TokenCVS(strings[line], sl);
    if (sl.count > 0) and (field < sl.count) then begin
        result := sl.Strings[field];
    end else begin
        result := '';
    end;
    sl.free;
end;

function TTextFileFormat.isUnicode(const FileName:string):boolean;
var
  txt: file;
  bytes: array[0..2] of byte;
  amt: integer;
begin
    FileMode := fmOpenRead;
    AssignFile(txt, FileName);
    Reset(txt, 1);
    try
        BlockRead(txt, bytes, 3, amt);
        result := (amt=3) and
        (bytes[0] = Encoding.Unicode.GetPreamble[0])
    finally
        CloseFile(txt);
    end;
end;

procedure TTextFileFormat.LoadFromFile(const FileName: string);
var i : integer;

    sl : TStringList;
begin

    if lastfile <> filename then begin
        if (isUnicode(FileName)) then begin
            inherited LoadFromFile(filename,Encoding.Unicode);
        end else begin
            inherited LoadFromFile(filename);
        end;

        if fSkippedLines <> 0 then begin
            for i := 0 to SkipedLines - 1 do begin
                if self.Count > 0 then
                    self.Delete(0);
            end;
        end;


        self.fFieldCount := self.Count;
        if self.FieldCount > 0 then begin
            sl := TStringList.Create;
            self.TokenCVS(self.Strings[0], sl);
            self.fFieldCount := sl.count;
            sl.free;
        end;
    end;

    lastFile := filename;
end;

procedure TTextFileFormat.Put(Index: Integer; const Value: string);
begin
    inherited Put(Index, Value);
end;

procedure TTextFileFormat.SetDelim(const Value: Char);
begin
    if fDelim <> Value then begin
        lastFile := '';
    end;
    fDelim := Value;
end;

procedure TTextFileFormat.SetSkippedLines(const Value: integer);
begin
    if fSkippedLines <> Value then begin
        lastFile := '';
    end;
    fSkippedLInes := Value;
end;

procedure TTextFileFormat.TokenCVS(line: string; var fields: TStringList);
begin

    fields.Delimiter := fDelim;
    fields.QuoteChar := '"';
    fields.StrictDelimiter := true;
    fields.DelimitedText := line;
end;

initialization
begin
    TextFileFormat := TTextFileFormat.Create;
end;
end.
