unit UnitToken;
{
    Purpose:
        String utils related to tokenization.
        Just look at all the bad naming conventions - shamefull.

    Updates:
        Included apostropes in wordChars for words like
        "Can't" and "Don't".

}

interface

function TokenString(var s: string; delim: string; TrimLeadingSpace : boolean = true) : string;
function clipOneChar(s : string) : string;
function clipManyChars(s : string; clip : string) : string; overload;
function clipManyChars(s : string; count : cardinal) : string; overload;
function CharCount(s : string; c : char) : cardinal;
function CutWord(var s: string; WordChars: string) : string;
function CutWordExclude(var s: string; delimiters : string) : string;
function CapitalizeWords(s : string; moredelimiters : string = '') : string;

implementation

uses strutils, sysutils, UnitUtil, Windows;


function CharCount(s : string; c : char) : cardinal;
var i : integer;
begin
    result := 0;
    for i := 1 to length(s) do begin
        if  s[i] = c then inc(Result);
    end;
end;


function BestFit(const AInput: AnsiString): AnsiString;
const
    CodePage = 20127; //20127 = us-ascii
var
    ws: WideString;
begin
    ws := WideString(AInput);
    SetLength(Result, WideCharToMultiByte(CodePage, 0, PWideChar(ws),Length(ws), nil, 0, nil, nil));
    WideCharToMultiByte(CodePage, 0, PWideChar(ws), Length(ws), PAnsiChar(Result), Length(Result), nil, nil);
end;

function CapitalizeWords(s : string; moredelimiters : string = '') : string;
var word : string;
    letter : string;
    delims : string;
    const wordChars : string = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRTSUVWXYZ0123456789''';
    const wordCharsLess : string = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRTSUVWXYZ';

// all punctuation on a keyboard, minus single quote
// NOTE: leading space intentded
    const puncChars : string = ' `~!@#$%^&*()_-+={[}]|\:;"<,>.?/';

begin
    result := '';
    s := lowercase(s);

    delims := puncChars + moredelimiters;

    while (s <> '') do begin

        word := CutWordExclude(s, delims);
        if (word <> '') then begin
            letter := UpperCase( BestFit(word[1]));
            if letter <> '?' then
                word[1] := letter[1];
        end;

        result := result + word;
    end;
end;
function CutWord(var s: string; WordChars: string) : string;
var i : longint;
begin
    i := 1;

    // find a "word"
    while (i < length(s))
        and (ContainsTerm(s[i],WordChars)) do begin
        inc(i);
    end;

    // find all chars leading up to the next word
    // stop before getting to the next word
    while ((i+1) < length(s))
        and (not ContainsTerm(s[i+1],WordChars)) do begin
        inc(i);
    end;

    // cut off the word and trailing delimiters
    result := LeftStr(s, i);
    s := rightstr(s, length(s) - i);
end;
function CutWordExclude(var s: string; delimiters : string) : string;
var i : longint;
begin
    i := 1;

    // find a "word"
    while (i < length(s))
        and (not ContainsTerm(s[i], delimiters)) do begin
        inc(i);
    end;

    // find all chars leading up to the next word
    // stop before getting to the next word
    while ((i+1) < length(s))
        and (ContainsTerm(s[i+1], delimiters)) do begin
        inc(i);
    end;

    // cut off the word and trailing delimiters
    result := LeftStr(s, i);
    s := rightstr(s, length(s) - i);
end;



function TokenString(var s: string; delim: string; TrimLeadingSpace : boolean = true) : string;
var delimPos: integer;
begin
    if TrimLeadingSpace then s := trimleft(s);

    delimPos := pos(delim, s);
    If (delimPos > 0) Then begin
        //'-- cut off first token and ditch the delimiter
        result := Leftstr(s, (delimPos - 1));
        s := rightstr(s,  length(s) - (length(delim) + length(result)));
    end Else begin
        //'-- return last token
        result := s;
        s := '';
    End;

end;


function clipOneChar(s : string) : string;
begin
    result := rightstr(s, length(s) - 1);
end;

function clipManyChars(s : string; clip : string) : string;
begin
    result := rightstr(s, length(s) - length(clip));
end;

function clipManyChars(s : string; count : cardinal) : string;
begin
    result := rightstr(s, length(s) - integer(count));
end;


end.
