unit UnitUndoRename;

interface

type TUndoRename = class(TObject)
    private
        NextIndex : Integer;
        function GetNextFilename : string;
    public
        constructor Create;
        destructor Destroy; override;
        procedure SaveStateToFile;
        procedure SaveSingleRenameToFile(oldname, newname : string);
        procedure LoadLastState;
        procedure CancelLastLoad;
        function HasState : boolean;
end;
var UndoRename : TUndoRename;
implementation

uses
    UnitUtil, UnitFrmUndo, System.IOUtils, UnitFrmGenerateBatch, Vcl.FileCtrl,
    System.SysUtils, System.Classes, UnitToken, UnitColumnManager, UnitFrmDummyShowPreview,
  UnitFrmMain;


constructor TUndoRename.Create;
var
    s : string;
begin
    ForceDirectories( UnitUtil.getUndoPath );
    if TFile.Exists(UnitUtil.GetUndoPath+'\noclean.txt') then begin
        // Program did not cleanly shut down
        // TODO: handle this case and alert the user
    end;

    NextIndex := 0;
    repeat
        s := Self.GetNextFilename;
        Inc(NextIndex);

        if TFile.Exists(s) then begin
            TFile.Delete(s);
        end else begin
            BREAK;
        end;
    until false;
    NextIndex := 0;

    TFile.Create(UnitUtil.GetUndoPath + '\notclean.txt').Free;
end;
destructor TUndoRename.Destroy;
begin
    if TFile.Exists(UnitUtil.GetUndoPath + '\notclean.txt') then
        TFile.Delete(UnitUtil.GetUndoPath + '\notclean.txt');
end;


function TUndoRename.HasState : boolean;
begin
    result := NextIndex <> 0;
end;
function TUndoRename.GetNextFilename : string;
begin
    result := UnitUtil.GetUndoPath + '\'  + Format('%.4d', [NextIndex]) + '.txt';
end;

procedure TUndoRename.SaveStateToFile;
    function DoInclude(i : Integer) : Boolean;
    begin
        Result := (FrmDummyShowPreview.GetChecked(i)) and
            (UnitColumnManager.GetNewName(i) <> '') and
            (UnitColumnManager.GetNewName(i) <> UnitColumnManager.GetName(i));
    end;
var sl : TStringList;
    s : string;
    i : integer;
    oldn, newn, base : string;

begin
    sl := TStringList.Create;
    s := self.GetNextFilename;


    base := IncludeTrailingPathDelimiter(FrmMain.GetPath);
    for i := 0 to frmmain.lvFileList.Items.Count -1 do begin
        oldn := UnitColumnManager.GetName(i);
        newn := UnitColumnManager.GetNewName(i);
        if DoInclude(i) then begin
            sl.Add('"' + base+oldn + '" "' + base+newn + '"');
        end;
    end;
    if sl.Count > 0 then begin
        inc(NextIndex);
        sl.SaveToFile(
            s,
            TEncoding.Unicode
        );
    end;
    sl.Free;
end;

procedure TUndoRename.LoadLastState;
var
    s : string;
    oldn, newn : string;
    sl : TStringList;
    j : integer;
begin
    if (NextIndex) > 0 then begin
        Dec(NextIndex);
        s := GetNextFilename;

        sl := TStringList.create;
        sl.LoadFromFile(s);

        frmUndo.ClearAll;
        for j := 0 to sl.Count-1 do begin
            s := sl.Strings[j];

            TokenString(s,'"',false);
            oldn := TokenString(s,'"',false);
            TokenString(s,'"',false);
            newn := TokenString(s,'"',false);

            frmUndo.AddUndo(oldn, newn);
        end;

        sl.Free;
    end;
end;

procedure TUndoRename.CancelLastLoad;
begin
    inc(NextIndex);
end;

procedure TUndoRename.SaveSingleRenameToFile(oldname, newname : string);
var sl : TStringList;
    s : string;

begin
    sl := TStringList.Create;
    s := self.GetNextFilename;
    inc(NextIndex);

    sl.Add('"' + oldname + '" "' + newname+ '"');
    if sl.Count > 0 then begin
        sl.SaveToFile(
            s,
            TEncoding.Unicode
        );
    end;
    sl.Free;
end;

initialization
begin
    UndoRename := TUndoRename.Create;
end;


finalization
begin
    UndoRename.Free;
end;
end.
