unit UnitUtil;

interface

uses ComCtrls, Windows, Graphics, SysUtils, Controls, StdCtrls, Menus, extctrls {TWinControl}, System.DateUtils;
function GetINIPath: string;
function GetINIPathOnly : string;
function StripNumberFormat(s : string ) : string;
function GetSizeOf(fullname : string) : double;
function SafeInt(s : string; default : integer = 0) : integer;

function GetFileList() : TListView;
 function dimcolor(c : TColor; alpha : double = 0.95) : TColor;

procedure CloneWinControl(inwc : TWinControl; outwc : TWinControl;  TopOffset : Integer = 0);

{function SetWindowTheme (hwnd: HWND; pszSubAppName: LPCWSTR;
  pszSubIdList: LPCWSTR): HRESULT; stdcall; external 'uxtheme.dll';}

procedure ShowPopupRight(b : TControl; pm : TPopupMenu);
procedure ShowPopupUnder(b : TControl; pm : TPopupMenu);


function StringIn(source : string; search : array of string) : boolean;
function ContainsTerm(term : string; search: string) : boolean;
function ContainsDigit(source : string) : boolean;
function ContainsOnly(source: string; c : Char) : boolean;

procedure ShowMessageI(i : integer);
function SetBlueColor(c : TColor; saturation : double) : TColor;

function MouseInControl(c : TControl) : boolean;
function ExpandEnvironment(s : string): string;

function IsIllegalFilename(filename: string) : boolean;
function GetUndoPath : string;

function ShadeColor(main, shade : TColor; percent : double) : TColor;
procedure StartClock;
function CurrentClock : cardinal;

function ReplaceAll(source : string; oldPattern,newPattern : string) : string;
function RemoveAll(source : string; Pattern : string) : string;

function StringIsInt(s : string) : boolean;
function StringIsDate(s : string) : boolean;


function SupportsLongerFilenames : boolean;
function LongerDirectoryExists(dir : string) : boolean;
{////////////////////}
{//}implementation{//}
{////////////////////}



uses   Forms, UnitFrmMain, math, types, dialogs, System.Character,
System.IOUtils, VarUtils, Variants, XSBuiltIns, UnitWildcards, StrUtils, Registry;


function LongerDirectoryExists(dir : string) : boolean;
begin
    result := SysUtils.DirectoryExists(dir);
    if not result then
        result := SysUtils.DirectoryExists('\\?\\'+dir);
end;


function SupportsLongerFilenames : boolean;
var maj, min, buildnum : DWORD; build : string;
    r : TRegistry;
begin
    result := false;
    r := TRegistry.Create(KEY_READ);
    r.RootKey := HKEY_LOCAL_MACHINE;

    if r.OpenKey('SOFTWARE\Microsoft\Windows NT\CurrentVersion', false) then begin
        maj := r.ReadInteger('CurrentMajorVersionNumber');
        min := r.ReadInteger('CurrentMinorVersionNumber');
        build := r.ReadString('CurrentBuildNumber');
    end;
    r.CloseKey;
    r.Free;

    result := not (maj < 10);
    if not result then EXIT;

    buildnum := 0;
    try
        buildnum := StrToInt(build);
    except
    end;
    result := not (buildnum < 1607)
end;


{$HINTS OFF} // these values are never used, but needed to test for String types
function StringIsInt(s : string) : boolean;
var long : LONGLONG;
begin
   try
        long := StrToInt(s);
        result := true;
    except
        result := false;
    end;
end;
function StringIsDate(s : string) : boolean;
var dt : TDateTime;
    fs : TFormatSettings;
    v : variant;
    function IsOfType(datestr : string; fs : TFormatSettings) : boolean;
    var dt : TDateTime;
    begin
        try
            dt := StrToDate(datestr, fs);
            result := true;
            EXIT;
        except
            result := false;
        end;

    end;
var testDate : string;
    wc : TRegExpWildcards;
    parts : TStringDynArray;
    i, j : integer;
begin
    fs := TFormatSettings.Create;

    wc := TRegExpWildcards.create;
    wc.SetSearchString(s);
    wc.SetSearchPattern('[a-zA-Z]+, [a-zA-Z]+ [0-9]+, [0-9]+');
    if wc.Matches then begin
        parts := SplitString(s, ', ');
        for i := 0 to length(parts)-1 do begin
            j := IndexText(parts[i], FormatSettings.LongDayNames);
            if (j = -1) and (parts[i] <> '') then begin
                if testDate = '' then begin
                    testDate := parts[i];
                end else begin
                    testDate := testDate +', ' + parts[i];
                end;
            end;
        end;
        s := testDate;
    end;
    wc.Free;

    try
        dt := VarToDateTime(s);
        result := true;
        EXIT;
    except
        result := false;
    end;

    try

        dt := StrToDate(s);
        result := true;
        EXIT;
    except
        result := false;
    end;


    try
        dt := XMLTimeToDateTime(s);
        result := true;
        EXIT;
    except
        result := false;
    end;
    try
        dt := XMLTimeToDateTime(s, true);
        result := true;
        EXIT;
    except
        result := false;
    end;

end;
{$HINTS ON}

function RemoveAll(source : string; Pattern : string) : string;
begin
    ReplaceAll(source,pattern,'');
end;
function ReplaceAll(source : string; oldPattern,newPattern : string) : string;
begin
    result := StringReplace(source,oldPattern,newPattern,[rfReplaceAll,rfIgnoreCase]);
end;


var lastTick : cardinal;
procedure StartClock;
begin
    lastTick := Windows.GetTickCount;
end;
function CurrentClock : cardinal;
begin
    result := WIndows.GetTickCount - lastTick
end;



function ShadeColor(main, shade : TColor; percent : double) : TColor;
var r,g,b : byte;
begin
    main := ColorToRGB(main);
    shade := ColorToRGB(shade);

    if shade=0 then begin
        result := main;
        EXIT;
    end;
    r:=GetRValue(main);
    r := min(254,Trunc(r*(1-percent) + GetRValue(Cardinal(shade)) * percent));
    g:=GetGValue(main);
    g := trunc(min(254,g*(1-percent) + GetGValue(Cardinal(shade)) * percent));
    b:=GetBValue(main);
    b := Trunc(min(254,b*(1-percent) + GetBValue(Cardinal(shade)) * percent));

    result := RGB(r,g,b);
end;




function GetUndoPath : string;
begin
    result := TPath.Combine(UnitUtil.GetINIPathOnly, 'undo');
end;


function ContainsOnly(source: string; c : Char) : boolean;
var i : char;
begin
    result := false;
    for i in source do begin
        if i <> c then EXIT;
    end;
    result := true;
end;

function ContainsDigit(source : string) : boolean;
var
    c : char;
begin
    for c in source do begin
        if isDigit(c) then begin
            result := true;
            EXIT;
        end;
    end;
    result := false;
end;

function IsIllegalFilename(filename: string) : boolean;
begin
	result := (pos('?',filename) <> 0)
		or (pos('<',filename) <> 0)
		or (pos('>',filename) <> 0)
        or (pos('"',filename) <> 0)
        or (pos(':',filename) <> 0)
        or (pos('|',filename) <> 0)
        or (pos('?',filename) <> 0)
        or (pos('*',filename) <> 0);
end;


function ExpandEnvironment(s : string): string;
const MY_LEN = 2 * MAX_PATH;  // TEST to allow longer filenames (Win10 change)
var
    buf: array[0..MY_LEN] of Char;
    i: DWORD;
begin
    i := ExpandEnvironmentStrings(PChar(s), buf, MY_LEN);
    if i = 0 then begin
        result := s;
    end else begin
        result := Trim(buf);
    end;
end;


function MouseInControl(c : TControl) : boolean;
begin
    result := PtInRect(c.ClientRect, c.ScreenToClient(mouse.CursorPos)); 
end;

function SetBlueColor(c : TColor; saturation : double) : TColor;
var r,g,b : byte;
begin
    c := ColorToRGB(c);
    r := GetRValue(c);
    g := GetGValue(c);
    b := GetBValue(c);

    result := RGB(
        min(r,255),
        min(g,255),
        min(trunc( b * saturation),255)
    );

end;

procedure ShowMessageI(i : integer);
begin
    Dialogs.ShowMessage(IntToStr(i));
end;

function ContainsTerm(term : string; search:string) : boolean;
begin
    result := Pos(term, search) <> 0;
end;
function StringIn(source : string; search : array of string) : boolean;
var i : integer;
begin
    result := false;
    for i := low(search) to high(search) do begin
        if source = search[i] then begin
            result := true;
            EXIT;
        end;
    end;
end;


procedure ShowPopupRight(b : TControl; pm : TPopupMenu);
var p : TPoint;
begin
    p := b.ClientToScreen(point(b.width, 0));
    pm.Popup(p.x, p.y);
end;

procedure ShowPopupUnder(b : TControl; pm : TPopupMenu);
var p : TPoint;
begin
    p := b.ClientToScreen(point(0, b.height));
    pm.Popup(p.x, p.y);
end;


var lv : TListView;
function GetFileList() : TListView;
begin
    if lv = nil then begin
        lv := frmmain.lvFileList;
    end;
    result := frmmain.lvFileList;
end;


procedure CloneWinControl(inwc : TWinControl; outwc : TWinControl; TopOffset : integer = 0);
var i : integer;
    lbl : TLabel;
    cbx : TComboBox;
    txt : TEdit;
    pnl : TPanel;
    rb : TRadioButton;

begin
    TPanel(outwc).autosize := false;
    outwc.Width := inwc.Width;
    outwc.Height := inwc.Height;

    for I := 0 to inwc.ControlCount  - 1 do begin
        if inwc.controls[i] is TPanel then begin
            pnl := TPanel.Create(outwc);
            pnl.Parent := outwc;
            pnl.top := TPanel(inwc.Controls[i]).top + TopOffset;
            pnl.left := TPanel(inwc.Controls[i]).left;
            pnl.width := TPanel(inwc.Controls[i]).width;
            pnl.anchors := TPanel(inwc.Controls[i]).anchors;
            pnl.align := TPanel(inwc.Controls[i]).align;

            CloneWinControl(TPanel(inwc.Controls[i]), pnl);
        end;
        if inwc.controls[i] is TComboBox then begin
            cbx := TComboBox.Create(outwc);
            cbx.parent := outwc;
            cbx.text := TComboBox(inwc.controls[i]).text;
            cbx.Items := TComboBox(inwc.controls[i]).items;
            cbx.ItemIndex := TComboBox(inwc.controls[i]).ItemIndex;
            cbx.Style := TComboBox(inwc.controls[i]).style;
            cbx.Top := TComboBox(inwc.controls[i]).top + TopOffset;
            cbx.Left := TComboBox(inwc.controls[i]).Left;
            cbx.Width := TComboBox(inwc.controls[i]).Width;
            cbx.height := TComboBox(inwc.controls[i]).height;
            cbx.Anchors := TComboBox(inwc.controls[i]).anchors;
            cbx.Align := TComboBox(inwc.controls[i]).Align;
        end;

        if inwc.controls[i] is TLabel then begin
            lbl := TLabel.Create(outwc);
            lbl.parent := outwc;
            lbl.Caption := TLabel(inwc.controls[i]).caption;
            lbl.Top := TLabel(inwc.controls[i]).top + TopOffset;
            lbl.Left := TLabel(inwc.controls[i]).Left;
            lbl.Width := TLabel(inwc.controls[i]).Width;
            lbl.height := TLabel(inwc.controls[i]).height;
            lbl.Anchors := TLabel(inwc.controls[i]).anchors;
            lbl.Align := TLabel(inwc.controls[i]).Align;
        end;

        if inwc.controls[i] is TRadioButton then begin
            rb := TRadioButton.Create(outwc);
            rb.Parent := outwc;
            rb.caption := TRadioButton(inwc.controls[i]).caption;
            rb.Top := TRadioButton(inwc.controls[i]).top + TopOffset;
            rb.Left := TRadioButton(inwc.controls[i]).Left;
            rb.Width := TRadioButton(inwc.controls[i]).Width;
            rb.Anchors := TRadioButton(inwc.controls[i]).anchors;
            rb.Align := TRadioButton(inwc.controls[i]).Align;
            rb.Checked := TRadioButton(inwc.controls[i]).Checked;

        end;
        if inwc.controls[i] is TEdit then begin
            txt := TEdit.Create(outwc);
            txt.parent := outwc;
            txt.text := TEdit(inwc.controls[i]).text;
            txt.Top := TEdit(inwc.controls[i]).top + TopOffset;
            txt.Left := TEdit(inwc.controls[i]).Left;
            txt.Width := TEdit(inwc.controls[i]).Width;
            txt.Anchors := TEdit(inwc.controls[i]).anchors;
            txt.Align := TEdit(inwc.controls[i]).Align;
        end;

    end;
    //TPanel(outwc).autosize := true;

end;



 function dimcolor(c : TColor; alpha : double = 0.95) : TColor;
    var r,g,b: byte;
    begin
        c := ColorToRGB(c);

        r := min(trunc(GetRValue(Cardinal(c))*alpha),$FF);
        g := min(trunc(GetGValue(Cardinal(c))*alpha),$FF);
        b := min(trunc(GetBValue(Cardinal(c))*alpha),$FF);

        result := RGB(r,g,b);
    end;

function SafeInt(s : string; default : integer = 0) : integer;
begin
    result := default;
    if s = '' then Exit;

    try
        result := StrToInt(s);
    except
    end;
end;


function GetINIPath: string;
begin
    result := GetINIPathOnly + 'rm.ini';
end;

function GetINIPathOnly: string;
begin
    result := IncludeTrailingPathDelimiter(ExtractFilePath(application.ExeName));
end;

function StripNumberFormat(s : string ) : string;
var i : integer;
begin

    try

        result := '';
        for i := length(s) downto 1 do begin
            if CharInSet(s[i], ['0'..'9','+','-', FormatSettings.DecimalSeparator]) then begin
                result := s[i] + result;
            end;
        end;
        //
    except
        s := '0';

    end;



    
end;

// Needed for files over 4 gigs in size
function GetSizeOf(fullname : string) : double;
var hFile : cardinal;
    LoDWord, HiDWord: LongWord;
begin
    hFile := Windows.CreateFile(PChar(fullname),           // open MYFILE.TXT
                GENERIC_READ,              // open for reading
                FILE_SHARE_READ,           // share for reading
                nil,                      // no security
                OPEN_EXISTING,             // existing file only
                FILE_ATTRIBUTE_NORMAL or
                FILE_ATTRIBUTE_HIDDEN or
                FILE_ATTRIBUTE_SYSTEM or
                FILE_ATTRIBUTE_DIRECTORY ,     // normal file
                0);                     // no attr. template

    LoDWord := Windows.GetFileSize(hFile, @HiDWord);
    Windows.CloseHandle(hFile);

    if (LoDWord = MAXDWORD) and (GetLastError <> 0) then
      Result := 0
    else begin
      Result := MAXDWORD;
      Result := (HiDWord * (Result + 1)) + LoDWord;
    end;
end;

initialization
    lv := nil;
end.
