unit UnitVideo;

interface


uses Classes, INIFIles{inifiles for THashedStringList};

const supported_extension : array[0..7] of string = (
	'.AVI',
    '.MPG','.MPEG',
    '.MOV','.MP4','.M4V',
    '.ASF','.WMV'
);




function GetVideoTags(name : string; tags : THashedStringList) : boolean;

function FileSupported(name : string) : boolean;

procedure GetExtraTags(tags : THashedStringList);

implementation

uses SysUtils,  StrUtils, UnitUtil, Windows, DateUtils;

var mytags : THashedStringList;


{$Q-}
function SwapEndian(Value: cardinal): cardinal; register; overload;
asm
  bswap eax
end;
{$Q+}

procedure GetExtraTags(tags : THashedStringList);
begin
    tags.AddStrings(mytags); 
end;

function FileSupported(name : string) : boolean;
begin
	name := uppercase(ExtractFileExt(name));
    result := StringIn(name, supported_extension);
end;

{ AVI,MPG/MPEG, MOV/MP4/M4V, ASF/WMV }
function GetRes_MPEG_New(name : string;  var x, y : longint) : boolean; forward;
function GetRez_AVI(name : string;  var x, y : longint) : boolean; forward;
function GetRes_MPEG(name : string;  var x, y : longint) : boolean; forward;
function GetRes_QT(name : string;  var x, y : longint) : boolean; forward;
function GetRes_WMV(name : string;  var x, y : longint) : boolean; forward;


function GetVideoTags(name : string; tags : THashedStringList) : boolean;
var
	ext : string;
    x,y : longint;
begin
	mytags.Clear;

	ext := lowercase(ExtractFileExt(name));
	if ext = '.avi' then begin
    	result := GetRez_Avi(name, x,y);
    end else if StringIn(ext,['.mpg','.mpeg']) then begin
    	result := GetRes_MPEG_New(name, x,y);
    end else if StringIn(ext,['.mov','.mp4','.m4v']) then begin
    	result := GetRes_QT(name, x,y);
    end else if StringIn(ext,['.asf','.wmv']) then begin
    	result := GetRes_WMV(name, x,y);
    end;

    tags.Values['?v_wd?'] := IntToStr(x);
    tags.Values['?v_ht?'] := IntToStr(y);

    GetExtraTags(tags);
end;

function GetRez_AVI(name : string;  var x, y : longint) : boolean;
var s : TFileStream;
	buff : array[0..3] of ansichar;
    c : cardinal;
    seconds_duration : int64;

    hr,mn,sc : word;
begin
	result := false;
	try
        s := TFileStream.Create(name, fmOpenRead);
		s.Seek(0, soFromBeginning);
        s.read(buff, sizeof(buff));

        if not (string(buff) = 'RIFF') then
        	EXIT;
        s.seek(32, soFromBeginning); {ms per frame}
        s.read(c, sizeof(c));

        seconds_duration := c;

    	s.seek(48, soFromBeginning);{total frames}
        s.read(c, sizeof(c));




        s.Seek(64, soFromBeginning);
        s.Read(x,4);
        s.Seek(68, soFromBeginning);
        s.read(y,4);


        seconds_duration := (seconds_duration * c) div 1000000;

        sc := seconds_duration mod 60;
        mn := (seconds_duration div 60) mod 60;
        hr := (seconds_duration div 60) div 60;

        mytags.values['?v_len?'] := format('%.2d-%.2d-%.2d',[hr,mn,sc]);

        result := true;
	finally
    	s.free;
    end;
end;


// $000001B3 Video Start Block
//
// MPEG video block
// Start code                       32 bits
			// horizontal frame size            12 bits
			// vertical frame size              12 bits
			// pixel aspect ratio                4 bits
			// frame rate                        4 bits
{
 1 = 23.976 frames/sec
 2 = 24
 3 = 25
 4 = 29.97
 5 = 30
 6 = 50
 7 = 59.94
 8 = 60}
			// bitrate                          18 bits
			// marker bit                        1 bit
			// VBV buffer size                  10 bits
			// constrained parameter flag        1 bit
			// intra quant. matrix flag          1 bit
			// intra quant. matrix values      512 bits (present if matrix flag == 1)
			// non-intra quant. matrix flag      1 bit
			// non-intra quant. matrix values  512 bits (present if matrix flag == 1)



function GetRes_MPEG_New(name : string;  var x, y : longint) : boolean;
     var s : TFileStream;
	i : longint;
    c : cardinal;
    quad : array[0..3] of byte;
    buff : array[0..2] of byte;
    ender : array [0..8192-1] of byte;
    w : word;
    hr,hr2, mn,mn2, sc,sc2 : word;
    cnt : integer;
const
	PACK_HEADER = $000001BA;
    SYSTEM_HEADER = $000001BB;
    PADDING_HEADER = $000001BE;
    SEQUENCE_HEADER = $000001B3;
    VIDEO_HEADER = $000001E0;
    AUDIO_HEADER = $000001C0;

    TIMESTAMP_HEADER = $000001B8;
const MAX_READS = 40000;

	procedure Decode(stamp : cardinal; var hr,mn,sc : word);
    begin
     	hr := stamp and $07C00000 shr 26;
        mn := stamp and $03F00000 shr 20;
        sc := stamp and $0007E000 shr 13; 
    end;
begin
	// TODO: find a better way to locate the Video Start Block header
    // This method may produce false positives

	result := false;
	try
        s := TFileStream.Create(name, fmOpenRead);

        s.Seek(0, soFromBeginning);
        c := 0;
    	i := 0;

        s.read(c, sizeof(c));
        c := SwapEndian(c);

        if c  = PACK_HEADER then begin
        	while (true) do begin

                case c of
                PACK_HEADER: s.Seek(8, soFromCurrent);
                SYSTEM_HEADER, PADDING_HEADER, AUDIO_HEADER: begin
                    s.read(w,sizeof(w));
                    w := (($FF and w) shl 8) or (($FF00 and w) shr 8);
                    s.Seek(w,soFromCurrent);
                end;
                VIDEO_HEADER:
                	s.read(w,sizeof(w));
                    {skip the length, for the program to search inside for a sequence_header}
                SEQUENCE_HEADER: begin
                    s.read(buff, sizeof(buff));

                    x := buff[0] shl 4  or ((buff[1] and $F0) shr 4);
                    y := buff[2] or ((buff[1] and $0F) shl 8);

                    s.Seek(5, soFromCurrent);
                    result := true;
                    //BREAK;
                end;
                TIMESTAMP_HEADER: begin
                    s.read(c, sizeof(c));
                    c := SwapEndian(c);
                    Decode(c, hr,mn,sc);
                    if result then BREAK;
                end;
                end;

                s.Read(c,sizeof(c));
                c := SwapEndian(c);

                if s.position > MAX_READS then
                	BREAK;
            end;

            s.Position := s.size - sizeof(ender); 
            s.read(ender, sizeof(ender));
            i := 0;
            hr2 := 0;
            mn2 := 0;
            sc2 := 0;
            cnt := 0;
            while result do begin
                if ender[i] = 0 then begin
                	inc(i);
                	if ender[i]=0 then begin
                        inc(i);
                    	if ender[i] = 1 then begin
                        	inc(i);
                            if ender[i] = TIMESTAMP_HEADER and $FF then begin
                                c := ender[i+1] shl 24;
                                c := c or (ender[i+2] shl 16);
                                c := c or (ender[i+3] shl 8);
                                c := c or (ender[i+4]);

                                Decode(c,hr2,mn2,sc2);

                                BREAK;
                            end;
                            inc(i);
                        end;
                    end else begin
                        inc(i);
                    end;
                end else begin
                	inc(i);
                end;

                if i+3 > high(ender) then begin
                	inc(cnt);
                    s.Position := s.size - (sizeof(ender) * cnt);
                    s.read(ender,sizeof(ender));
                    i := 0;
                end;

                if cnt = 50 then BREAK;
            end;

            if (hr2+mn2+sc2) > 0 then begin
            	if hr<hr2 then hr2 := hr2 - hr;
                if mn<mn2 then mn2 := mn2 - mn;
                if sc<sc2 then sc2 := sc2 - sc;
               	mytags.values['?v_len?'] := format('%.2d-%.2d-%.2d',[hr2,mn2,sc2]);
            end;

        end else begin
            s.Position := 0;
            while
                not ((quad[0]=0) and (quad[1]=0) and (quad[2]=1) and (quad[3]=$b3))
                and (c < MAX_READS) do begin
                s.Read(quad,sizeof(quad));
                inc(c);

                if c = 1 then begin
                    if not ((quad[0]=0) and (quad[1]=0) and (quad[2]=1))  then
                        EXIT;
                end;
            end;
            if (c < MAX_READS) then begin
                s.read(buff, sizeof(buff));

                x := buff[0] shl 4  or ((buff[1] and $F0) shr 4);
                y := buff[2] or ((buff[1] and $0F) shl 8);
                result := true;
            end;
        end;
	except

    end;
    s.free;    
end;

function GetRes_MPEG(name : string;  var x, y : longint) : boolean;
var s : TFileStream;
	i : longint;
    c : cardinal;
    quad : array[0..3] of byte;
    buff : array[0..2] of byte;


const PACK_HEADER = $000001BA;



const MAX_READS = 40000;
begin
	// TODO: find a better way to locate the Video Start Block header
    // This method may produce false positives

	result := false;
	try
        s := TFileStream.Create(name, fmOpenRead);

        s.Seek(0, soFromBeginning);
        c := 0;
    	i := 0;
        while
        	not ((quad[0]=0) and (quad[1]=0) and (quad[2]=1) and (quad[3]=$b3))
        	and (c < MAX_READS) do begin
        	s.Read(quad,sizeof(quad));
            inc(c);

            if c = 1 then begin
            	if not ((quad[0]=0) and (quad[1]=0) and (quad[2]=1))  then
                	EXIT;
            end;
        end;
       	if (c < MAX_READS) then begin
            s.read(buff, sizeof(buff));

            x := buff[0] shl 4  or ((buff[1] and $F0) shr 4);
            y := buff[2] or ((buff[1] and $0F) shl 8);
            result := true;
       	end;
	finally
    	s.free;
    end;
end;

// initial MOV header
//XX XX XX XX 66 74 79 70
//....ftyp



function GetRes_QT(name : string;  var x, y : longint) : boolean;
var s : TFileStream;

	cnt : cardinal;
    bigsize : Int64;
    atomtype : ansistring;

    endoffile : boolean;
const
    containers = 'moov trak clip matt edts tref mdia minf dinf udta cmov rmra rmda gmhd stbl meta';

    function GetAtom : string;
    var
    	buff : array[0..3] of ansichar;
        size : cardinal;

    const
    	toplevel = 'ftyp pdin moov moof mfra free skip junk wide pnot pict meta meco uuid mdat';
    begin
    	s.read(size, sizeof(size));
        s.read(buff, sizeof(buff));

        if (size = 0) and ContainsTerm(string(buff), toplevel)  then begin
            endoffile := true;
        end;
        if (size = 0) and (string(buff)='') then begin
            endoffile := true;
        end;

        bigsize := SwapEndian(size);
        bigsize := bigsize - 8;

        result := string(buff);
    end;
    function UTCToLocalTime(inDate: TDateTime): TDateTime;
    var
        st1, st2 : TSystemTime;
        tzi : TTimeZoneInformation;
    begin
        GetTimeZoneInformation(tzi);
        DateTimeToSystemTime(inDate, st1);
        SystemTimeToTzSpecificLocalTime(@tzi, st1, st2);

        result := SystemTimeToDateTime(st2);
    end;
    function GetBigDate(inDate : int64) : TDateTime;
    begin
        result := UnixToDateTime(inDate);
        result := dateutils.IncYear(result, -66); // 1904 vs 1970 timestamps

        result := UTCToLocalTIme(result);
    end;
    function GetSmallDate(inDate : cardinal) : TDateTime;
    var bigInDate : int64;
    begin
        bigInDate := inDate;
        result := GetBigDate(bigInDate);
    end;

    procedure HandleUserData;
    var
    	buff : array[0..3] of ansichar;
        datasize : cardinal;
        databig : int64;
        datatype : string;
        str : AnsiString;
        usersize : int64;
    begin
        if bigsize <= 0 then EXIT;

        usersize := s.Position + bigsize;

        while (s.Position < usersize) do begin
            if s.Position >= s.Size then BREAK;

            s.read(datasize, sizeof(datasize));
            databig := SwapEndian(datasize) - 8;

            s.read(buff, sizeof(buff));
            datatype := string(buff);




            if datatype[1] = chr($A9) then begin
                // dunno what this size-looking data is for
                // looks like a string length word and language word

                datatype[1] := '_';
                s.Seek(4, soFromCurrent);
                setlength(str, databig-4);
                s.read(str[1], databig-4);
            end else begin
                setlength(str, databig);
                s.read(str[1], databig);
            end;


            // this can contain a $00 0D X X and then the string
            // no idea what this extra data is

            mytags.values['?v_qt_'+datatype+'?'] := str;
        end;

        s.Seek(usersize, soFromBeginning);
    end;

    procedure HandleTrackHeader;
    var
        dt : TDateTime;
        dateC, dateM : cardinal; // 1904
        bigdateC, bigDateM : int64;
        bigdate : int64;
        datesize : integer;
        po : Int64;
        version : byte;
        b : byte;
        c : cardinal;
        duration : int64;
        tempx, tempy : cardinal;
    begin
        po := s.Position;
        // version byte
        s.Read(version, sizeof(version));


        // 3 bytes of flags
        s.Read(b, sizeof(b));
        c := b shl 16;
        s.Read(b, sizeof(b));
        c := c or (b shl 8);
        s.Read(b, sizeof(b));
        c := c or b;

        // track enabled and used in the movie
        if ((c and $03) > 0)  then begin
            datesize := 0;
            if version = 0 then begin
                s.Read(dateC, sizeof(dateC));
                s.Read(dateM, sizeof(dateM));
                datesize := sizeof(dateC) * 2;

                dateC := SwapEndian(dateC);
                dateM := SwapEndian(dateM);
            end;

            if version = 0 then begin
                if datec <> 0 then begin
                    dt := GetSmallDate(dateC);
                    mytags.values['?v_qt_date_created?'] := DateTimeToStr(dt);
                end;

                if datem <> 0 then begin
                    dt := GetSmallDate(dateM);
                    mytags.values['?v_qt_date_modified?'] := DateTimeToStr(dt);
                end;
            end;



            s.seek(16-datesize, soFromCurrent);
            s.read(c, sizeof(c));
            duration := SwapEndian(c);



            s.seek(50, soFromCurrent);
            s.read(c, sizeof(c));
            c := SwapEndian(c);
            tempx := c;
            s.read(c, sizeof(c));
            c := SwapEndian(c);
            tempy := c;

            if (tempx=0) or (tempy=0) then begin
            end else begin
                x := tempx;
                y := tempy;

                result := true;
            end;
        end;

        // rollback
        s.Seek(po, soFromBeginning);
        s.Seek(bigsize, soFromCurrent);
    end;

    procedure HandleIList;
    var
        ilstsize : int64;
        po : int64;
        atomtype : string;
        c : cardinal;
        str : string;
    begin
        // UNTESTED CODE, never found an IList
        ilstsize := s.Position + bigsize;
        po := s.position;
        while s.Position < ilstsize do begin
            atomtype := GetAtom;
            s.read(c, sizeof(c));
            c := SwapEndian(c) - 16;

            s.Seek(12, soFromCurrent);

            setlength(str, c);
            s.read(str[1], c);

            if atomtype[1] = chr($A9) then atomtype[1] := '_';
            mytags.values['?v_qt_'+atomtype+'?'] := str;
        end;

        s.Position := po;
        s.Seek(bigsize, soFromCurrent);
    end;

    procedure HandleMediaHeader;
    var
        po : int64;
        version : byte;
        c : cardinal;
        duration : cardinal;
        timescale : cardinal;
        seconds : cardinal;
        hr,mn,sc : word;
    begin
        po := s.Position;
        s.Read(version, sizeof(version));
        // 32 bit value
        if version = 0 then begin
            s.seek(12-sizeof(version), soFromCurrent);
            s.read(c, sizeof(c));
            timescale := SwapEndian(c);
            s.Read(c, sizeof(c));
            duration := SwapEndian(c);
            if timescale <> 0 then begin
                seconds := duration div timescale;
                sc := (seconds mod 60);
                mn := (seconds div 60) mod 60;
                hr := (seconds div 60) div 60;

                mytags.values['?v_len?'] := format('%.2d-%.2d-%.2d',[hr,mn,sc]);
            end;
        end;
        // rollback and skip this atom
        s.Position := po;
        s.Seek(bigsize, soFromCurrent);
    end;
begin
	result := false;
	try
        s := TFileStream.Create(name, fmOpenRead);
		s.Seek(0, soFromBeginning);
        cnt := 0;

		// 4 bytes size (-8 bytes, included header size), 4 byte header ID
        while (true) do begin
            if s.Position >= s.size then
            	BREAK;

        	atomtype := GetAtom;
            if endoffile then BREAK;

            if cnt = 0 then begin
                if atomtype <> 'ftyp' then EXIT;
            end;
            inc(cnt);


            if (ContainsTerm(atomtype, containers)) then begin
                if (atomtype = 'udta') then begin
                    HandleUserData;
                end;
            	// if header is a container, skip to the first nested header
                if atomtype = 'meta' then begin
                	s.seek(4, soFromCurrent);
                end;

                CONTINUE;
            end;

            // If tkhd header has an empty resolution, roll back and skip
            // to the next atom
           	if (atomtype = 'ilst') then begin
                HandleIlist;
            end else if (atomtype = 'mdhd') then begin
                HandleMediaHeader
            end else if (atomtype = 'tkhd') then begin
            	HandleTrackHeader;
			end else begin
            	if s.Position + bigsize >= s.Size  then BREAK;
                s.Seek(int64(bigsize), soFromCurrent);
            end;
    	end;
        s.free;
	except
    	s.free;
    end;
end;

// tkhd header
// version	1
//
//




// Header Object
// GUID 16 bytes
// size 8 bytes
// header count 4 bytes
// reserved 2 bytes

//'GETID3_ASF_Header_Object'                       => '75B22630-668E-11CF-A6D9-00AA0062CE6C',
// 'GETID3_ASF_Data_Object'                         => '75B22636-668E-11CF-A6D9-00AA0062CE6C',

// 'GETID3_ASF_Stream_Properties_Object'            => 'B7DC0791-A9B7-11CF-8EE6-00C00C205365',

// 'GETID3_ASF_Video_Media'                         => 'BC19EFC0-5B4D-11CF-A8FD-00805F5C442B'
// 'GETID3_ASF_Metadata_Object'                     => 'C5F8CBEA-5BAF-4877-8467-AA8C44FA4CCA',
//'GETID3_ASF_Header_Extension_Object'             => '5FBF03B5-A92E-11CF-8EE3-00C00C205365',
// 'GETID3_ASF_Metadata_Library_Object'             => '44231C94-9498-49D1-A141-1D134E457054',
//'GETID3_ASF_Content_Description_Object'          => '75B22633-668E-11CF-A6D9-00AA0062CE6C',
//'GETID3_ASF_Extended_Content_Description_Object' => 'D2D0A440-E307-11D2-97F0-00A0C95EA850',


//'GETID3_ASF_File_Properties_Object'              => '8CABDCA1-A947-11CF-8EE4-00C00C205365',

function GetRes_WMV(name : string;  var x, y : longint) : boolean;
var s : TFileStream;
	buff : array[0..3] of ansichar;

    size : int64;
    smallsize : cardinal;
    po : int64;
    headcnt : longint;
    guid : cardinal;
    cnt : cardinal;

    metasize : int64;
    tempsize : int64;
    temppo : int64;
    w : word;
    b : byte;
    c : cardinal;
    i64 : int64;
    i,j,k : integer;
    hr,mn,sc : word;


    procedure ParseExtendedContentDescriptionObject;
    var i,j : integer;
    	cnt : word;
        namelen : word;
        datatype : word;
        datalen : word;
        dataname, datavalue : string;
        fakechar : word;
    begin
    	s.read(cnt, sizeof(cnt));
        for i := 0 to cnt - 1 do begin
            s.read(namelen, sizeof(namelen));
            dataname := '';
            for j := 0 to (namelen div 2) - 1 do begin
                s.read(fakechar, sizeof(fakechar));
                dataname := dataname + char(byte(fakechar and $FF));
                if fakechar = 0 then BREAK;
            end;
            dataname := leftstr(dataname, length(dataname)-1);
            s.read(datatype, sizeof(datatype));
            s.read(datalen, sizeof(datalen));
            case datatype of
            0:  begin
                for j := 0 to (datalen div 2) - 1 do begin
                    s.read(fakechar, sizeof(fakechar));
                    datavalue := datavalue + char(byte(fakechar and $FF));
                    if fakechar = 0 then BREAK;

                end;
                datavalue := leftstr(datavalue, length(datavalue)-1);
            end;
            1: begin
                s.read(b, sizeof(b));
                datavalue := IntToStr(b);
            end;
            2: begin
                s.read(w, sizeof(w));
                datavalue := IntToStr(w);
            end;
            3:  begin
                s.read(c, sizeof(c));
                datavalue := IntToStr(c);
            end;
            4:  begin
                s.read(i64, sizeof(i64));
                datavalue := IntToStr(i64);
            end;
            5:  begin
                s.read(w, sizeof(w));
                datavalue := IntToStr(w);
            end;
            end;
            dataname := lowercase(dataname);
            mytags.Values['?v_ms_' + string(dataname) + '?'] := string(datavalue);
        end;
    end;
    procedure ParseContentDescObject;
    var
    	lens : array [0..4] of word;
        i,j : integer;
        dataname, datavalue : string;
        fakechar : word;
    begin
        s.read(lens, sizeof(lens));
        for i := low(lens) to high(lens) do begin
            case i of
            0: dataname := 'title';
            1: dataname := 'author';
            2: dataname := 'copyright';
            3: dataname := 'description';
            4: dataname := 'rating';
            end;
            datavalue := '';
            for j := 0 to (lens[i] div 2) - 1 do begin
                s.read(fakechar, sizeof(fakechar));
                datavalue := datavalue + char(byte(fakechar and $FF));
                if fakechar = 0 then BREAK;
            end;
            datavalue := leftstr(datavalue, length(datavalue)-1);

            mytags.Values['?v_ms_' + string(dataname) + '?'] := string(datavalue);
        end;
    end;
const
	HEADER_OBJECT = $75B22630;
	STREAM_PROP_OBJECT = $B7DC0791;
    VIDEO_MEDIA_OBJECT = $BC19EFC0;
    HEADER_EXT_OBJECT = $5FBF03B5;
    METADATA_OBJECT = $C5F8CBEA;
    METADATA_LIBRARY_OBJECT = $44231C94;
    CONTENT_DESC_OBJECT = $75B22633;
    EXT_CONTENT_DESC_OBJECT = $D2D0A440;
    FILE_PROP_OBJECT = $8CABDCA1;
begin
	result := false;
	try
        s := TFileStream.Create(name, fmOpenRead);

		s.Seek(0, soFromBeginning);
        s.read(guid, sizeof(guid));
        if guid <> HEADER_OBJECT then
        	EXIT;
        s.Seek(12, soFromCurrent);
        s.read(size, sizeof(size));
        s.read(headcnt, sizeof(headcnt));
        //size := size - 24;
        s.Seek(2, soFromCurrent);


        for i := 0 to headcnt - 1 do begin
            s.read(guid, sizeof(guid));
        	s.Seek(12, soFromCurrent);
            s.read(size, sizeof(size));
            size := size - 24;

           	if (guid = CONTENT_DESC_OBJECT) then begin
            	po := s.Position;
                ParseContentDescObject;
                s.Position := po;
            end else if (guid = STREAM_PROP_OBJECT) then begin
            	po := s.Position;
            	s.read(guid, sizeof(guid));
                if (guid <> VIDEO_MEDIA_OBJECT) then begin
                	s.Position := po;  // roll back and skip the object as normal
                end else begin
                    s.Seek(50,soFromCurrent);

                    s.read(x, sizeof(x));
                    s.read(y, sizeof(y));

                    result := true;
                    //EXIT;
                end;
                s.Position := po;
            end else if (guid = FILE_PROP_OBJECT) then begin
            	po := s.Position;
                //16,8,8,8
                s.seek(40, soFromCurrent);
                s.read(i64, sizeof(i64));

                i64 := i64 div 10000000;

                sc := (i64 mod 60);
                mn := (i64 div 60) mod 60;
                hr := (i64 div 60) div 60;

                mytags.values['?v_len?'] := format('%.2d-%.2d-%.2d',[hr,mn,sc]);

                s.Position := po;
            end;

            s.Seek(size, soFromCurrent);
            if s.Position >= s.size then
            	BREAK;
       	end;
	finally
    	s.free;
    end;
end;

{
#########################################################
# ASF GUID signatures
#
#base ASF object guids
use constant Header                        => 0x75b22630;
use constant data                          => 0x75b22636;
use constant simple_index                  => 0x33000890;

#header object guids
use constant file_properties               => 0x8cabdca1;
use constant stream_properties             => 0xb7dc0791;
use constant stream_bitrate_properties     => 0x7bf875ce;
use constant content_description           => 0x75b22633;
use constant extended_content_encryption   => 0x298ae614;
use constant script_command                => 0x1efb1a30;
use constant marker                        => 0xf487cd01;
use constant header_extension              => 0x5fbf03b5;
use constant bitrate_mutual_exclusion      => 0xd6e229dc;
use constant codec_list                    => 0x86d15240;
use constant extended_content_description  => 0xd2d0a440;
use constant error_correction              => 0x75b22635;
use constant stream_bitrate_porperties     => 0x7bf875ce;
use constant padding                       => 0x1806d474;

#stream properties object stream type guids
use constant audio_media                   => 0xf8699e40;
use constant video_media                   => 0xbc19efc0;
use constant command_media                 => 0x59dacfc0;

#stream properties object error correction type guids
use constant no_error_correction           => 0x20fb5700;
use constant audio_spread                  => 0xbfc3cd50;

#mutual exclusion object exclusion type guids
use constant mutex_bitrate                 => 0xd6e22a01;
use constant mutex_unknown                 => 0xd6e22a02;

#from mplayer
use constant audio_conceal_none            => 0x49f1a440;
use constant header_2_0                    => 0xD6E229D1;
#########################################################
}




            	{
if (guid = HEADER_EXT_OBJECT) or (guid = METADATA_LIBRARY_OBJECT)  then begin
                po := s.position;

                s.Seek(18, soFromCurrent);
                s.read(smallsize, sizeof(smallsize));
                smallsize := smallsize - 46;
                metasize := s.Position + smallsize;

                while s.Position < metasize do begin
                    s.read(guid, sizeof(guid));
                    s.Seek(12, soFromCurrent);
                    s.read(tempsize, sizeof(tempsize));
                    tempsize := tempsize - 24;
                    if guid = METADATA_OBJECT then begin
                    	temppo := s.Position;

                        s.Read(w, sizeof(w));
                        for j := 0 to w - 1 do begin
                            s.seek(4,soFromCurrent);
                            s.read(namelen, sizeof(namelen));
                            s.read(datatype, sizeof(datatype));
                            s.read(datalen, sizeof(datalen));
                            dataname := '';
                            datavalue := '';
                            for k := 0 to (namelen div 2) - 1 do begin
                                s.read(fakechar, sizeof(fakechar));
                                dataname := dataname + char(byte(fakechar and $FF));
                            end;
                            dataname := leftstr(dataname, length(dataname)-1);
                            case datatype of
                            0:  begin
                                for k := 0 to (datalen div 2) - 1 do begin
                                    s.read(fakechar, sizeof(fakechar));
                                    datavalue := datavalue + char(byte(fakechar and $FF));
                                end;
                                datavalue := leftstr(datavalue, length(datavalue)-1);
                            end;
                            1: begin
                                s.read(b, sizeof(b));
                                datavalue := IntToStr(b);
                            end;
                            2: begin
                            	s.read(w, sizeof(w));
                                datavalue := IntToStr(w);
                            end;
                            3:  begin
                            	s.read(c, sizeof(c));
                                datavalue := IntToStr(c);
                            end;
                            4:  begin
                                s.read(i64, sizeof(i64));
                                datavalue := IntToStr(i64);
                            end;
                            5:  begin
                                s.read(w, sizeof(w));
                                datavalue := IntToStr(w);
                            end;
                            end;
                            dataname := lowercase(dataname);
                            mytags.Values['?v_ms_' + string(dataname) + '?'] := string(datavalue);
                        end;

                        s.Position := temppo;
                    end else begin
                	end;
                    s.seek(tempsize, soFromCurrent);
                end;

                s.Position := po;  }



initialization
begin
	mytags := THashedStringList.create;
end;
end.
