unit UnitWildcards;
{
    Purpose:
        (Search/Replace/Find)
        for string that can contain ? and * wildcards
        ? = exactly 1 char
        * = zero or more characters

    Updates:
        New "filemode" to accept inconsitant "*.*" and "*."
        Bug fix for special case when search string was a single '*'

}
interface

uses Classes {for TStringList}, UnitStates, System.RegularExpressions;


type TRegExpWildcards = class;  // declare but don't define for cyclic reference
TWildcards = class(TObject)
    private
        //scontentOriginal : string;
        scontent : string;
        spattern : string;
        curIndex : cardinal;
        isFilemode : boolean;
        isAlwaysTrue : boolean;
        isExtensionless : boolean;
        states : array of TState;

        Proxy : TRegExpWildcards;
        function GetMatch(var startIndex : cardinal; var endIndex : cardinal) : boolean;
        function GetFirstLiteralIndex : integer;
        function HasLeadingAsterisk : boolean;

    public
        destructor Destroy; override;

        procedure AssignFilemode(value : boolean); virtual;
        procedure SetSearchString(str : string);   virtual;
        procedure SetSearchPattern(str : string); virtual;

        function GetStr(startindex, endindex : cardinal) : string;
        function FindString : string;
        function Matches : boolean;
        function FirstFound : Boolean; overload;
        function FindFirst(var startIndex : cardinal; var endIndex : cardinal) : boolean; virtual;
        function FindNext(var startIndex : cardinal; var endIndex : cardinal) : boolean; virtual;
        function SearchReplaceWith(str : string; replaceall : boolean) : string; overload; virtual;
        function SearchReplaceWith(s : string; sFind: string; sReplace : string; replaceall : boolean) : string; overload; virtual;
end;



TRegExpWildcards = class(TWildcards)
    private
        //re : TRegExpr;
        re : TRegEx;
        match : TMatch;
        caseSensitive : boolean;
    public
        function LastError : integer;
        procedure SetCaseSensitive(value : boolean);

        constructor Create;
        destructor Destroy; override;
        procedure AssignFilemode(value : boolean); override;
        procedure SetSearchString(str : string); override;
        procedure SetSearchPattern(str : string); override;

        function FirstFound : Boolean; virtual;
        function FindFirst(var startIndex : cardinal; var endIndex : cardinal) : boolean; override;
        function FindNext(var startIndex : cardinal; var endIndex : cardinal) : boolean; override;
        function SearchReplaceWith(str : string; replaceall : boolean) : string; overload; override;
        function SearchReplaceWith(s : string; sFind: string; sReplace : string; replaceall : boolean) : string; overload; override;

end;



{////////////////////}
{//}implementation{//}
{////////////////////}

uses UnitToken, SysUtils, StrUtils, UnitUtil, vcl.dialogs;
var UseRegExp : boolean = false;

// public
destructor TWildcards.Destroy;
begin
    if assigned(proxy) then FreeAndNil(proxy);

    inherited;
end;

procedure TWildcards.AssignFilemode(value: boolean);
begin
    if assigned(Proxy) then begin
        Proxy.AssignFilemode(value);
        EXIT;
    end;

    self.isFilemode := value;
end;
procedure TWildcards.SetSearchString(str : string);
begin
    if assigned(Proxy) then begin
        Proxy.SetSearchString(str);
        EXIT;
    end;

    scontent := str;
    //scontentOriginal := str;
end;
procedure TWildcards.SetSearchPattern(str : string);
var i, j, k : integer;
    FirstLiteral : integer;
    isSearch : boolean;
begin
    Proxy := nil;
    if StrUtils.ContainsText(str, '?reg?') then begin
        str := UnitUtil.ReplaceAll(str, '?reg?', '');

        Proxy := TRegExpWildcards.Create;
        Proxy.SetSearchPattern(str);
        EXIT;
    end;


    self.isAlwaysTrue := false;
    if (self.isFilemode) then begin
        self.isExtensionless := (length(str) <> 0) and (str[length(str)] = '.');
        if (self.isExtensionless) then begin
            str := copy(str,1,length(str)-1);
        end;

        if StringIn(str, ['*','*.*']){pos('?'+str+'?','?*??*.*?') <> 0} then begin
            self.isAlwaysTrue := true;
            EXIT;
        end;

    end;

    spattern := lowercase(str);

    //-------------------------------------------------------
    // Simplify the expression into logical equivilants
    //-------------------------------------------------------

    // turn "*???"/"*???*??" into "???*"/"?????**"
    // this makes the asterisk case last when a search like "asdf??*" is given
    // state engine requires asterisk last
    while ContainsTerm('*?', spattern)  do begin
        spattern := stringreplace(spattern, '*?', '?*', [rfReplaceAll, rfIgnoreCase]);
    end;

    // eliminate redundant *'s
    while ContainsTerm('**', spattern) do begin
        spattern := stringreplace(spattern, '**','*',[rfReplaceAll, rfIgnoreCase]);
    end;


    // clean up last call's states
    // set a worst case size
    for i := low(states) to high(states) do begin
        if (states[i] <> nil) then begin
            states[i].Free;
        end;
    end;
    if (length(spattern) = 0) then begin
        setlength(states, 0);
        exit;
    end else begin
        setlength(states, length(spattern) + 1);
    end;
    //
    // create the state engine
    //
    //
    k := length(spattern);
    if k = 0 then exit;
    j := 0;
    FirstLiteral := -1;
    isSearch := false;

    for i := 1 to k do begin
        case spattern[i] of
            '?' : begin
                // move forward always
                States[j] := TState.Create;
                States[j].SetState('?', j + 1, j + 1);

                // last state
                if (i = k) then begin
                    States[j].SetState('?', EOF_STATE, EOF_STATE);
                end;
                j := j + 1;
            end;
            '*' : begin
                // fall back to the first literal after an asterisk
                isSearch := true;
            end;
            else begin
                // fall forward if literal matches, fall back to
                // first literal otherwise

                States[j] := TState.Create;
                // very first literal
                if (FirstLiteral = -1) then begin
                    FirstLiteral := j;
                end;

                // first literal after an asterisk
                // or must be perfect match
                if (isSearch) then begin
                    FirstLiteral := j;
                    isSearch := false;
                end;
                States[j].SetState(spattern[i], j + 1, FirstLiteral);

                // Last state
                if (i = k) then begin
                    States[j].SetMatchState(EOF_STATE);
                end;

                j := j + 1;
            end;
        end;
    end;

    // set the first starting state
    // create the final "good" state if expression ended in an asterisks
    // correct the length of the states array

    if (spattern = '*') then begin
        states[0] := TState.create;
        states[0].SetState('*',0,0);
        states[0].SetIsEnding(true); 
    end else begin
        States[0].SetIsStarting(true);

        if (spattern[k] = '*') then begin
            States[j] := TState.Create;
            States[j].SetState('*', j, j);
        end else begin
            j := j - 1;
        end;
        States[j].SetIsEnding(true);

    end;
    if (length(states) <> j + 1) then begin
        setlength(states, j + 1);
    end;
end;

function TWildcards.GetStr(startindex, endindex: cardinal): string;
begin
    if assigned(Proxy) then begin
        result := Proxy.GetStr(startindex, endindex);
        EXIT;
    end;

    result := midstr(self.scontent, startindex, endindex-startindex+1);
end;
function TWildcards.FindString: string;
var i, j : cardinal;
begin
    if assigned(Proxy) then begin
        result := Proxy.FindString;
        EXIT;
    end;

    result := '';
    if self.FindFirst(i,j) then
        result := copy(self.scontent,i,j-i+1);
end;
function TWildcards.Matches: boolean;
var startidx, endidx : cardinal;
begin
    if assigned(Proxy) then begin
        result := Proxy.Matches;
        EXIT;
    end;

    result := false;
    if FindFirst(startidx, endidx) then begin
        result := (startidx = 1) and (endidx=length(self.scontent));
    end;
end;
function TWildcards.FirstFound : Boolean;
var s,e : cardinal;
begin
    if assigned(Proxy) then begin
        result := proxy.FirstFound;
        EXIT;
    end;

    result := self.findfirst(s,e);
end;
function TWildcards.FindFirst(var startIndex : cardinal; var endIndex : cardinal) : boolean;
var i : integer;
begin
    if assigned(Proxy) then begin
        result := Proxy.FindFirst(startindex, endIndex);
        EXIT;
    end;


    startindex := 0;
    endindex := 0;
    if (self.isFilemode) then begin
        if (self.isExtensionless) then begin
            i := pos('.', scontent);

            if (i <> 0) and not (i = length(scontent)) then begin
                result := false;
                EXIT;
            end;
        end;

        if (self.isAlwaysTrue) then begin
            result := true;
            startIndex := 1;
            endIndex := length(scontent);
            EXIT;
        end;
    end;



    curIndex := 1;
    if spattern = '' then begin
        result := false;
    end else begin
        result := self.FindNext(startIndex, endIndex);
    end;
end;
function TWildcards.FindNext(var startIndex : cardinal; var endIndex : cardinal) : boolean;
begin
    if assigned(Proxy) then begin
        result := Proxy.FindNext(startIndex, endIndex);
        EXIT;
    end;


    if (self.isFilemode and self.isAlwaysTrue) then begin
        result := false;
        EXIT;
    end;

   result := GetMatch(startIndex, endIndex);
end;
function TWildcards.SearchReplaceWith(str : string; ReplaceAll : boolean) : string;
var i, j : cardinal;
    b : boolean;
    s : string;
begin
    if assigned(Proxy) then begin
        result := Proxy.SearchReplaceWith(str, ReplaceAll);
        EXIT;
    end;

    // ignore an empty pattern
    if (spattern = '') then begin
        result := scontent;
        exit;
    end;

    curIndex := 0;
    i := 0;
    j := 0;
    s := scontent;
    b := self.FindFirst(i,j);
    if (i = 1) and (j = length(scontent)) and b then begin
        // BUG:
        // I didn't make sure the FindFirst was a match for this case!

        // freaky case where pattern matches entire string and tries to
        // replace with a blank will infinitely loop
        result := str;
    end else begin
        while (b and ReplaceAll) do begin
            delete(scontent, i, j - i + 1);
            insert(str, scontent, i);
            curIndex := i + length(str);

            b := self.FindNext(i,j);
        end;
        if (b and not ReplaceAll) then begin
            delete(scontent, i, j - i + 1);
            insert(str, scontent, i);
        end;

        result := scontent;
    end;



    scontent := s;
    curIndex := 0;
end;
function TWildcards.SearchReplaceWith(s : string; sFind: string; sReplace : string; replaceall : boolean) : string;
begin
    if StrUtils.ContainsText(sFind, '?reg?') then begin
        sFind := UnitUtil.ReplaceAll(sFind, '?reg?', '');

        Proxy := TRegExpWildcards.Create;
        result := Proxy.SearchReplaceWith(s, sfind, sreplace, replaceall);
        EXIT;
    end;

    // special case, wildcard consumes entire source string
    if (sfind = '*') then begin
        result := sReplace;
    // special case, search string is empty
    end else if (sfind = '') then begin
        result := s;
    end else begin
        self.SetSearchPattern(sFind);
        self.SetSearchString(s);
        result := self.SearchReplaceWith(sReplace, replaceall);
    end;
end;


// private
function TWildcards.GetMatch(var startIndex : cardinal; var endIndex : cardinal) : boolean;
var NextState, LastState : integer;
    IsMatch, LastIsMatch : boolean;
    i, testi : integer;
    s : string;
begin
    if assigned(Proxy) then begin
        result := Proxy.getMatch(startIndex, endIndex);
        EXIT;
    end;


    NextState := 0; LastState := 0;
    IsMatch := false;
    LastIsMatch := false;
    StartIndex := 1; // a search term of nothing but metacharaters has no start index


    //
    // find (starting/ending)index
    // EOF_STATE means stop consuming characters (pattern doesn't end in an asterisk)
    s := lowercase(scontent);
    i := curIndex;
    while i <= length(scontent) do begin
    //for i := curIndex to length(scontent) do begin
        endIndex := i;

        IsMatch := states[nextstate].IsMatch(s[i]);
        // Account for X amound of question mark states before the
        // first literal and any asterisk and adjust starting index
        case states[nextstate].GetMatchChar of
            '*', '?':;
            else begin
                testi := self.GetFirstLiteralIndex;
                if (NextState = testi) and
                    (IsMatch) then begin
                    if (HasLeadingAsterisk) then begin
                        startIndex := curIndex;
                    end else begin
                        startIndex := (i - self.GetFirstLiteralIndex);
                    end;
                end;
            end;
        end;

        LastState := NextState;
        nextstate := states[nextstate].GetNextState(s[i]);

        if LastIsMatch and (not IsMatch) and (nextstate = self.GetFirstLiteralIndex) then begin
            // on a failure, allow the state machine to retest the current
            // character when the machine is reset to the first literal index
        end else begin
            inc(i);
        end;
        LastIsMatch := IsMatch;

        if (NextState = EOF_STATE) then begin
            break;
        end;
    end;

    //
    // we've reached the end of the string, or end of the state machine
    // generate final verdict
    // update index pointer for next search
    //
    result := IsMatch and states[LastState].GetIsEnding;
    if (result) then begin
        curIndex := endIndex + 1;
    end;
end;
function TWildcards.GetFirstLiteralIndex : integer;
var i : integer;
begin
    result := -1;

    for i := low(states) to high(states) do begin
        if not ContainsTerm(states[i].GetMatchChar, '?*') {pos(states[i].GetMatchChar, '?*') = 0} then begin
            result := i;
            break;
        end;
    end;
end;
function TWildcards.HasLeadingAsterisk : boolean;
var i, j : integer;
begin
    // find first asterisk and first literal in pattern (not state machine)
    i := pos('*', spattern);
    j := 1;
    while (j <= length(spattern)) and
        (ContainsTerm(spattern[j],'*?')) do
        inc(j);

    result := (i > 0) and (i < j);
end;




{ TRegExpWildcards }
//
//
//

procedure TRegExpWildcards.SetCaseSensitive(value : boolean);
begin
    caseSensitive := value;
end;
procedure TRegExpWildcards.AssignFilemode(value: boolean);
begin
    raise Exception.Create('Illegal use of Wildcard class.');
end;

constructor TRegExpWildcards.Create;
begin
    caseSensitive := true;
    //re := TRegEx.Create;
end;
destructor TRegExpWildcards.Destroy;
begin
  inherited;
 // re.Free;
end;

procedure TRegExpWildcards.SetSearchPattern(str: string);
begin
    if str='' then begin
        self.spattern := '';
        EXIT;
    end;

    self.spattern := str;

    if caseSensitive then begin
        re := TRegEx.Create(str, []);
    end else begin
        re := TRegEx.Create(str, [roIgnoreCase]);
    end;



    //re.Expression := spattern;
end;
procedure TRegExpWildcards.SetSearchString(str: string);
begin
    self.scontent := str;
end;


function TRegExpWildcards.FirstFound : Boolean;
var s,e : cardinal;
begin
    result := self.findfirst(s,e);
end;
function TRegExpWildcards.FindFirst(var startIndex, endIndex: cardinal): boolean;
var s : string;
begin
    result := false;
    if (spattern='') then EXIT;

    try
        match := re.Match(self.scontent);
        result := match.Success;

        //result := re.Exec(scontent);
        if result  then begin
//            startIndex := re.MatchPos[0];
//            endIndex := startIndex + re.MatchLen[0]-1;
            startIndex := match.Index;
            endIndex := startindex + match.Length - 1;
        end;
    except on e: exception do
        begin
            s := e.Message;
            result := false;
        end;


    end;
end;
function TRegExpWildcards.FindNext(var startIndex, endIndex: cardinal): boolean;
begin
//    result := re.ExecNext;
//    if result then begin
//        startIndex := re.MatchPos[0];
//        endIndex := startIndex + re.MatchLen[0]-1;
//    end;
    match := match.NextMatch;
    result := match.Success;
    if result then begin
        startIndex := match.Index;
        endIndex := startindex + match.Length - 1;
    end;
end;


function TRegExpWildcards.LastError: integer;
begin
//    result := re.LastError;
    result := 0;
end;

function TRegExpWildcards.SearchReplaceWith(s, sFind, sReplace: string; replaceall: boolean): string;
begin
    try
        self.SetSearchPattern(sfind);
        self.SetSearchString(s);

        result := self.SearchReplaceWith(sReplace, replaceall);
    except
        result := s;
    end
end;
function TRegExpWildcards.SearchReplaceWith(str: string; replaceall: boolean): string;
var i : integer;
begin
    try
        if self.spattern = '' then begin
            result := scontent;
        end else begin
            result := scontent;
            if re.IsMatch(scontent) then
                if replaceall then begin
                    result := re.Replace(scontent, str);
                end else begin
                    result := re.Replace(scontent, str, 1);
                end;
        end;
    except
        result := scontent;
    end;
end;


end.
